/* ccdv.c */
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#define SETCOLOR_SUCCESS	(gANSIEscapes ? "\033\1331;32m" : "")
#define SETCOLOR_FAILURE	(gANSIEscapes ? "\033\1331;31m" : "")
#define SETCOLOR_WARNING	(gANSIEscapes ? "\033\1331;33m" : "")
#define SETCOLOR_NORMAL		(gANSIEscapes ? "\033\1330;39m" : "")

#define TEXT_BLOCK_SIZE 8192
#define INDENT 2

#define CMD_INSTALL     1
#define CMD_INSTALL_DIR 2
#define CMD_MKNOD       3
#define CMD_LN          4
#define CMD_RM          5
#define CMD_STRIP       6
#define CMD_FILEPP      7
#define CMD_SHELLPP     8

size_t gNBufUsed = 0, gNBufAllocated = 0;
char *gBuf = NULL;
int gCCPID;
char gAction[512] = "";
char gTarget[512] = "";
char gSubTarget[40] = "";
char gLastTarget[512] = "";
int gTargets = 0;
char gAr[32] = "";
char gArLibraryTarget[512] = "";
int gDumpCmdArgs = 0;
char gArgsStr[800];
int gColumns = 80;
int gANSIEscapes = 0;
int gExitStatus = 95;

static void
DumpFormattedOutput(void)
{
	char *cp;
	char spaces[8 + 1] = "        ";
	char *saved;
	int curcol;
	int i;

	curcol = 0;
	saved = NULL;
	for (cp = gBuf + ((gDumpCmdArgs == 0) ? strlen(gArgsStr) : 0); ; cp++) {
		if (*cp == '\0') {
			if (saved != NULL) {
				cp = saved;
				saved = NULL;
			} else break;
		}
		if (*cp == '\r')
			continue;
		if (*cp == '\t') {
			saved = cp + 1;
			cp = spaces + 8 - (8 - ((curcol - INDENT - 1) % 8));
		}
		if (curcol == 0) {
			for (i = INDENT; --i >= 0; )
				putchar(' ');
			curcol = INDENT;
		}
		putchar(*cp);
		if (++curcol == (gColumns - 1)) {
			putchar('\n');
			curcol = 0;
		} else if (*cp == '\n')
			curcol = 0;
	}
	free(gBuf);
}	/* DumpFormattedOutput */



/* Difftime(), only for timeval structures.  */
static void TimeValSubtract(struct timeval *tdiff, struct timeval *t1, struct timeval *t0)
{
	tdiff->tv_sec = t1->tv_sec - t0->tv_sec;
	tdiff->tv_usec = t1->tv_usec - t0->tv_usec;
	if (tdiff->tv_usec < 0) {
		tdiff->tv_sec--;
		tdiff->tv_usec += 1000000;
	}
}	/* TimeValSubtract */



static void
Wait(void)
{
	int pid2, status;

	do {
		status = 0;
		pid2 = (int) waitpid(gCCPID, &status, 0);
	} while (((pid2 >= 0) && (! WIFEXITED(status))) || ((pid2 < 0) && (errno == EINTR)));
	if (WIFEXITED(status))
		gExitStatus = WEXITSTATUS(status);
}	/* Wait */



static int
SlurpProgress(int fd)
{
	char s1[512];
	char *newbuf;
	int nready;
	size_t ntoread;
	ssize_t nread;
	struct timeval now, tnext, tleft;
	fd_set ss;
	fd_set ss2;
	const char *trail = "/-\\|", *trailcp;
        int width = gColumns > sizeof s1+8 ? sizeof s1+8 : gColumns;

	trailcp = trail;
	snprintf(s1, width-8, "%s%s%s... ", gAction, gTarget[0] ? " " : "", gTarget);
	printf("\r%-*s%-8s", width-9, s1, "");
	fflush(stdout);

	gettimeofday(&now, NULL);
	tnext = now;
	tnext.tv_sec++;
	tleft.tv_sec = 1;
	tleft.tv_usec = 0;
	FD_ZERO(&ss2);
	FD_SET(fd, &ss2);
	for (;;) {
		if (gNBufUsed == (gNBufAllocated - 1)) {
			if ((newbuf = (char *) realloc(gBuf, gNBufAllocated + TEXT_BLOCK_SIZE)) == NULL) {
				perror("ccdv: realloc");
				return (-1);
			}
			gNBufAllocated += TEXT_BLOCK_SIZE;
			gBuf = newbuf;
		}
		for (;;) {
			ss = ss2;
			nready = select(fd + 1, &ss, NULL, NULL, &tleft);
			if (nready == 1)
				break;
			if (nready < 0) {
				if (errno != EINTR) {
					perror("ccdv: select");
					return (-1);
				}
				continue;
			}
			gettimeofday(&now, NULL);
			if ((now.tv_sec > tnext.tv_sec) || ((now.tv_sec == tnext.tv_sec) && (now.tv_usec >= tnext.tv_usec))) {
				tnext = now;
				tnext.tv_sec++;
				tleft.tv_sec = 1;
				tleft.tv_usec = 0;
				printf("\r%-*s%c%-5s", width-7, s1, *trailcp, "");
				fflush(stdout);
				if (*++trailcp == '\0')
					trailcp = trail;
			} else {
				TimeValSubtract(&tleft, &tnext, &now);
			}
		}
		ntoread = (gNBufAllocated - gNBufUsed - 1);
		nread = read(fd, gBuf + gNBufUsed, ntoread);
		if (nread < 0) {
			if (errno == EINTR)
				continue;
			perror("ccdv: read");
			return (-1);
		} else if (nread == 0) {
			break;
		}
		gNBufUsed += nread;
		gBuf[gNBufUsed] = '\0';
	}
	snprintf(s1, width-8, "%s%s%s: ", gAction, gTarget[0] ? " " : "", gTarget);
	Wait();
	if (gExitStatus == 0) {
		printf("\r%-*s", width-9, s1);
		printf(" [%s%s%s]", ((gNBufUsed - strlen(gArgsStr)) < 4) ? SETCOLOR_SUCCESS : SETCOLOR_WARNING, "OK", SETCOLOR_NORMAL);
		printf("%-3s\n", "");
	} else {
		printf("\r%-*s", width-9, s1);
		printf(" [%s%s%s]", SETCOLOR_FAILURE, "ERROR", SETCOLOR_NORMAL);
		printf("\n");
		gDumpCmdArgs = 1;	/* print cmd when there are errors */
	}
	fflush(stdout);
	return (0);
}	/* SlurpProgress */



static int
SlurpAll(int fd)
{
	char *newbuf;
	size_t ntoread;
	ssize_t nread;

	printf("%s%s%s.\n", gAction, gTarget[0] ? " " : "", gTarget);
	fflush(stdout);

	for (;;) {
		if (gNBufUsed == (gNBufAllocated - 1)) {
			if ((newbuf = (char *) realloc(gBuf, gNBufAllocated + TEXT_BLOCK_SIZE)) == NULL) {
				perror("ccdv: realloc");
				return (-1);
			}
			gNBufAllocated += TEXT_BLOCK_SIZE;
			gBuf = newbuf;
		}
		ntoread = (gNBufAllocated - gNBufUsed - 1);
		nread = read(fd, gBuf + gNBufUsed, ntoread);
		if (nread < 0) {
			if (errno == EINTR)
				continue;
			perror("ccdv: read");
			return (-1);
		} else if (nread == 0) {
			break;
		}
		gNBufUsed += nread;
		gBuf[gNBufUsed] = '\0';
	}
	Wait();
	gDumpCmdArgs = (gExitStatus != 0);	/* print cmd when there are errors */
	return (0);
}	/* SlurpAll */



static const char *
Unprefix(const char *path)
{
	const char *prefix;

	if (!(prefix = getenv("prefix")))
		return path;

	if (strncmp(prefix, path, strlen(prefix)) == 0)
		return path + strlen(prefix);

	return path;
}	/* Unprefix */


static const char *
Basename(const char *path)
{
	const char *cp;
	cp = strrchr(path, '/');
	if (cp == NULL)
		return (path);
	return (cp + 1);
}	/* Basename */


static const char *
Extension(const char *path)
{
	const char *cp = path;
	cp = strrchr(path, '.');
	if (cp == NULL)
		return ("");
	return (cp);
}	/* Extension */


static const char *
LastPart(const char *path)
{
	const char *cp = path;
	cp = strrchr(path, '-');
	if (cp == NULL)
		return ("");
	return (cp);
}	/* LastPart */


static void
Usage(void)
{
	fprintf(stderr, "Usage: ccdv /path/to/cc CFLAGS...\n\n");
	fprintf(stderr, "I wrote this to reduce the deluge Make output to make finding actual problems\n");
	fprintf(stderr, "easier.  It is intended to be invoked from Makefiles, like this.  Instead of:\n\n");
	fprintf(stderr, "\t.c.o:\n");
	fprintf(stderr, "\t\t$(CC) $(CFLAGS) $(DEFS) $(CPPFLAGS) $< -c\n");
	fprintf(stderr, "\nRewrite your rule so it looks like:\n\n");
	fprintf(stderr, "\t.c.o:\n");
	fprintf(stderr, "\t\t@ccdv $(CC) $(CFLAGS) $(DEFS) $(CPPFLAGS) $< -c\n\n");
	fprintf(stderr, "ccdv 1.0.0 is Free under the GNU Public License.  Enjoy!\n");
	fprintf(stderr, "  -- Mike Gleason, NcFTP Software <http://www.ncftp.com>\n");
	exit(96);
}	/* Usage */

int
main(int argc, char **argv)
{
	int pipe1[2];
	int devnull;
	char emerg[256];
	int fd;
	int nread;
	int i;
	int cc = 0;
	int as = 0;
	int depending = 0;
	int precompiling = 0;
	int cmd = 0;
	const char *quote;
	char *p;

	if (argc < 2)
		Usage();

	snprintf(gAction, sizeof(gAction), "Running %s", Basename(argv[1]));
	memset(gArgsStr, 0, sizeof(gArgsStr));
	for (i = 1; i < argc; i++) {
		quote = (strchr(argv[i], ' ') != NULL) ? "\"" : "";
		snprintf(gArgsStr + strlen(gArgsStr), sizeof(gArgsStr) - strlen(gArgsStr), "%s%s%s%s%s", (i == 1) ? "" : " ", quote, argv[i], quote, (i == (argc - 1)) ? "\n" : "");
	}

	for (i = 1; i < argc; i++) {
		if (i == 1) {
			const char* exe = Basename(argv[i]);

			if (strncmp(exe, "install", 7) == 0) {
				cmd = CMD_INSTALL;
			} else if (strcmp(exe, "mknod") == 0 ||
				   strcmp(exe, "mknod_elinux") == 0) {
				cmd = CMD_MKNOD;
				strcpy(gAction, "Creating device");
			} else if (strcmp(exe, "ln") == 0) {
				cmd = CMD_LN;
				strcpy(gAction, "Creating hard link");
			} else if (strcmp(exe, "rm") == 0) {
				cmd = CMD_RM;
				strcpy(gAction, "Removing");
			} else if (strcmp(exe, "strip") == 0 ||
				   strcmp(LastPart(exe), "-strip") == 0 ||
				   strcmp(exe, "strip-cris") == 0) {
				cmd = CMD_STRIP;
				strcpy(gAction, "Stripping");
			} else if (strcmp(exe, "filepp") == 0) {
				cmd = CMD_FILEPP;
				strcpy(gAction, "Preprocessing");
			} else if (strcmp(exe, "ar") == 0 ||
				   strcmp(LastPart(exe), "-ar") == 0 ||
				   strcmp(exe, "ar-cris") == 0) {
				snprintf(gAr, sizeof(gAr), "%s", exe);
			}
		} else if (!cmd &&
			   strcasecmp(Extension(argv[i]), ".spp") == 0) {
			cmd = CMD_SHELLPP;
			strcpy(gAction, "Preprocessing");
			snprintf(gTarget, sizeof(gTarget), "%s", Unprefix(argv[i]));
			gTarget[sizeof gTarget - 1] = '\0';
		} else if (!cmd &&
			   (strcmp(argv[i], "-o") == 0) && ((i + 1) < argc)) {
			if (strcasecmp(Extension(argv[i + 1]), ".o") != 0 &&
			    strcmp(Extension(argv[i + 1]), ".s") != 0 &&
			    strcmp(Extension(argv[i + 1]), ".i") != 0) {
				strcpy(gAction, "Linking");
				snprintf(gTarget, sizeof(gTarget), "%s", Basename(argv[i + 1]));
			}
		} else if (strchr("-+", (int) argv[i][0]) != NULL) {
			if (cmd) {
				if (cmd == CMD_INSTALL && argv[i][1] == 'd' && !argv[i][2]) {
					cmd = CMD_INSTALL_DIR;
				} else if ((cmd == CMD_INSTALL ||
					    cmd == CMD_INSTALL_DIR ||
					    cmd == CMD_MKNOD) &&
					   strchr("ogm", argv[i][1]) &&
					   !argv[i][2]) {
					i++;
				} else if (cmd == CMD_LN) {
					if ((argv[i][1] != '-' && strchr(&argv[i][1], 's')) ||
					    (argv[i][1] == '-' && strcmp(&argv[i][2], "symbolic") == 0)){
						strcpy(gAction, "Creating symbolic link");
					}
				} else if (cmd == CMD_STRIP &&
					   strchr("FIOKNRo", argv[i][1]) &&
					   !argv[i][2]) {
					if (argv[i][1] == 'o' && i+1 < argc) {
						snprintf(gLastTarget, sizeof(gLastTarget), "%s", Unprefix(argv[i+1]));
					}
					i++;
				} else if (cmd == CMD_FILEPP) {
					if (strchr("DImMo", argv[i][1]) &&
					    !argv[i][2]) {
						if (argv[i][1] == 'o' && i+1 < argc) {
							snprintf(gLastTarget, sizeof(gLastTarget), "%s", Unprefix(argv[i+1]));
						}
						i++;
					} else if (!strcmp(argv[i], "-imacros")) {
						i++;
					}
				}
			} else if (strcmp(argv[i], "-M") == 0 ||
				   strcmp(argv[i], "-MM") == 0) {
				depending++;
			} else if (strcmp(argv[i], "-E") == 0) {
				precompiling++;
			} else if (strncmp(argv[i], "-DL_", 4) == 0) {
				strncpy(gSubTarget, &argv[i][4], sizeof gSubTarget);
				gSubTarget[sizeof gSubTarget - 1] = '\0';
				if ((p = strchr(gSubTarget, '='))) {
					*p = '\0';
				}
			}
		} else if (cmd == CMD_INSTALL || cmd == CMD_INSTALL_DIR || cmd == CMD_LN) {
			if (*gLastTarget) {
				snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "%s ", Unprefix(gLastTarget));
			}
			snprintf(gLastTarget, sizeof(gLastTarget), argv[i]);
			gLastTarget[sizeof gLastTarget - 1] = '\0';
			gTargets++;
		} else if (cmd == CMD_RM || cmd == CMD_STRIP || cmd == CMD_FILEPP) {
			snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "%s ", Unprefix(argv[i]));
			gTargets++;
		} else if (cmd == CMD_MKNOD) {
			if (!*gTarget) {
				snprintf(gTarget, sizeof(gTarget), Unprefix(argv[i]));
				gTarget[sizeof gTarget - 1] = '\0';
			} else if (strlen(argv[i]) == 1) {
				switch (*argv[i]) {
				case 'c':
				case 'u':
					strcpy(gAction, "Creating character device");
					break;

				case 'b':
					strcpy(gAction, "Creating block device");
					break;

				case 'p':
					strcpy(gAction, "Creating pipe");
					break;

				default:
					break;
				}
			}
		} else if (strncasecmp(Extension(argv[i]), ".c", 2) == 0) {
			cc++;
			snprintf(gTarget, sizeof(gTarget), "%s", Basename(argv[i]));
		} else if (strcasecmp(Extension(argv[i]), ".S") == 0) {
			as++;
			snprintf(gTarget, sizeof(gTarget), "%s", Basename(argv[i]));
		} else if ((gArLibraryTarget[0] == '\0') && (strcasecmp(Extension(argv[i]), ".a") == 0)) {
			snprintf(gArLibraryTarget, sizeof(gArLibraryTarget), "%s", Basename(argv[i]));
		}
	}

	switch (cmd) {
	case CMD_INSTALL_DIR:
		if (strlen(gTarget))
			strcpy(gAction, "Creating directories");
		else
			strcpy(gAction, "Creating directory");
		snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "%s", Unprefix(gLastTarget));
		break;

	case CMD_INSTALL:
		strcpy(gAction, "Installing");
		snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "in %s", Unprefix(gLastTarget));
		break;

	case CMD_STRIP:
	case CMD_FILEPP:
		if (*gLastTarget) {
			snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "to %s ", Unprefix(gLastTarget));
		}
		/* Fall through */
	case CMD_RM:
		if (*gTarget) {
			gTarget[strlen(gTarget) - 1] = '\0';
		}
		break;

	case CMD_LN:
		if (gTargets > 2) {
			strcat(gAction, "s to");
			snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "in %s", Unprefix(gLastTarget));
		} else if (gTargets == 2) {
			snprintf(gAction + strlen(gAction), sizeof(gAction) - strlen(gAction), " %s to", Unprefix(gLastTarget));
			gTarget[strlen(gTarget) - 1] = '\0';
		} else {
			snprintf(gTarget, sizeof(gTarget), "%s to %s", Basename(gLastTarget), Unprefix(gLastTarget));
		}
		break;

	case CMD_MKNOD:
	case CMD_SHELLPP:
		break;

	default:
		if ((gAr[0] != '\0') && (gArLibraryTarget[0] != '\0')) {
			strcpy(gAction, "Creating library");
			snprintf(gTarget, sizeof(gTarget), "%s", gArLibraryTarget);
		} else if (cc > 0) {
			if (depending) {
				strcpy(gAction, "Generating dependencies for");
			} else {
				if (precompiling) {
					strcpy(gAction, "Preprocessing");
				} else {
					strcpy(gAction, "Compiling");
				}
				if (*gSubTarget) {
					snprintf(gTarget + strlen(gTarget), sizeof(gTarget) - strlen(gTarget), "(%s)", gSubTarget);
				}
			}
		} else if (as > 0) {
			strcpy(gAction, "Assembling");
		}
		break;
	}

	if (pipe(pipe1) < 0) {
		perror("ccdv: pipe");
		exit(97);
	}

	(void) close(0);
	devnull = open("/dev/null", O_RDWR, 00666);
	if ((devnull != 0) && (dup2(devnull, 0) == 0))
		close(devnull);

	gCCPID = (int) fork();
	if (gCCPID < 0) {
		(void) close(pipe1[0]);
		(void) close(pipe1[1]);
		perror("ccdv: fork");
		exit(98);
	} else if (gCCPID == 0) {
		/* Child */
		(void) close(pipe1[0]);		/* close read end */
		(void) dup2(pipe1[1], 2);	/* use write end on stderr */
		if (isatty(1) && pipe1[1] != 1) {	/* use write end on stdout */
			(void) dup2(pipe1[1], 1);
			(void) close(pipe1[1]);
		}
		execvp(argv[1], argv + 1);
		perror(argv[1]);
		exit(99);
	}

	/* parent */
	(void) close(pipe1[1]);		/* close write end */
	fd = pipe1[0];			/* use read end */

	if (!isatty(1)) {
		if (isatty(2)) {
			dup2(2, 1);
		}
		else {
			exit(97);
		}
	}

	gColumns = (getenv("COLUMNS") != NULL) ? atoi(getenv("COLUMNS")) : 80;
	gANSIEscapes = (getenv("TERM") != NULL) && (strstr("vt100:vt102:vt220:vt320:xterm:ansi:linux:scoterm:scoansi:dtterm:cons25:rxvt:Eterm", getenv("TERM")) != NULL);
	gBuf = (char *) malloc(TEXT_BLOCK_SIZE);
	if (gBuf == NULL)
		goto panic;
	gNBufUsed = 0;
	gNBufAllocated = TEXT_BLOCK_SIZE;
	if (strlen(gArgsStr) < (gNBufAllocated - 1)) {
		strcpy(gBuf, gArgsStr);
		gNBufUsed = strlen(gArgsStr);
	}

	if (isatty(1)) {
		if (SlurpProgress(fd) < 0)
			goto panic;
	} else {
		if (SlurpAll(fd) < 0)
			goto panic;
	}
	DumpFormattedOutput();
	exit(gExitStatus);

panic:
	gDumpCmdArgs = 1;	/* print cmd when there are errors */
	DumpFormattedOutput();
	while ((nread = read(fd, emerg, (size_t) sizeof(emerg))) > 0)
		(void) write(2, emerg, (size_t) nread);
	Wait();
	exit(gExitStatus);
}	/* main */
/* eof ccdv.c */
