/****************************************************************************
*!
*! FILE NAME  : cbl.h
*!
*! DESCRIPTION: cbl specific stuff
*!
*! ---------------------------------------------------------------------------
*! (C) Copyright 1998-2006, Axis Communications AB, LUND, SWEDEN
*!***************************************************************************/

#ifndef CBL_H
#define CBL_H

#include <compiler.h>
#include <reg_map.h>
#include <vcpu.h>
#include <dma.h>

#define ADDR_SEQ   0x3801ff00
#define ADDR_ID    0x3801ff04
#define ADDR_LEVEL 0x3801ff08

/* size of ethernet headers src,dst,length,crc */
#define ETHERNET_HDR_SIZE (6+6+2+4)

typedef struct packet_info_T {
  udword type;
  udword addr;
  udword size;
} packet_info_T;

typedef struct set_register_T {
  udword type;
  udword addr;
  udword val;
} set_register_T;

typedef struct br_T {
  udword type;
  udword baudrate;
} br_T;

typedef struct pause_loop_T {
  udword type;
  udword pause;
} pause_loop_T;

typedef struct get_register_T {
  udword type;
  udword addr;
} get_register_T;

typedef struct mem_verify_T {
  udword type;
  udword addr;
  udword val;
} mem_verify_T;

typedef struct mem_test_T {
  udword type;
  udword from;
  udword to;
} mem_test_T;

typedef struct mem_dump_T {
  udword type;
  udword from_addr;
  udword to_addr;
} mem_dump_T;

typedef struct mem_clear_T {
  udword type;
  udword from_addr;
  udword to_addr;
} mem_clear_T;

typedef struct flash_T {
  udword type;
  unsigned char *source;
  unsigned char *dest;
  udword size;
} flash_T;

typedef struct nand_flash_T {
  udword type;
  unsigned char *source;
  unsigned char *dest;
  unsigned char *maxdest;
  udword size;
} nand_flash_T;

typedef struct jump_T {
  udword type;
  udword addr;
} jump_T;

typedef struct bne_T {
  udword type;
  udword addr;
  udword target;
} bne_T;

#define NETWORK_HEADER_LENGTH sizeof(struct packet_header_T)

int   flash_write(const unsigned src, unsigned int dst, const unsigned int size);
byte  crc_correct(void);
int   read_data(void);
void  boot_crt0(void);
void  boot_crt1(void);
int   handle_network_read(void);

void  send_net_ack(void);
void  send_net_string(char *str);
void  start_net_trans(void);
void  send_net_hex(udword v, byte nl);

void  send_ser_ack(void);
void  send_serial_loop(void);
void  send_ser_string(char *str);
void  send_ser_hex(udword v, byte nl);
void  send_ser_uword(uword v);

void  next_level_boot(void);

void  ser_init(void);
void  net_init(unsigned boot_level);
void  parallel_init(void);
void  print_descr(dma_descr_data *d);
void set_level(void);

extern volatile udword  bytes_to_read;
extern volatile udword	target_address;

extern udword	nbr_read;

extern byte    interface;
extern byte    set_dest;
extern udword  last_timeout;

extern struct packet_header_T tx_header;
extern struct packet_header_T rx_header;
extern dma_descr_data volatile rx_descr;
extern dma_descr_data volatile rx_descr2;
extern dma_descr_data volatile tx_descr;

extern uword timeout_limit;  
extern udword seq;
extern byte serial_up;

extern int quiet;

//#define printf if (!quiet) printf
//#define fprintf if (!quiet) fprintf

extern uword HTONS(uword);
extern udword HTONL(udword);

#define NTOHL(x) HTONL(x)

#endif
