/*!***************************************************************************
*!
*! FILE NAME  : common_init.c
*!
*! DESCRIPTION: Used in level 1 and 2.
*!
*! ---------------------------------------------------------------------------
*! HISTORY
*!
*! DATE         NAME               CHANGES
*! ----         ----               -------
*! 19980326     Ronny Ranerup      Initial version for ETRAX 100
*! 2005         ronny              Ported/rewritten for ETRAX FS
*! ---------------------------------------------------------------------------
*! (C) Copyright 1998-2006, Axis Communications AB, LUND, SWEDEN
*!***************************************************************************/

/*

   Misc notes:

   It is very important to keep this file short. This and the boot
   interface specific parts must fit into the first (network) boot
   packet.

   */

/**************************  INCLUDES  *************************************/
#include <e100boot.h>
#include <cbl.h>
#include <timer_defs.h>

/**************************  CONSTANTS  ************************************/

/**************************  TYPE DEFINITIONS  *****************************/

/**************************  EXTERNALS  ************************************/

/**************************  EXTERNAL FUNCTIONS  ***************************/
static void start(void);
void crt1(void) asm ("crt1");
extern void level1_boot(void);
/**************************  LOCALS  ***************************************/
/**************************  GLOBALS  **************************************/
udword last_timeout;

 /* How many bytes has been read from current file */
udword nbr_read = 0;

/********************** FUNCTION DEFINITION SECTION *************************/
void 
crt1(void)
{
  /* Do this only once so we don't reset the timers and destroy the 32
     bit timer-register used as random number generator */
#if 0
{				/* for debugging, putc on ser0 */
#include <ser_defs.h>
#include <config_defs.h>
  {
    reg_config_rw_clk_ctrl rw_clk_ctrl = REG_RD(config, regi_config, rw_clk_ctrl);
    rw_clk_ctrl.fix_io = regk_config_yes;
    REG_WR(config, regi_config, rw_clk_ctrl, rw_clk_ctrl);
  }
  
  {
    reg_ser_rw_tr_baud_div rw_tr_baud_div = {0,};
    rw_tr_baud_div.div = 109; 
    REG_WR(ser, regi_ser0, rw_tr_baud_div, rw_tr_baud_div);
  }

  { /* set base_freq and enable */
    reg_ser_rw_tr_ctrl rw_tr_ctrl = REG_RD(ser, regi_ser0, rw_tr_ctrl);
    
    rw_tr_ctrl.base_freq = regk_ser_f100;
    rw_tr_ctrl.en    = regk_ser_yes;
    
    REG_WR(ser, regi_ser0, rw_tr_ctrl, rw_tr_ctrl);
  }
  

  //  while(1) {
    reg_ser_rs_stat_din rs_stat_din;
        reg_ser_rw_dout rw_dout = {'1'};
	//    reg_ser_rw_dout rw_dout;
    //    rw_dout.data = *(char*)0x38000806;
	//        rw_dout.data = *(char*)0x3800001e;
    
    do { 
      rs_stat_din = REG_RD(ser, regi_ser0, rs_stat_din);
    } while (rs_stat_din.tr_rdy == regk_ser_no);
    REG_WR(ser, regi_ser0, rw_dout, rw_dout);
   
    //  }
}
#endif

#if 1
 {
    /* set timer div */
    reg_timer_rw_tmr0_div r = 0x1000;
    REG_WR(timer, regi_timer, rw_tmr0_div, r);
  }

  {
    /* set clock frequency */
    reg_timer_rw_tmr0_ctrl r = REG_RD(timer, regi_timer, rw_tmr0_ctrl);
    r.freq = regk_timer_f100;
    r.op   = regk_timer_ld;
    REG_WR(timer, regi_timer, rw_tmr0_ctrl, r);
  }
  
  {
    /* enable interrupts */
    reg_timer_rw_intr_mask r = REG_RD(timer, regi_timer, rw_intr_mask);
    r.tmr0 = regk_timer_off;
    r.trig = regk_timer_off;
    r.cnt  = regk_timer_off;
    REG_WR(timer, regi_timer, rw_intr_mask, r);
  }
  
  {
    /* start timer */
    reg_timer_rw_tmr0_ctrl r = REG_RD(timer, regi_timer, rw_tmr0_ctrl);
    r.op   = regk_timer_run;
    REG_WR(timer, regi_timer, rw_tmr0_ctrl, r);
  }
  
  start();
#endif
}

static void
start(void)
{

  //  set_level();			/* initialize level counter */
  
  next_level_boot();		/* level 1 or level 2 */

}


/* return true if 0.1 second has passed */

int
timeout(void)
{
  volatile int now = REG_RD(timer, regi_timer, r_time);
  int elapsed;
  int wait_time = 10000000;

  elapsed = last_timeout - now;

  if (elapsed < 0) {
    elapsed = -elapsed;
  }

  if (elapsed > wait_time) {
    last_timeout = now;
    return(TRUE);
  }

  return(FALSE);

}

