/* Extra functionality in simlibc. */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <quick.h>
#include <reg_rdwr.h>
#include <reg_map.h>
#include <intr_vect_defs.h>
#include <dma_defs.h>
#include <dma.h>
#include <simdef.h>

#include <timer_defs.h>

unsigned short _Flags; 
extern int argc;
extern char**argv;

#define DISABLE_SAVE() \
 __asm__ volatile ("move ccs,%0\n\tdi" \
                                 : "=rm" (_Flags)    \
                                 : /* no inputs */   \
                                 : "memory")

#define RESTORE() \
 __asm__ volatile ("move %0,ccs" : /* no outputs */ : "rm" (_Flags) : "memory")

inline unsigned int
htonl(unsigned int x)
{
  __asm__ ("swapwb %0" : "=r" (x) : "0" (x));
  
  return(x);
}

inline unsigned short
htons(unsigned short x)
{
  __asm__ ("swapb %0" : "=r" (x) : "0" (x));
  
  return(x);
}

unsigned 
CPU_GET_SIM_TIME() {
  return (unsigned) REG_RD_INT( timer, regi_timer, r_time );
}

void 
CPU_ENABLE_INT_VEC(unsigned vecnr) 
{

  PREEMPT_OFF_SAVE();

  unsigned r = REG_RD_INT( intr_vect, regi_irq, rw_mask );

  r |= 1 << (vecnr-0x31);
  REG_WR_INT(intr_vect, regi_irq, rw_mask, r);

  PREEMPT_RESTORE();

}

void 
CPU_DISABLE_INT_VEC(unsigned vecnr) 
{

  PREEMPT_OFF_SAVE();

  unsigned r = REG_RD_INT( intr_vect, regi_irq, rw_mask );

  r &= ~(1 << (vecnr-0x31));
  REG_WR_INT(intr_vect, regi_irq, rw_mask, r);

  PREEMPT_RESTORE();
}

int 
CPU_IS_ACTIVE_INT_VEC(unsigned vecnr)
{
  unsigned r = REG_RD_INT( intr_vect, regi_irq, r_vect );
  return (r >> (vecnr-0x31)) & 1;
}

int 
CPU_IS_ENABLED_INT_VEC(unsigned vecnr)
{
  unsigned r = REG_RD_INT( intr_vect, regi_irq, rw_mask );
  return (r >> (vecnr-0x31)) & 1;

}

void 
CPU_KICK_DOG() 
{
}

void 
print_hex(unsigned int u)
{
  printf("%8.8x\n", u);
}

//typedef unsigned int shmem_handle;

int
cmp_shared( shmem_handle handle1, shmem_handle handle2, size_t size ) 
{
  printf("cmp_shared called!\n");
  exit(0);
}

int
cmp_offset_shared( shmem_handle handle1, unsigned offset1,
                   shmem_handle handle2, unsigned offset2, 
                   size_t size ) {
  printf("cmp_offset_shared called!\n");
  exit(0);
}


void 
free_shared( shmem_handle handle )
{
  printf("free_shared called!\n");
  exit(0);
}

void 
(free)(void *ptr)
{
  /* When using sos malloc is a syscall. We don't bother freeing that
     memory now. Should implement it though. */
}

void 
print_int(unsigned int u)
{
  printf("%d\n", u);
}

void 
print_uint(unsigned int u)
{
  printf("%u\n", u);

}

void 
print_mem_hex( void *ptr, size_t size )
{
  char *end = &((char*)ptr)[size];
  char *p = (char*)ptr;
  
  for( ; p<=end; p+=16) {
    unsigned i;
    printf("%8.8x: ", p);
    for(i=0 ; i!=16 && &p[i]<=end; i++) {
      printf("%2.2x\n", p[i]);
    }
  }
}

unsigned
random(void)
{
  return(rand());
}

void 
kick_dog()
{
  
}

void 
print_str(const char *c)
{
  int i;
  
  printf("%s", c);
}

void 
hook_debug(int t)
{

}

void
init_libc()
{
  
  //  stdin = stdout = freopen("/dev/ttyS0", "rw", stdout);
}

/* This sets up argv,argc and copies the data from sim memory to simulated memory. */

void
copy_argv(int *argc, char **argv)
{

}

void
print_packet(char *buf, int len)
{

}

void
print_packet_tr(char *buf, int len)
{

}

void
print_descr(char *buf)
{

}

void
print_buf(char *buf, int len)
{

}

void
print_ep_descr(char *buf)
{

}
print_ep_list(char *buf)
{

}

print_sb_descr(char *buf)
{

}

#ifndef USING_SOS
void
exit(int status)
{
  int i;

  _Flags = 0;

  /*  print_int(argc);*/
  
  stdin = stdout = stderr = NULL;
  
  while(1);

}

void 
preempt_off()
{
  /* empty when not using sos */
}
void 
preempt_on()
{
  /* empty when not using sos */
}

#endif

int 
gettimeofday(struct timeval *tv, struct timezone *tz)
{
  
  unsigned r_time = (unsigned) REG_RD_INT( timer, regi_timer, r_time );

  if (tv) {
    tv->tv_sec  = r_time/100000000UL;
    tv->tv_usec = r_time/100;
  }
  
  return(r_time/100);
}

int 
times(struct tms *tms)
{
}

int 
times2(struct tms *tms)
{
}


void
_iemem_print()
{

}

void
_iemem_printall()
{

}

void
CPU_SLEEP(unsigned a) 
{ 
    volatile unsigned i; 
    
    a/=32;

    for(i=0; i<=a; i++); 
}


#ifdef USING_SOS
// ----------------------------------------------------------- dma_start_simple
void *dma_start_simple( unsigned inst, void *data, unsigned size ) {
  unsigned  seed = 311;
  unsigned  csize;

  dma_descr_group   *grp;  
  dma_descr_context *ctx;
  dma_descr_data    *d;

  // Allocate a random cache line, and place group descr at a random offset
  grp = (dma_descr_group*) alloc_random_data( 32, 32, 5, &seed, &csize );
  grp = (dma_descr_group*) ((unsigned)grp + rand_range(0, 16));

  // Allocate a random cache line, and place context descr in it
  ctx = (dma_descr_context*) alloc_random_data( 32, 32, 5, &seed, &csize );
  // Only one possible location of 32-byte context in a cache line

  // Allocate a random cache line, and place data descr at a random offset
  d = (dma_descr_data*) alloc_random_data( 32, 32, 5, &seed, &csize );
  d = (dma_descr_data*) ((unsigned)d + rand_range(0, 16));

  // Init group descriptor
  //
  grp->next             = virt2phys(grp);
  grp->eol              = 1;
  grp->tol              = 1;
  grp->bol              = 1;
  grp->intr             = 0;
  grp->en               = 1;
  grp->dis              = 0;
  grp->md               = rand_range(0, 65535);
  grp->up               = (dma_descr_group*)(rand_range(0, 65535)<<8 
                                             | rand_range(0, 65535));
  grp->down.context     = virt2phys(ctx);
  
  // Init context descriptor
  //
  ctx->next             = virt2phys(ctx);
  ctx->eol              = 1;
  ctx->intr             = 0;
  ctx->store_mode       = 0;
  ctx->en               = 1;
  ctx->dis              = 0;
  ctx->md0              = rand_range(0, 65535);
  ctx->md1              = rand_range(0, 65535)<<8 | rand_range(0, 65535);
  ctx->md2              = rand_range(0, 65535)<<8 | rand_range(0, 65535);
  ctx->md3              = rand_range(0, 65535)<<8 | rand_range(0, 65535);
  ctx->md4              = rand_range(0, 65535)<<8 | rand_range(0, 65535);
  ctx->saved_data       = virt2phys(d);
  ctx->saved_data_buf   = (char*) data;

  // Init data descriptor
  //
  d->next    = virt2phys(d);
  d->buf     = (char*) data;
  d->eol     = 1;
  d->out_eop = 1;     // Ignored by input channel
  d->intr    = 1;
  d->wait    = 0;
  d->in_eop  = 0;     // Set by input channel, ignored by output channel
  d->md      = rand_range(0, 65535);
  d->after   = (char*)((unsigned)data + size);

  // start dma
  //
  DMA_ENABLE(inst);
  DMA_START_GROUP( inst, virt2phys(grp) );

  return( grp );
}

// ------------------------------------------------------- dma_start_simple_out
void *dma_start_simple_out( unsigned inst, void *data, unsigned size ) {
  return( dma_start_simple( inst, data, size ) );
}

// -------------------------------------------------------- dma_start_simple_in
int dma_start_simple_in( unsigned inst, void *data, unsigned size ) {

  reg_dma_rw_ack_intr    rw_ack_intr;
  reg_dma_rw_intr_mask   rw_intr_mask;
  reg_dma_r_intr         r_intr;
  reg_dma_rw_saved_data  rw_saved_data;   // NOT used: Start of data buffer
  reg_dma_rw_data_buf    rw_data_buf;     // NOT used: Current buffer pos

  dma_descr_group       *grp;
  dma_descr_context     *ctx;
  dma_descr_data        *d;

  DMA_ENABLE(inst);    // Enable DMA to be able to write to DMA registers

  // Clear current interrupts
  //
  rw_ack_intr.group       = 1;
  rw_ack_intr.ctxt        = 1;
  rw_ack_intr.data        = 1;
  rw_ack_intr.in_eop      = 1;
  rw_ack_intr.stream_cmd  = 1;
  REG_WR( dma, inst, rw_ack_intr, rw_ack_intr );

  // Disable all interrupts
  //
  rw_intr_mask.group      = 0;
  rw_intr_mask.ctxt       = 0;
  rw_intr_mask.data       = 0;
  rw_intr_mask.in_eop     = 0;
  rw_intr_mask.stream_cmd = 0;
  REG_WR( dma, inst, rw_intr_mask, rw_intr_mask );

  // Setup descriptors and start reception
  //
  grp = (dma_descr_group*) dma_start_simple( inst, data, size );
  ctx = phys2virt(grp->down.context);
  d   = phys2virt(ctx->saved_data);
  
  // Wait for all data by polling in_eop intr, 
  // if no data arrives watchdog will tell you!
  //
  do {
    r_intr = REG_RD(dma, inst, r_intr );
    CPU_SLEEP( 100 );
  } while( !r_intr.in_eop );

  return( d->in_eop ? (d->after - d->buf) : -1 );
}
#endif /* USING_SOS */

