#include <stdio.h>
#include <errno.h>
#include <dma_defs.h>
#include <ser_defs.h>
#include <eth_defs.h>
#include <dma.h>
#include <reg_map.h>
#include <vcpu.h>
#include <stddef.h>
#include <config_defs.h>
#include <strmux_defs.h>
#include <simdef.h>
#include "stdiostuff.h"

#define SET_CURRENT_CONTEXT( inst, context) \
  REG_WR( dma, inst, rw_group_down, context );

extern struct packet_header_T tx_header;

FILE ttyS0 = (FILE)NULL;
FILE ttyS1 = (FILE)NULL;
FILE ttyS2 = (FILE)NULL;
FILE ttyS3 = (FILE)NULL;
FILE eth0  = (FILE)NULL;

extern char __eth0_dma_buf[1024];

unsigned int eth0_dest_0   = 0;
unsigned short eth0_dest_1 = 0;
char freopen_first_time = 1;
unsigned r_network_tr_ctrl;

dma_descr_context volatile descr_context_tx __attribute__ ((aligned (32))) = {0,};
static dma_descr_data    volatile tx_descr         __attribute__ ((aligned (32))) = {0,};
dma_descr_data    volatile tx_descr2        __attribute__ ((aligned (32))) = {0,};

/* function prototypes */
static void init_dma0(void);
static void init_dma1(void);
static void init_eth_rec(void);
static void init_eth_tr(void);

void
init_ser(unsigned n)
{
  unsigned regi = regi_ser0;
  
  switch (n) {
  case 1:
    regi = regi_ser1;
    break;
    
  case 2:
    regi = regi_ser2;
    break;
    
  case 3:
    regi = regi_ser3;
    break;
  }
  
  {
    reg_config_rw_clk_ctrl rw_clk_ctrl = REG_RD(config, regi_config, rw_clk_ctrl);
    if (rw_clk_ctrl.fix_io == regk_config_no) { /* Start fixed i/o block clock region. */
      reg_config_rw_clk_ctrl rw_clk_ctrl = REG_RD(config, regi_config, rw_clk_ctrl);
      rw_clk_ctrl.fix_io = regk_config_yes;
      REG_WR(config, regi_config, rw_clk_ctrl, rw_clk_ctrl);
    }
  }
  { /* check if ser is already enabled, if so, return */
    reg_ser_rw_tr_ctrl rw_tr_ctrl = REG_RD(ser, regi, rw_tr_ctrl);
    
    if (rw_tr_ctrl.en == regk_ser_yes) {
      return;
    }
  }
  
  reg_ser_rw_rec_baud_div rw_rec_baud_div = {0,};
  rw_rec_baud_div.div = 109; 
  REG_WR(ser, regi, rw_rec_baud_div, rw_rec_baud_div);
  
  reg_ser_rw_tr_baud_div rw_tr_baud_div = {0,};
  rw_tr_baud_div.div = 109; 
  REG_WR(ser, regi, rw_tr_baud_div, rw_tr_baud_div);
  
  { /* set base_freq and enable */
    reg_ser_rw_tr_ctrl rw_tr_ctrl = REG_RD(ser, regi, rw_tr_ctrl);
    
    rw_tr_ctrl.base_freq = regk_ser_f100;
    rw_tr_ctrl.en    = regk_ser_yes;
    //    rw_tr_ctrl.auto_cts  = regk_ser_yes;
    //    rw_tr_ctrl.auto_rts  = regk_ser_yes;
    
    REG_WR(ser, regi, rw_tr_ctrl, rw_tr_ctrl);
  }
 
  if (0){
   volatile unsigned i;
      for(i=0;i!=10000000;i++);	/* Let the line go down. */
  }
}
 
FILE *
freopen(const char *fnam, const char *mode, FILE *stream)
{
  char serial_devices[]  = "/dev/ttyS";
  char network_devices[] = "/dev/eth";
  int i;
  unsigned fnamelen = 0;

  if (fnam) {
    fnamelen = strlen(fnam);
    if ((strncmp(serial_devices, fnam, strlen(serial_devices)) == 0) 
	&& (fnamelen > strlen(serial_devices))) {
      
      switch(fnam[fnamelen-1]) {
	
      case '0':
	if (!ttyS0) {
	  ttyS0 = (FILE)1;
	  init_ser(0);
	}
	stream = &ttyS0;
	return(stream);
	break;
	
      case '1':
	if (!ttyS1) {
	  ttyS1 = (FILE)1;
	  init_ser(1);
	}
	stream = &ttyS1;
	return(stream);
	break;
	
      case '2':
	if (!ttyS2) {
	  ttyS2 = (FILE)1;
	  init_ser(2);
	}
	stream = &ttyS2;
	return(stream);
	break;
	
      case '3':
	if (!ttyS3) {
	  ttyS3 = (FILE)1;
	  init_ser(3);
	}
	stream = &ttyS3;
	return(stream);
	break;
      }
    }
    
    else if ((strncmp(network_devices, fnam, strlen(network_devices)) == 0) 
	     && (fnamelen > strlen(network_devices))) {
      
      if (freopen_first_time) {
	//    eth0_dest_0 = (unsigned)*R_DMA_CH8_FIRST; /* get destination address */
	//    eth0_dest_1 = (unsigned short)*R_DMA_CH9_FIRST;
	freopen_first_time = 0;
      }
      
      switch(fnam[fnamelen-1]) {
      case '0':
	if (!eth0) {
	  eth0 = (FILE)1;
	  stream = &eth0;

	  PREEMPT_OFF_SAVE();

	  { /* start ethernet/dma clock region */
	    reg_config_rw_clk_ctrl rw_clk_ctrl = REG_RD(config, regi_config, rw_clk_ctrl);
	    if (rw_clk_ctrl.dma01_eth0 == regk_config_no) { /* Start fixed i/o block clock region. */
	      rw_clk_ctrl.dma01_eth0 = regk_config_yes;
	      REG_WR(config, regi_config, rw_clk_ctrl, rw_clk_ctrl);
	    }
	  }
	  
	  { /* enable dma0 in strmux */
	    reg_strmux_rw_cfg rw_cfg = REG_RD(strmux, regi_strmux, rw_cfg);
	    rw_cfg.dma0 = regk_strmux_eth0;
	    REG_WR(strmux, regi_strmux, rw_cfg, rw_cfg);
	  }
	  
	  reg_eth_rw_gen_ctrl rw_gen_ctrl = REG_RD(eth, regi_eth0, rw_gen_ctrl);
	  if (rw_gen_ctrl.en == regk_eth_yes) {
	    PREEMPT_RESTORE();
	    return (stream);
	  }
	  
	  { /* set up descriptors and packet header */
	    descr_context_tx.eol = regk_dma_yes;
	    descr_context_tx.en  = regk_dma_yes;
	    descr_context_tx.saved_data = (dma_descr_data*)virt2phys((unsigned)DESCR_CONTEXT_TX_SAVED_DATA);
	    descr_context_tx.saved_data_buf = (char*)virt2phys((unsigned)DESCR_CONTEXT_TX_SAVED_DATA_BUF);
	    /* pretend we have sent one packet already */
	    descr_context_tx.dis            = regk_dma_yes;
	    tx_descr.next    = (dma_descr_data*)(virt2phys((unsigned)&tx_descr2));
	    tx_descr.buf     = (char*)virt2phys((unsigned)&tx_header);
	    tx_descr.after   = (char*)virt2phys((unsigned)&tx_header+sizeof(struct packet_header_T));
	    
	    tx_descr2.buf     = (char*)virt2phys((unsigned)__eth0_dma_buf);
	    tx_descr2.eol     = regk_dma_yes;
	    tx_descr2.out_eop = regk_dma_yes;
	    tx_descr2.wait    = regk_dma_yes;
	    tx_descr2.after   = (char*)virt2phys((unsigned)(__eth0_dma_buf+TX_BUF_SIZE));
	    
	    *(unsigned*)&tx_header.src[0]        = htonl(0x02408c00); 
	    *(unsigned short*)&tx_header.src[4]  = htons(0x0100); 
	    *(unsigned*)&tx_header.dest[0]       = (unsigned int) eth0_dest_0;
	    *(unsigned short*)&tx_header.dest[4] = (unsigned short) eth0_dest_1;
	    tx_header.length = 0;
	    tx_header.snap1  = htonl(0xaaaa0300);
	    tx_header.snap2  = htonl(0x408c8856);
	    tx_header.tag    = htonl(0xfffffffe);
	    tx_header.seq    = 0;
	    tx_header.type   = 0;   /* string */
	    
	    {
	      /* uncache descriptors for fputc */
	      volatile unsigned foo    = *(unsigned*)(((unsigned)&descr_context_tx)+8192);
	      volatile unsigned bar    = *(unsigned*)(((unsigned)&tx_descr)+8192);
	      volatile unsigned foobar = *(unsigned*)(((unsigned)&tx_descr2)+8192);

	      volatile unsigned foo2   = *(unsigned*)(((unsigned)&descr_context_tx)+8192*2);
	      volatile unsigned bar2   = *(unsigned*)(((unsigned)&tx_descr)+8192*2);
	      volatile unsigned foobar2= *(unsigned*)(((unsigned)&tx_descr2)+8192*2);
	    }
	  }

	  init_dma1();
	  init_dma0();
	  init_eth_rec();
	  init_eth_tr();

	  { /* enable network */ 
	    reg_eth_rw_gen_ctrl rw_gen_ctrl = {
	      .loopback = regk_eth_no,
	      .protocol = regk_eth_ether,
	      .phy      = regk_eth_mii,
	      .en       = regk_eth_yes
	    };
	    REG_WR(eth, regi_eth0, rw_gen_ctrl, rw_gen_ctrl);
	  }
	  
	  /* Ethernet enabled, preempt on. Only one process will
	     execute below. */
	  PREEMPT_RESTORE();
	  return(stream);
	  break;
	}
      }
    }
  }
  return(NULL);
}

void
init_dma0(void)
{ 
  
  /* Reset dma0. */
  reg_dma_rw_cfg rw_cfg = REG_RD( dma, regi_dma0, rw_cfg );
  rw_cfg.en   = regk_dma_no; 
  rw_cfg.stop = regk_dma_no; 
  REG_WR( dma, regi_dma0, rw_cfg, rw_cfg);

#if 0
  /* restore descriptors */
  descr_context_tx.dis            = regk_dma_no;
  descr_context_tx.saved_data     = 
    (dma_descr_data*)virt2phys((unsigned)DESCR_CONTEXT_TX_SAVED_DATA);
  descr_context_tx.saved_data_buf = 
    (char*)virt2phys((unsigned)DESCR_CONTEXT_TX_SAVED_DATA_BUF);
#endif
  /* enable dma0 */
  rw_cfg.en = regk_dma_yes; 
  REG_WR( dma, regi_dma0, rw_cfg, rw_cfg);
  
  /* set context descriptor */
  SET_CURRENT_CONTEXT(regi_dma0, virt2phys((unsigned)&descr_context_tx));
  
  /* load context descriptor */
  DMA_WR_CMD( regi_dma0, regk_dma_load_c );
  
}

void
init_dma1(void)
{ 
  
  /* Reset dma1. */
  reg_dma_rw_cfg rw_cfg = REG_RD( dma, regi_dma1, rw_cfg );
  rw_cfg.en   = regk_dma_no; 
  rw_cfg.stop = regk_dma_no; 
  REG_WR( dma, regi_dma1, rw_cfg, rw_cfg);

  /* enable dma1 */
  rw_cfg.en = regk_dma_yes; 
  REG_WR( dma, regi_dma1, rw_cfg, rw_cfg);
  
}


void 
init_eth_rec(void)
{
  {
    reg_eth_rw_rec_ctrl rw_rec_ctrl = {
      .max_size   = regk_eth_size1518,
      .duplex     = regk_eth_half,
      .bad_crc    = regk_eth_discard,
      .oversize   = regk_eth_discard,
      .broadcast  = regk_eth_discard,
      .individual = regk_eth_no,
      .ma0        = regk_eth_yes,
      .ma1        = regk_eth_no
    };
    
    { /* full duplex selected on boot pins? */
      reg_config_r_bootsel r_bootsel = REG_RD(config, regi_config, r_bootsel);
      
      if (r_bootsel.full_duplex == regk_config_yes) {
	rw_rec_ctrl.duplex = regk_eth_full;
      }
    
      REG_WR(eth, regi_eth0, rw_rec_ctrl, rw_rec_ctrl);
    }
    
    { /* address */
      { /* set lower part of station address register */
	reg_eth_rw_ma0_lo rw_ma0_lo = {
	  .addr = NET_GA_LO_32
	};
	REG_WR(eth, regi_eth0, rw_ma0_lo, rw_ma0_lo);
      }
      
      { /* set higher part of station address register */
	reg_eth_rw_ma0_hi rw_ma0_hi = {
	  .addr = NET_GA_HI_16
	};
	REG_WR(eth, regi_eth0, rw_ma0_hi, rw_ma0_hi);
      }
    }
  }
}

void 
init_eth_tr(void) 
{
  { /* initialize transmitter */
    reg_eth_rw_tr_ctrl rw_tr_ctrl = {
      .crc        = regk_eth_yes,
      .pad        = regk_eth_yes,
      .retry      = regk_eth_yes,
      .ignore_col = regk_eth_no,
      .cancel     = regk_eth_no,
      .hsh_delay = regk_eth_no
    };
    
    REG_WR(eth, regi_eth0, rw_tr_ctrl, rw_tr_ctrl);
  }
}  
  
