#include <syscall.h>
#include <reg_map.h>
#include <intr_vect_defs.h>
#include <reg_rdwr.h>
#include <xalloc.h>
#include <simdef.h>

/*----------------------------------------------------------------------*/
/* function prototypes */

static void di(void);
static void ei(void);

/*----------------------------------------------------------------------*/
/* 
   
   This file is compiled into libc/processes and not sos.               
   make install in libc to compile

*/

#define EI() __asm__ volatile ("break 1")
#define DI() __asm__ volatile ("break 2")

#define SYSCALL(p) __asm__ volatile \
("\
	move.d %0, $r0\n\
	break 0\n\
" \
 : /* outputs */ \
 : "i" (p) \
 : "$r0");

static reg_intr_vect_rw_mask old_rw_mask;

void 
CPU_STOP_SIM(void) 
{ 
  exit(EXIT_FAILURE); 
}

void 
CPU_WATCHDOG_TIMEOUT(unsigned u)
{
  /* we let sos handle this for now */
}

void *
alloc_random_data(unsigned min_size,
		  unsigned max_size,
		  unsigned align,
		  unsigned *seed,
		  unsigned *csize ) {
  SYSCALL(SYSCALL_ALLOC_RANDOM_DATA);
}

void *
malloc(size_t size)
{
  unsigned ret_size;

  return(alloc_random_data(size, size+32, 1, (unsigned*)&_Randseed, &ret_size));
}

int 
getpid() 
{

  SYSCALL(SYSCALL_GETPID);
  
}

/* This is called automagically at the end of all process interrupt
   handlers. */
void 
proc_intr_epilogue()
{
  yield();
}

/* This function is just so that we get epilogue address to sos. */
void CPU_SET_INT_VECT_2(unsigned vecnr, unsigned addr, unsigned epilogue)
{
  SYSCALL(SYSCALL_SET_INTR_VECT);
}

void CPU_SET_INT_VEC(unsigned vecnr, unsigned addr)
{
  CPU_SET_INT_VECT_2(vecnr, addr, (unsigned)&proc_intr_epilogue);
}

void
yield()
{

  SYSCALL(SYSCALL_YIELD);

}

unsigned
virt2phys(unsigned addr)
{

  SYSCALL(SYSCALL_VIRT2PHYS);

}

unsigned
phys2virt(unsigned addr)
{

  SYSCALL(SYSCALL_PHYS2VIRT);

}

void
exit(int status)
{

  preempt_on();
  SYSCALL(SYSCALL_EXIT);
  while(1);			/* just to shut up gcc */
}

void 
preempt_off()
{


  DI();   /* call sos routine to disable interrupts */

}

void 
preempt_on()
{

  EI();  /* call sos routine to enable interrupts */

}

int 
preempt_is_off()
{
  unsigned ccs;

  __asm__ volatile ("move $ccs, %0" : "=r" (ccs));
    
  return(!(ccs&0x20));
}

shmem_handle 
alloc_shared( unsigned size ) {
  printf("alloc_shared called!\n");
  exit(0);
}

void *
ptr_shared( shmem_handle handle ) {
  printf("ptr_shared called!\n");
  exit(0);
}

void 
fill_random_shared( shmem_handle h, unsigned offset, unsigned size, 
                        unsigned *seed ) {
  printf("fill_random_shared called!\n");
  exit(0);
}

shmem_handle 
file2shared( char *filename, unsigned *size ) {
  printf("file2shared called!\n");
  exit(0);
}

int 
rand_range( int min, int max ) {
  return ((rand()%(max-min+1)) + min);
}

void 
print_descr_context(dma_descr_context *descr) 
{
  printf("print_context_descr: 0x%8.8x\n", (unsigned)descr);
  printf("  next           : 0x%8.8x\n", ((unsigned*)descr)[0]);
  printf("  eol            : %d\n", (((unsigned*)descr)[1]>>0)&1);
  printf("  intr           : %d\n", (((unsigned*)descr)[1]>>4)&1);
  printf("  store_mode     : %d\n", (((unsigned*)descr)[1]>>6)&1);
  printf("  en             : %d\n", (((unsigned*)descr)[1]>>7)&1);
  printf("  dis            : %d\n", (((unsigned*)descr)[1]>>15)&1);
  printf("  md0            : 0x%4.4x\n", ((unsigned*)descr)[1]>>16);
  printf("  md1            : 0x%8.8x\n", ((unsigned*)descr)[2]);
  printf("  md2            : 0x%8.8x\n", ((unsigned*)descr)[3]);
  printf("  md3            : 0x%8.8x\n", ((unsigned*)descr)[4]);
  printf("  md4            : 0x%8.8x\n", ((unsigned*)descr)[5]);
  printf("  saved_data     : 0x%8.8x\n", ((unsigned*)descr)[6]);
  printf("  saved_data_buf : 0x%8.8x\n", ((unsigned*)descr)[7]);

}

void 
print_descr_data(dma_descr_data *descr) 
{
  printf("print_data_descr: 0x%8.8x\n", ((unsigned*)descr));
  printf("  next           : 0x%8.8x\n", ((unsigned*)descr)[0]);
  printf("  buf            : 0x%8.8x\n", ((unsigned*)descr)[1]);
  printf("  eol            : %d\n",     (((unsigned*)descr)[2]>>0)&1);
  printf("  out_eop        : %d\n",     (((unsigned*)descr)[2]>>3)&1);
  printf("  intr           : %d\n",     (((unsigned*)descr)[2]>>4)&1);
  printf("  wait           : %d\n",     (((unsigned*)descr)[2]>>5)&1);
  printf("  in_eop         : %d\n",     (((unsigned*)descr)[2]>>11)&1);

  printf("  md             : 0x%4.4x\n",  ((unsigned*)descr)[2]>>16);
  printf("  after          : 0x%8.8x\n",  ((unsigned*)descr)[3]);

}

void 
print_descr_group(dma_descr_group *descr) 
{

  printf("print_group_descr: 0x%8.8x\n", ((unsigned*)descr));
  printf("  next           : 0x%8.8x\n", ((unsigned*)descr)[0]);
  printf("  eol            : %d\n",     (((unsigned*)descr)[1]>>0)&1);
  printf("  tol            : %d\n",     (((unsigned*)descr)[1]>>1)&1);
  printf("  bol            : %d\n",     (((unsigned*)descr)[1]>>2)&1);
  printf("  intr           : %d\n",     (((unsigned*)descr)[1]>>4)&1);
  printf("  en             : %d\n",     (((unsigned*)descr)[1]>>7)&1);
  printf("  dis            : %d\n",     (((unsigned*)descr)[1]>>15)&1);

  printf("  md             : 0x%4.4x\n",  ((unsigned*)descr)[1]>>16);
  printf("  up             : 0x%8.8x\n",  ((unsigned*)descr)[2]);
  printf("  down           : 0x%8.8x\n",  ((unsigned*)descr)[3]);

}

