//==========================================================================
//
//      rflflash.h
//
//      Flash programming used in Axis RFL
//
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    hendrikr@axis.com
// Contributors: hendrikr@axis.com
// Date:         2002-05-31
// Purpose:      
// Description:  
//              
// This code is part of RedBoot (tm).
//
// The flash code is synchronized with the client part flash.c v. 1.26.
// A design based on a mirrored address space with one address bank
// base address and a number of unit sizes would be desirable. However,
// time does not permit a re-design.
//
//####DESCRIPTIONEND####
//
//==========================================================================

#define AXISRFL_NANDFLASH 1

#ifndef _RFLFLASH_H_
#define _RFLFLASH_H_

/*
 * Error codes returned as bit flags.
 */
extern unsigned long errno_flash;

enum
  {
    ERROR_RESET = 0x00000000,
    ERROR_UNDEFINED_BUSWIDTH = 0x00000001,
    ERROR_UNDEFINED_INTERLEAVE = 0x00000002,
    ERROR_ERASE_FAILED = 0x00000004,
    ERROR_BUSY_WAIT_TIME_OUT = 0x00000008,
    ERROR_WRITE_VERIFICATION_FAILED = 0x00000010,
    ERROR_NOT_ENOUGH_MEMORY = 0x00000020,
    ERROR_TOO_MANY_REGIONS = 0x00000040,
    ERROR_NO_CHIP_FOUND = 0x00000080,
    ERROR_READ_FAILED = 0x00000100,
    ERROR_WRITE_FAILED = 0x00000200,

    INFO_NOT_VALID = 0x00010000,
    INFO_NO_HWID = 0x00020000,
    INFO_NO_IP = 0x00040000,
    INFO_NO_MAC = 0x00080000
  };

/*
 * The address space used for flash memories.
 */
#define FLASH_ADDRESS_BASES 2

/*
 * These values must represent the number of bytes needed to meet the 
 * device type (x8, x16, x32).  Eg. a 32 bit device is 4 x 8 bytes. 
 * These numbers are used in calculations.
 */
#define TYPE_X16 (16 / 8)

/*
 * The interleave and bus width choices. Sort of: 16 or 32 bit bus width.
 */
#define INTERLEAVE_BASE 1
#define BUSWIDTH_BASE 2

/*
 * Used for time critical operations.
 */
#define nop() __asm__("nop")

/*
 * Retry attempts on failure.
 */
#define RETRY_VERIFY_WRITE 10

/*
 * Retry to read the toggle flag.
 */
#define RETRY_BUSY_TOGGLE_FLAG 0x0fffffff

enum
  {
    /* Addresses */
    ADDR_UNLOCK_1			= 0x0555,
    ADDR_UNLOCK_2			= 0x02AA,
    ADDR_MANUFACTURER			= 0x0000,
    ADDR_DEVICE_ID			= 0x0001,
    ADDR_CFI_QUERY			= 0x0055,

    /* Commands */
    CMD_UNLOCK_DATA_1			= 0x00AA,
    CMD_UNLOCK_DATA_2			= 0x0055,
    CMD_MANUFACTURER_UNLOCK_DATA	= 0x0090,
    CMD_PROGRAM_UNLOCK_DATA		= 0x00A0,
    CMD_RESET_DATA			= 0x00F0,
    CMD_SECTOR_ERASE_UNLOCK_DATA_1	= 0x0080,
    CMD_SECTOR_ERASE_UNLOCK_DATA_2	= 0x0030,
    CMD_CFI_QUERY_DATA			= 0x0098,

    /* Offsets */
    OFFSET_CFI_ID			= 0x10,
    OFFSET_CFI_SIZE			= 0x27,
    OFFSET_CFI_BLOCK_COUNT		= 0x2C,
    OFFSET_CFI_BLOCK			= 0x2D,

    /* Manufacturers */
    MANUFACTURER_AMD			= 0x01,
    MANUFACTURER_ATMEL  		= 0x1F,
    MANUFACTURER_FUJITSU		= 0x04,
    MANUFACTURER_SST			= 0xBF,
    MANUFACTURER_ST			= 0x20,
    MANUFACTURER_TOSHIBA		= 0x98,

    /* AMD devices */
    AM29F800BB				= 0x00012258,
    AM29F800BT				= 0x000122D6,
    AM29LV800BB				= 0x0001225B,
    AM29LV800BT				= 0x000122DA,
    AM29LV160BB				= 0x00012249,
    AM29LV160BT				= 0x000122C4,


    /* Atmel devices */
    AT49xV16x   			= 0x001F00C0,
    AT49xV16xT  			= 0x001F00C2,
    AT49BV32xAT  			= 0x001F00C9,

    /* Fujitsu devices */
    MBM29LV160TE			= 0x000422C4,
    MBM29LV160BE			= 0x00042249,

    /* SST devices */
    SST39LF800				= 0x00BF2781,
    SST39LF160				= 0x00BF2782,

    /* ST devices */
    /* Used in 5600, similar to AM29LV800, but no unlock bypass. */
    M29W800T				= 0x002000D7,

    /* Toshiba devices */
    TC58FVT160				= 0x009800C2,
    TC58FVB160				= 0x00980043,
    TC58FVT800				= 0x0098004F,

    /* Toggle bit mask */
    D6_MASK				= 0x40
  };

/*
 * The maximum number of supported different regions (blocks) on a chip.
 */
#define MAX_REGIONS 16

/*
 * Each region (block) starts at an offset in a bank. The sector size is
 * fixed within the region and there is a fixed number of sectors. Thus, the
 * size of the region is the sector size times the number of sectors.
 */
struct region
{
  unsigned long offset;
  unsigned int sector_size;
  unsigned int numsectors;
};

/*
 * The base address of a chip is the address of the correspondent bank where
 * it is mounted. Two chips may be interleaved which increases the bus width,
 * typically from 16 to 32 bits. The total size of the chip is either obtained
 * by CFI probing or hardcoded for chips which do not support CFI. Thus, the
 * number of regions and the number of fixed size sectors within each region
 * determines the size of the flash. However, all regions can not necessarily
 * be used due to the layout.
 */
struct chip
{
  volatile unsigned char *base;
  unsigned char interleave;
  unsigned char buswidth;
  unsigned int size;
  unsigned short numregions;
  struct region regions[MAX_REGIONS];
};

extern unsigned long
rflflash_init (void);

extern unsigned long
rflflash_read (unsigned long offset, unsigned char *datum, 
	       unsigned long size, unsigned long *readlen);

extern unsigned long
rflflash_checksum (unsigned long offset, unsigned long size, 
		   unsigned long *ret_csum);

#ifdef AXISRFL_NANDFLASH
extern unsigned long
rflflash_write (unsigned char *source, unsigned long offset, 
		unsigned long end_offset, unsigned long size);
#else
extern unsigned long
rflflash_write (unsigned char *source, unsigned long offset, 
		unsigned long size);
#endif

#ifdef AXISRFL_NANDFLASH
extern unsigned long
rflflash_nand_pagesize(void);

extern unsigned long
rflflash_nand_size(void);
#endif

#endif // _RFLFLASH_H_
