#include <vcpu.h>
#include <reg_map.h>
#include <gio_defs.h>
#include <bif_core_defs.h>
#include <pinmux_defs.h>
//#include "system.h"
//#include <simdef.ah>

#define SET(x) {\
  int a = REG_RD_INT(gio, regi_gio, rw_pa_dout);\
  a |= (1 << x); \
  REG_WR_INT(gio, regi_gio, rw_pa_dout, a);\
}

#define CLR(x) {\
  int a = REG_RD_INT(gio, regi_gio, rw_pa_dout);\
  a &= ~(1 << x); \
  REG_WR_INT(gio, regi_gio, rw_pa_dout, a);\
}

#define BUSY \
  !(REG_RD_INT(gio, regi_gio, r_pa_din)  & (1 << BY))

#define WAIT(a) \
  { volatile unsigned i; for(i=0; i!=a; i++); }

// GPIO pins connected to NAND flash
#define CE 4
#define CLE 5
#define ALE 6
#define BY 7

#define NAND_RD_ADDR 0x90000000
#define NAND_WR_ADDR 0x94000000

unsigned rd_cycles  = 0;
unsigned cmd_rd_end = 0;

unsigned char buffer[1048];

/* port pa toggling program, position independent, from led.S */
unsigned char test_prog[1048] =
{
  0x6f, 0x0e, 0x00, 0x80, 0x03, 0xb0, 0x6f, 0x1e ,
  0xff, 0x00, 0x00, 0x00, 0xe0, 0x1b, 0x6f, 0x0e ,
  0x08, 0xa0, 0x01, 0xb0, 0x6f, 0x1e, 0xff, 0x00 ,
  0x00, 0x00, 0xe0, 0x1b, 0x6f, 0x2e, 0x00, 0x00 ,
  0x00, 0x00, 0x6f, 0x3e, 0xff, 0x00, 0x00, 0x00 ,
  0x6f, 0x0e, 0x00, 0xa0, 0x01, 0xb0, 0xe0, 0x2b ,
  0xe0, 0x3b, 0xf7, 0xe0, 0xb0, 0x05, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,

  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ,

};

unsigned char empty[1048] = {
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
};

void 
read_status(void)
{

  __asm__ volatile ("nop\n\tnop\n\tnop\n\t");
  /* read status */
  SET(CLE);
  SET(CLE);
  
  *(volatile long*)NAND_WR_ADDR = 0x70;	/* read status */
  //  delay(100);
  
  WAIT(10);
  
  CLR(CLE);
  CLR(CLE);
  
  unsigned char stat = *(volatile unsigned char*)NAND_RD_ADDR;
  printf("status: 0x%x\n", stat);
}

void 
erase(void)
{
  printf(">>erase\n");

  CLR(CE);
  CLR(ALE);

  {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0x60;	/* block erase */
    
    WAIT(10);
    
    CLR(CLE);
    CLR(CLE);
  }

  {
    SET(ALE);
    SET(ALE);
    
    *(volatile long*)NAND_WR_ADDR = 0x00;
    *(volatile long*)NAND_WR_ADDR = 0x00;
    *(volatile long*)NAND_WR_ADDR = 0x00;
    
    if (rd_cycles >= 4) {
      *(volatile long*)NAND_WR_ADDR = 0x00;
    }
    
    if (rd_cycles >= 5) {
      *(volatile long*)NAND_WR_ADDR = 0x00;
    }
    
    WAIT(10);
    
    CLR(ALE);
    CLR(ALE);
  }
  
  {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0xd0;	/* confirm erase */
    
    WAIT(10);
    
    CLR(CLE);
    CLR(CLE);
  }
  
  while (BUSY) {
    read_status();
  }
  read_status();

  SET(CE);
  SET(CE);

  printf("<<erase\n");
}

void 
program(unsigned char* data, int addr)
{
  int i;

  printf(">>program\n");

  CLR(CE);
  CLR(CE);

  CLR(ALE);
  CLR(ALE);

  {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0x80;	/* cmd */
    
    WAIT(10);
    
    CLR(CLE);
    CLR(CLE);
  }
  
  {
    SET(ALE);
    SET(ALE);
    
    *(volatile long*)NAND_WR_ADDR = addr&0xff; //0x00;
    *(volatile long*)NAND_WR_ADDR = (addr>>8)&0x0f; //0x00;
    *(volatile long*)NAND_WR_ADDR = (addr>>12)&0xff; //0x00;
    
    if (rd_cycles >= 4) {
      *(volatile long*)NAND_WR_ADDR = (addr>>20)&0xff; //0x00;
    }

    if (rd_cycles >= 5) {
      *(volatile long*)NAND_WR_ADDR = (addr>>28)&0xff; //0x00;
    }
    /* fifth cycle */
    //  *(volatile long*)NAND_WR_ADDR = 0x00;
    
    WAIT(10);
    
    CLR(ALE);
    CLR(ALE);
  }
  
  for (i = 0; i < 128; i++) {    
    *(volatile long*)NAND_WR_ADDR = data[i];
    WAIT(100);
  }

  {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0x10;	/* confirm */
    //  delay(100);
    
    WAIT(10);
    
    CLR(CLE);
    CLR(CLE);
  }

  while(BUSY) {
    read_status();
  }
  read_status();

  SET(CE);
  SET(CE);

  printf("<<program\n");
}

void 
read(unsigned char *data)
{
  int i;

  printf(">read\n");

  CLR(CE);
  CLR(CE);

  CLR(ALE);
  CLR(ALE);

  {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0x00;	/* read command */
    //  delay(100);
    WAIT(10);
    CLR(CLE);
    CLR(CLE);
  }
  
  {
    SET(ALE);
    SET(ALE);
    *(volatile long*)NAND_WR_ADDR = 0x00;
    *(volatile long*)NAND_WR_ADDR = 0x00;
    *(volatile long*)NAND_WR_ADDR = 0x00;	/* why was this 1? */
    if (rd_cycles >= 4) {
      *(volatile long*)NAND_WR_ADDR = 0x00;
    }
    if (rd_cycles >= 5) {
      *(volatile long*)NAND_WR_ADDR = 0x00;
    }
    WAIT(10);
    CLR(ALE);
    CLR(ALE);
  }

  if (cmd_rd_end) {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0x30;
    
    WAIT(10);
    CLR(CLE);
    CLR(CLE);
    WAIT(10);
  }
  //  delay(100);

  while(BUSY);

  for (i = 0; i < 128; i++) {
    unsigned char val;
    val = *(volatile unsigned char*)NAND_RD_ADDR;
    printf("%d %x\n", i, val);
    if (val != data[i])
      //printl("Data error. Got %X expected %X\n", val, data[i]);
      printf("Data error at %d. Got 0x%02x expected 0x%02x\n" 
	     ,i, val, data[i]);
  }

  SET(CE);
  SET(CE);

  printf("<read\n");
}

unsigned
read_sig()
{

  printf(">read_sig\n");

  CLR(CE);
  CLR(CE);
  CLR(ALE);
  CLR(ALE);

  {
    SET(CLE);
    SET(CLE);
    
    *(volatile long*)NAND_WR_ADDR = 0x90;	/* read command */
    
    WAIT(10);
    
    CLR(CLE);
    CLR(CLE);
  }
  
  {
    SET(ALE);
    SET(ALE);
    
    *(volatile long*)NAND_WR_ADDR = 0x00;	/* address */
    
    WAIT(10);
    
    CLR(ALE);
    CLR(ALE);
  }
  
  WAIT(10);

  volatile unsigned char man = *(volatile unsigned char*)NAND_RD_ADDR;
  volatile unsigned char dev = *(volatile unsigned char*)NAND_RD_ADDR;

  {
    printf("Manufacturer code 0x%x, ", man);
    
    switch (man) {
    case 0x20: {
      printf("ST");
      break;
    }
    default: {
      printf("unknown");
      break;
    }
    }
    printf("\n");
  }
  
  {
    printf("Device code 0x%x, ", dev);
    switch (dev) {
    case 0xf1: {
      printf("NAND01GW3B");
      rd_cycles  = 4;
      cmd_rd_end = 1;
      break;
      /* bs 101011 */
    }
    case 0xda: {
      printf("NAND02GW3B");
      rd_cycles  = 5;
      cmd_rd_end = 1;
      break;
      /* bs 111011 */
    }
    case 0x75: {
      printf("NAND256W3A");
      rd_cycles  = 3;
      cmd_rd_end = 0;
      break;
    }
    default: {
      rd_cycles  = 0;
      cmd_rd_end = 0;
      printf("unknown");
      break;
    }
    }
    printf("\n");

    printf("Cycles: %d, 0x30 cmd %d.\n", rd_cycles, cmd_rd_end);
  }
  SET(CE);
  SET(CE);


  printf("<read_sig\n");
  return(rd_cycles);
}

int 
nandwr(void)
{
  int i;

  printf("######################################################################\n");

  { /* pinmux */
    reg_pinmux_rw_pa pa = REG_RD(pinmux, regi_pinmux, rw_pa);
    
    pa.pa0 = pa.pa1 = pa.pa2 = pa.pa3 = pa.pa4 = 
      pa.pa5 = pa.pa6 = pa.pa7 = regk_pinmux_yes;
    REG_WR(pinmux, regi_pinmux, rw_pa, pa);
  }

  { /* gio */
    reg_gio_rw_pa_oe pa_oe = REG_RD(gio, regi_gio, rw_pa_oe);
    
    pa_oe.oe |= (1 << CE);
    pa_oe.oe |= (1 << ALE);
    pa_oe.oe |= (1 << CLE);
    pa_oe.oe &= ~ (1 << BY);
    REG_WR(gio, regi_gio, rw_pa_oe, pa_oe);
  }
  
  { /* bif */
    reg_bif_core_rw_grp3_cfg bif_cfg = REG_RD(bif_core, regi_bif_core, rw_grp3_cfg);
    
    bif_cfg.gated_csp0 = regk_bif_core_rd;
    bif_cfg.gated_csp1 = regk_bif_core_wr;
    REG_WR(bif_core, regi_bif_core, rw_grp3_cfg, bif_cfg);
  }

  for (i = 0; i < 128; i++) {
    buffer[i] = i & 0xff;
    buffer[i] = test_prog[i];
  }

  {
    unsigned cnt = 0;
    while(cnt < 10) {
      cnt = read_sig() ? cnt+1 : 0;
    }
  }

  erase();
  read(empty);  

  program(buffer, 0);
  read(buffer);  

#if 1
  __asm__ volatile 
    ("\
jump %0\n\t
nop\n\t
" : : "a" (buffer));
#endif


  return 0;
}

//unsigned virt2phys(unsigned a) { return a;};
