/*!***************************************************************************
*!
*! FILE NAME  : net_init.c
*!
*! DESCRIPTION: Used in both level 1 and 2 but executes a little differently 
*! depending on level.
*!
*! ---------------------------------------------------------------------------
*! (C) Copyright 1998-2006, Axis Communications AB, LUND, SWEDEN
*!***************************************************************************/

#include <e100boot.h>	
#include <cbl.h>
#include <eth_defs.h>
#include <dma_defs.h>
#include <config_defs.h>
#include <timer_defs.h>
#include <dma.h>
#include <strmux_defs.h>

#define NET_GA_LO_32 0x008c4001
#define NET_GA_HI_16 0x0002

/***********************************************************************/
/* rx */
static dma_descr_context volatile descr_context_rx 
__attribute__ ((aligned (32))) = 
{
  .eol            = regk_dma_yes,
  .intr           = regk_dma_no,
  .store_mode     = regk_dma_no,
  .en             = regk_dma_yes,
  .dis            = regk_dma_yes, /* pretend we have sent one packet already */
  .md0            = 0,
  .md1            = 0,
  .md2            = 0,
  .md3            = 0,
  .md4            = 0,
  .saved_data     = (dma_descr_data*)&rx_descr,
  .saved_data_buf = (char*)&rx_header,
};

/* this is where we receive packet data */
dma_descr_data volatile rx_descr2
__attribute__ ((aligned (32))) = 
{
  .buf    = (char*)0x38000800,
  .eol    = regk_dma_yes,
  /* allow a full ethernet packet minus packet header received in rx_descr */
  .after  = (char*)0x38000800+1518-sizeof(struct packet_header_T), 
  
};

/* this is where we receive the header */
dma_descr_data volatile rx_descr 
__attribute__ ((aligned (32))) = 
{
  .next    = (dma_descr_data*)&rx_descr2,
  .buf     = (char*)&rx_header,
  .after   = (char*)&rx_header+sizeof(struct packet_header_T),
};

static struct packet_header_T rx_header;

/***********************************************************************/
/* tx */

dma_descr_data volatile tx_descr
__attribute__ ((aligned (32))) = 
{
  .next = NULL,
  .buf  = (char*)&tx_header,
  .eol  = regk_dma_yes,
  .out_eop = regk_dma_yes,
  .intr    = regk_dma_no,
  .wait    = regk_dma_no,
  .in_eop  = regk_dma_no,
  .md      = 0,
  .after   = (char*)&tx_header+sizeof(struct packet_header_T),
};

static dma_descr_context volatile descr_context_tx 
__attribute__ ((aligned (32))) = {
  .eol            = regk_dma_yes,
  .intr           = regk_dma_no,
  .store_mode     = regk_dma_no,
  .en             = regk_dma_yes,
  .dis            = regk_dma_yes, /* pretend we have sent one packet already */
  .md0            = 0,
  .md1            = 0,
  .md2            = 0,
  .md3            = 0,
  .md4            = 0,
  .saved_data     = (dma_descr_data*)&tx_descr,
  .saved_data_buf = (char*)&tx_header,
};

struct packet_header_T tx_header = {
  .dest   = { 0x01, 0x40, 0x8c, 0x00, 0x04, 0x00 }, /* same as boot rom */
  .src    = { 0x01, 0x40, 0x8c, 0x00, 0x02, 0x00 }, /* same as boot rom */
  .length = 0,
  .snap1  = 0x0003aaaa ,	/* htonl(aaaa0300) */
  .snap2  = 0x56888c40,		/* htonl(408c8856) */
  .tag    = 0xfeffffff,		/* htonl(fffffffe) */
  .seq    = 0,
  .type   = 0,   /* string */
};

/***********************************************************************/

/* Have we set the destination address in tx_header? */
static byte   dest_set = FALSE;

/* Sequence number of next wanted packet */
udword seq;	   

#define DESCR_CONTEXT_TX_SAVED_DATA_BUF (char*)&tx_header

#define MAX_DATA_SIZE 1500

#define SET_CURRENT_CONTEXT( inst, context) \
  REG_WR( dma, inst, rw_group_down, context );

#ifndef NTOHL     
#define NTOHL(x) HTONL(x)
#endif

#ifndef NTOHS
#define NTOHS(x) HTONS(x)
#endif

/**************************************************************************/
/* function prototypes */

void init_dma0(void);
void init_dma1(void);
inline udword HTONL(udword x);
inline uword HTONS(uword x);
extern int timeout(void);
void net_boot(void);
void start_tx(void);
void send_net_ack(void);
/**************************************************************************/

void 
net_init(unsigned boot_level)
{

  if (boot_level == 1) {
    seq = 0;		
    /* Get a random value to use as id. */
    tx_header.id     = REG_RD(timer, regi_timer, r_time);

  }
  else {			/* level 2 */
    seq = *(unsigned*)ADDR_SEQ;	/* inherit seq from level 1 */
    tx_header.id = *(unsigned*)ADDR_ID;
  }
  
  /* timer01 is used as timer, read it from id above. */
  last_timeout     = tx_header.id;
  

  { /* enable dma0 in strmux */
    reg_strmux_rw_cfg rw_cfg = REG_RD(strmux, regi_strmux, rw_cfg);
    rw_cfg.dma0 = regk_strmux_eth0;
    REG_WR(strmux, regi_strmux, rw_cfg, rw_cfg);
  }

  { /* initialize transmitter */
    {
      reg_eth_rw_tr_ctrl rw_tr_ctrl = {
	.crc        = regk_eth_yes,
	.pad        = regk_eth_yes,
	.retry      = regk_eth_yes,
	.ignore_col = regk_eth_no,
	.cancel     = regk_eth_no,
	.hsh_delay  = regk_eth_no
      };
      
      REG_WR(eth, regi_eth0, rw_tr_ctrl, rw_tr_ctrl);
    }
        
#if 1
    {
      //START (ETHERNET INTERRUPT FIX)
      reg_eth_rw_clr_err rw_clr_err;
      rw_clr_err.clr = regk_eth_yes;
      REG_WR( eth, regi_eth0, rw_clr_err, rw_clr_err);

      reg_eth_rw_ack_intr rw_ack_intr = {0xFFFF};
      REG_WR( eth, regi_eth0, rw_ack_intr, rw_ack_intr);
      //END (ETHERNET INTERRUPT FIX)
    }
#endif
  }

  { /* initialize receiver */
    { 
      reg_eth_rw_rec_ctrl rw_rec_ctrl = {
        .max_size   = regk_eth_size1518,
        .duplex     = regk_eth_half,
        .bad_crc    = regk_eth_discard,
        .oversize   = regk_eth_discard,
        .broadcast  = regk_eth_discard,
        .individual = regk_eth_no,
        .ma0        = regk_eth_yes,
        .ma1        = regk_eth_no
      };
      

      { /* full duplex selected on boot pins? */
	
	reg_config_r_bootsel r_bootsel = REG_RD(config, regi_config, r_bootsel);
	if (r_bootsel.full_duplex == regk_config_yes) {
	  rw_rec_ctrl.duplex = regk_eth_full;
	}
	
	REG_WR(eth, regi_eth0, rw_rec_ctrl, rw_rec_ctrl);
      }
    }

    { /* address */
      { /* set lower part of station address register */
        reg_eth_rw_ma0_lo rw_ma0_lo = {
          .addr = NET_GA_LO_32
        };
        REG_WR(eth, regi_eth0, rw_ma0_lo, rw_ma0_lo);
      }
      
      { /* set higher part of station address register */
        reg_eth_rw_ma0_hi rw_ma0_hi = {
          .addr = NET_GA_HI_16
        };
        REG_WR(eth, regi_eth0, rw_ma0_hi, rw_ma0_hi);
      }
    }
  }
  
  { /* enable network */ 
    reg_eth_rw_gen_ctrl rw_gen_ctrl = {
      .loopback = regk_eth_no,
      .protocol = regk_eth_ether,
      .phy      = regk_eth_mii_clk,
      .en       = regk_eth_yes
    };
    REG_WR(eth, regi_eth0, rw_gen_ctrl, rw_gen_ctrl);
  }
  
}

void
init_dma0(void)
{ 

  /* reset dma0 */
  reg_dma_rw_cfg rw_cfg = REG_RD( dma, regi_dma0, rw_cfg );
  rw_cfg.en   = regk_dma_no; 
  rw_cfg.stop = regk_dma_no; 
  REG_WR( dma, regi_dma0, rw_cfg, rw_cfg);
  
  /* restore descriptors */
  descr_context_tx.dis            = regk_dma_no;
  descr_context_tx.saved_data     = (dma_descr_data*)&tx_descr;
  descr_context_tx.saved_data_buf = (char*)&tx_header;

  /* enable dma0 */
  rw_cfg.en = regk_dma_yes; 
  REG_WR( dma, regi_dma0, rw_cfg, rw_cfg);
  
  /* set context descriptor */
  SET_CURRENT_CONTEXT(regi_dma0, (unsigned)&descr_context_tx);
  
  /* load context descriptor */
  DMA_WR_CMD( regi_dma0, regk_dma_load_c );
  
}


/**************************************************************************/

void
init_dma1(void)
{ 
  
  /* Reset dma1. */
  reg_dma_rw_cfg rw_cfg = REG_RD( dma, regi_dma1, rw_cfg );
  rw_cfg.en   = regk_dma_no; 
  rw_cfg.stop = regk_dma_no; 
  REG_WR( dma, regi_dma1, rw_cfg, rw_cfg);

  /* restore descriptors */
  rx_descr2.in_eop = regk_dma_no;
  
  {
    unsigned bytes_left = bytes_to_read - nbr_read;
    /* how many Bytes should there be In this Descr? */
    unsigned bid = ( bytes_left + CRC_LEN < MAX_DATA_SIZE ?
		     bytes_left + CRC_LEN : MAX_DATA_SIZE);
    /* Less than minimum eth packet? */
    bid = bid + NETWORK_HEADER_LENGTH + CRC_LEN< 64 ? 64 : bid;
    rx_descr2.after = (char*)rx_descr2.buf + bid;
  }

  /* rx_descr does not need restore. It is never changed by the dma
     since eop can never be set in it. */

  descr_context_rx.saved_data     = (dma_descr_data*)&rx_descr;
  descr_context_rx.saved_data_buf = rx_descr.buf;
  descr_context_rx.dis            = regk_dma_no;

  /* enable dma1 */
  rw_cfg.en = regk_dma_yes; 
  REG_WR( dma, regi_dma1, rw_cfg, rw_cfg);
  
  /* set context descriptor */
  SET_CURRENT_CONTEXT(regi_dma1, (unsigned)&descr_context_rx);

  /* load context descriptor */
  DMA_WR_CMD( regi_dma1, regk_dma_load_c );
  
  /* load start position and start burst */
  DMA_WR_CMD( regi_dma1, (regk_dma_load_d | regk_dma_burst) );
  
}

/**************************************************************************/
/* we got a packet, check if it was to us, restart receiver */

int
handle_network_read(void)
{
  unsigned bytes_in_pkt = 0;

  /* Was it to me and correct sequence number? */
  if ( (rx_header.id == tx_header.id) && (NTOHL(rx_header.seq) == (seq+1)) ) { 
    last_timeout = REG_RD(timer, regi_timer, r_time);
    bytes_in_pkt = (rx_descr2.after - rx_descr2.buf) - CRC_LEN;
    nbr_read        += bytes_in_pkt;
    rx_descr2.buf   += bytes_in_pkt;
    /* after/in_eop set in init_dma1() */

    if (!dest_set) { /* Copy destination address from first received packet */
      dest_set = TRUE;
      *(udword*)&tx_header.dest[0] = *(udword*)&rx_header.src[0];
      *(uword*)&tx_header.dest[4]  = *(uword*)&rx_header.src[4];
    }
  
    seq++;
    send_net_ack();		/* reply */
    
  }

  /* restart receiver if more data is wanted */
  if(nbr_read < bytes_to_read) {
    init_dma1();		
  }
  
  return(bytes_in_pkt);
}

/**************************************************************************/

void
send_net_ack(void)
{

  tx_header.seq    = HTONL(seq);
  tx_header.type   = HTONL(ACK);
  tx_header.length = HTONS(64-ETHERNET_HDR_SIZE);

  tx_descr.wait    = regk_dma_yes;
  tx_descr.out_eop = regk_dma_yes;
  tx_descr.eol     = regk_dma_yes;

  //  tx_header.id     = nbr_read;
  start_tx();
  
}

/**************************************************************************/
/* start and wait for a transmission */
void
start_tx(void)
{

  /*  send_serial_string(">start_network_trans\r\n");*/

  init_dma0();

  { /* Clear transmit errors. */
    reg_eth_rw_clr_err rw_clr_err = {
      .clr = regk_eth_yes
    };
    REG_WR(eth, regi_eth0, rw_clr_err, rw_clr_err);
  }

  /* load start position and start burst */
  DMA_WR_CMD( regi_dma0, (regk_dma_load_d | regk_dma_burst) );

  while ( (descr_context_tx.dis == regk_dma_no) ) {
    ;				/* wait */
    /* add exc-col test */
  }
}

/**************************************************************************/
