#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

ssize_t safe_read(int fd, void *buf, size_t count)
{
	ssize_t ret;

	do {
		ret = read(fd, buf, count);
	} while (ret < 0 && errno == EINTR);

	return ret;
}

/*
 * This utility calculate the checksum for an image
 */
int main(int argc, char **argv)
{
	int fd;
	int i;
	long sum=0;
	struct stat sb;
	unsigned char* addr; 
	int count;
	char* filename;

	/* file parameter */
	if (argc != 2) {
		fprintf(stderr, "Usage: imgsum <filename>\n");
		exit(1);
	}		
	filename = argv[1];

	/* get image file info */
	if (stat(filename, &sb) != 0) {
		perror("stat");
		exit(1);
	}
	
	/* allocate memory for the image */
	addr = (unsigned char*) malloc(sb.st_size);
	if (addr == NULL) {
		fprintf(stderr, "Out of memory\n");
		exit(1);
	}
	
	/* open image file */
	if ((fd = open(filename, O_RDONLY)) < 1) {
		perror("open");
		exit(1);
	}
	
	/* fetch image into memory */
	if ((count = safe_read(fd, addr, sb.st_size)) < 0) {
		perror("read");
		exit(1);
	}

	close(fd);

	/* start counting */
	i = 0;
	while (i < count) { 		
		/*  This is the inner loop */
		sum += *((unsigned char *)(addr+i++));
	}

	printf("%16lu", sum);
	return 0;
}
