# Hey Emacs, this Makefile is in -*- makefile -*- mode!

###########################################################################

PACKAGE_DIR ?= .

BUILD_DIR := build/$(AXIS_BUILDTYPE)

ifneq ($(AXIS_BUILDTYPE),host)
INT_CONFIG_FLAGS := --host=$(AXIS_BUILDTYPE)
else
INT_CONFIG_FLAGS :=
endif

###########################################################################

all:	all_autoconf
config:	config_autoconf
install:	install_autoconf
uninstall:	uninstall_autoconf
clean:	clean_autoconf

all_autoconf:	config
	$(MAKE) -C $(BUILD_DIR) all

config_autoconf:	$(BUILD_DIR)/.target-configured

$(BUILD_DIR)/.target-configured:	$(BUILD_DIR)/Makefile
	touch $@

$(BUILD_DIR)/Makefile:	$(BUILD_DIR)/installit $(PACKAGE_DIR)/configure
	if $(PACKAGE_DIR)/configure --help 2>/dev/null | grep -q -- --config-cache; then \
	  USE_CONFIG_CACHE=--config-cache; \
	fi; \
	cd $(BUILD_DIR) && \
	CC="$(CC)" \
	CXX="$(CXX)" \
	OBJCOPY="$(OBJCOPY)" \
	LD="$(LD)" \
	AR="$(AR)" \
	RANLIB="$(RANLIB)" \
	STRIP="$(STRIP)" \
	INSTALL="$(shell pwd)/$(BUILD_DIR)/installit" \
	RM="$(RM)" \
	CFLAGS="$(CFLAGS)" \
	CXXFLAGS="$(CXXFLAGS)" \
	LDFLAGS="$(LDFLAGS)" \
	PKG_CONFIG_PATH="$(prefix)/usr/lib/pkgconfig:$(prefix)/lib/pkgconfig" \
	$(CONFIG_VARS) \
	../../$(PACKAGE_DIR)/configure $$USE_CONFIG_CACHE $(INT_CONFIG_FLAGS) $(CONFIG_FLAGS)
ifeq ($(AXIS_TARGET_ARCH), cris)
	if [ -f $(BUILD_DIR)/libtool ]; then \
	  perl -pi -e 's/-fPIC/-fpic/g' $(BUILD_DIR)/libtool; \
	fi
endif

$(BUILD_DIR)/installit:
	$(INSTALL) -d $(BUILD_DIR)
	{ \
	  echo '#!/bin/sh'; \
	  echo; \
	  echo '$(INSTALL) -p "$$@"'; \
	} > "$@"
	chmod +x "$@"

ifneq ($(wildcard $(PACKAGE_DIR)/autogen.sh),)
ifneq ($(wildcard $(PACKAGE_DIR)/configure.ac),)
$(PACKAGE_DIR)/configure:	$(PACKAGE_DIR)/autogen.sh $(PACKAGE_DIR)/configure.ac
else
$(PACKAGE_DIR)/configure:	$(PACKAGE_DIR)/autogen.sh $(PACKAGE_DIR)/configure.in
endif
	cd $(PACKAGE_DIR) && \
	$(AUTOGEN_VARS) ./autogen.sh $(AUTOGEN_FLAGS)
	touch $@
endif

ifdef CONFIG_CACHE_VARS
$(BUILD_DIR)/Makefile:	$(BUILD_DIR)/config.cache

$(BUILD_DIR)/config.cache:
	$(INSTALL) -d $(BUILD_DIR)
	for line in $(CONFIG_CACHE_VARS); do \
	  echo "$$line" >> "$(BUILD_DIR)/config.cache"; \
	done
endif

install_autoconf:	config
	$(MAKE) -C $(BUILD_DIR) install
ifdef INCLUDE_FILES
	for f in $(INCLUDE_FILES); do \
	  dir=`dirname "$(prefix)$$f"`; \
	  file=`basename "$(prefix)$$f"`; \
	  metafiles --prefix "$$dir" include "$$file"; \
	done
endif
ifdef IGNORE_FILES
	for f in $(IGNORE_FILES); do \
	  dir=`dirname "$(prefix)$$f"`; \
	  file=`basename "$(prefix)$$f"`; \
	  metafiles --prefix "$$dir" ignore "$$file"; \
	done
endif

uninstall_autoconf:	config
	$(MAKE) -C $(BUILD_DIR) uninstall

clean_autoconf:
	$(RM) -r $(BUILD_DIR)
	rmdir build 2>/dev/null || true

###########################################################################
