# Hey Emacs, this Makefile is in -*- makefile -*- mode!

###########################################################################

# Try 'make axishelp' to display the possible targets

###########################################################################
# First of all, check that we're running a compatible make version.
AXIS_REQUIRED_MAKE_VERSION=3.80

AXIS_THIS_MAKE_VERSION=$(firstword $(MAKE_VERSION) 3.68-or-older)

# (This'll break once we reach GNU make version 3.100, but we'll have
# other methods by then, perhaps nsort or regexps.)
ifneq ($(AXIS_REQUIRED_MAKE_VERSION), $(firstword \
  $(sort $(AXIS_THIS_MAKE_VERSION) $(AXIS_REQUIRED_MAKE_VERSION))))
$(error GNU make version $(AXIS_REQUIRED_MAKE_VERSION) or newer is required,\
	this is $(AXIS_THIS_MAKE_VERSION))
endif

# Then we include the product configuration (if it exists).

-include $(AXIS_TOP_DIR)/.config

###########################################################################
# Variables needed by multiple rule files.

makefrag_name = .target-makefrag

## 
## The following build types exist:
## cris-axis-linux-gnu           - CRIS/Linux 2.4/2.6 with shared glibc
## cris-axis-linux-gnuuclibc     - CRIS/Linux 2.4/2.6 with shared uClibc
## crisv32-axis-linux-gnu        - CRISv32/Linux 2.6 with shared glibc
## crisv32-axis-linux-gnuuclibc  - CRISv32/Linux 2.6 with shared uClibc
## arm-axis-linux-gnu            - ARM/Linux 2.4/2.6 with shared glibc
## arm-axis-linux-gnuuclibc      - ARM/Linux 2.4/2.6 with shared uClibc
## clinux                        - CRIS/Linux 2.4 with static uClibc
## elinux                        - CRIS/Linux 2.0 NO_MMU with uC-libc
## host                          - Host build with debug
AXIS_BUILDTYPES  = cris-axis-linux-gnu
AXIS_BUILDTYPES += cris-axis-linux-gnuuclibc
AXIS_BUILDTYPES += crisv32-axis-linux-gnu
AXIS_BUILDTYPES += crisv32-axis-linux-gnuuclibc
AXIS_BUILDTYPES += arm-axis-linux-gnu
AXIS_BUILDTYPES += arm-axis-linux-gnuuclibc
AXIS_BUILDTYPES += clinux
AXIS_BUILDTYPES += elinux
AXIS_BUILDTYPES += host

###########################################################################

# We need to have an all rule here to make sure it is always the first.
all:

## 
## The following special targets exists:

## axishelp                      - This help
axishelp:
	@grep -h '^## ' $(AXIS_TOP_DIR)/tools/build/rules/*.mak

###########################################################################
