# Hey Emacs, this Makefile is in -*- makefile -*- mode!

###########################################################################

AXIS_WARNING_PREFIX = ---------- WARNING:
AXIS_ERROR_PREFIX   = ---------- ERROR:
AXIS_LIB_MISMATCH = The code in $(shell pwd) cannot be linked with\
 $(AXIS_WILL_USE_LIB)

# A nifty macro to make testing gcc features easier
check_opt=$(shell if $(1) $(2) -S -o /dev/null -xc /dev/null >/dev/null 2>&1; \
        then echo "$(2)"; else echo "$(3)"; fi)

###########################################################################

ifeq ($(AXIS_TARGET_OS),elinux)
# This is for gcc-cris & Co. of R25 or higher.  Get it; don't
# use older versions or R25 without glibc installed.

# Without "-melinux", the compiler will not get the right
# include-files and have the right defines etc. so it should be
# seen as part of CC rather than CFLAGS.
CC_FLAGS    = -melinux

# This is for compatibility with cris-dist-1.7, where a.out was the
# default, but which does not understand -mcrisaout.  We'll only do the
# costly assignment if we don't inherit the value, which is a " " or
# -mcrisaout.
ifndef AXIS_LD_OUTPUT_FORMAT_OPTION
AXIS_LD_OUTPUT_FORMAT_OPTION := $(shell `gcc-cris -print-prog-name=ld` -V | sed -n -e 's/.* crisaout/-mcrisaout/p') #
endif

# eLinux always uses uClibc.  Though a check that the app can use uClibc
# is in order.
ifdef AXIS_USABLE_LIBS
ifneq ($(filter $(AXIS_USABLE_LIBS),UCLIBC),UCLIBC)
$(error $(AXIS_ERROR_PREFIX) $(AXIS_LIB_MISMATCH))
endif # UCLIBC not in AXIS_USABLE_LIBS
endif # AXIS_USABLE_LIBS

# OBSOLETE: Remove whenever possible
ELINUX    = 1

endif # AXIS_TARGET_OS == elinux

#--------------------------------------------------------------------------

ifeq ($(AXIS_TARGET_OS),linux)

# Here we will check AXIS_WILL_USE_LIB against AXIS_USABLE_LIBS which
# represents what the thing we want to compile can be linked with.  If
# library and compile variable settings (like CC) aren't supposed to be
# set or are not applicable, then AXIS_USABLE_LIBS should not be set.

ifdef AXIS_USABLE_LIBS
ifneq ($(filter $(AXIS_USABLE_LIBS),$(AXIS_WILL_USE_LIB)),$(AXIS_WILL_USE_LIB))
$(error $(AXIS_ERROR_PREFIX) $(AXIS_LIB_MISMATCH))
endif
endif # AXIS_USABLE_LIBS

# Now that we've checked that AXIS_WILL_USE_LIB matches AXIS_USABLE_LIBS,
# we set the other build vars.

CC_FLAGS  =

ifneq ($(AXIS_WILL_USE_LIB),UCLIBC)
ifeq ($(AXIS_WILL_USE_LIB),GLIBC)
# Using glibc, linking dynamically like on the host.

# We prefer getting <asm/...> and <linux/...> from the kernel we compile,
# rather than what was installed with the compiler, so put it first in the
# system include path.  Though we have to go through a separate directory
# with just the linux and asm dirs, rather than use $(AXIS_KERNEL_DIR)/include,
# lest we also drag in the other subdirs, which collide with the normal
# gcc-cris -mlinux system-includes, for example <net/...>.

CC_FLAGS += -isystem $(prefix)/include -isystem $(prefix)/usr/include
else
$(error $(AXIS_ERROR_PREFIX) Unexpected library $(AXIS_WILL_USE_LIB).)
endif # AXIS_WILL_USE_LIB == GLIBC
endif # AXIS_WILL_USE_LIB != UCLIBC

AXIS_LD_OUTPUT_FORMAT_OPTION =

endif # AXIS_TARGET_OS == linux

#--------------------------------------------------------------------------

ifeq ($(AXIS_TARGET_CPU)-$(AXIS_TARGET_OS),crisv10-linux)

CC_FLAGS += -mlinux

AXIS_LD_OUTPUT_FORMAT_OPTION = -mcrislinux

# Check if the compiler supports -mno-mul-bug-workaround, in which case
# it shall be used.
NO_MUL_WORKAROUND := $(call check_opt,gcc-cris,-mno-mul-bug-workaround,)
CC_FLAGS += $(NO_MUL_WORKAROUND)

endif # AXIS_TARGET_CPU == crisv10 && AXIS_TARGET_OS == linux

#--------------------------------------------------------------------------

ifeq ($(AXIS_TARGET_CPU)-$(AXIS_TARGET_OS),crisv32-linux)

CC_FLAGS += -mlinux -march=v32

AXIS_LD_OUTPUT_FORMAT_OPTION = -mcrislinux

endif # AXIS_TARGET_CPU == crisv32 && AXIS_TARGET_OS == linux

#--------------------------------------------------------------------------

ifeq ($(AXIS_TARGET_ARCH),cris)

# This is for building for the CRIS architecture.

ifeq ($(AXIS_WILL_USE_LIB),UCLIBC)
# We use a wrapper (gcc_cris) when compiling with uC-libc
CC        = $(AT_PRETTY) $(CCACHE) gcc_cris $(CC_FLAGS) -muclibc=$(prefix)
CXX       = $(AT_PRETTY) $(CCACHE) g++_cris $(CC_FLAGS) -muclibc=$(prefix)
else
CC        = $(AT_PRETTY) $(CCACHE) gcc-cris $(CC_FLAGS)
CXX       = $(AT_PRETTY) $(CCACHE) g++-cris $(CC_FLAGS)
endif

OBJCOPY   = objcopy-cris
LD        = $(AT_PRETTY) ld-cris $(AXIS_LD_OUTPUT_FORMAT_OPTION)
AR        = $(AT_PRETTY) ar-cris
RANLIB    = ranlib-cris
STRIP     = $(AT_PRETTY) strip-cris

endif # AXIS_TARGET_ARCH == cris

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_TARGET_ARCH),arm)

# This is for building for the ARM architecture.

CC_FLAGS  += -march=armv4 -mtune=arm9tdmi -D__ARM__

ifeq ($(AXIS_WILL_USE_LIB),UCLIBC)
# We use a wrapper (arm_linux_gcc) when compiling with uC-libc
CC        = $(AT_PRETTY) $(CCACHE) arm_linux_gcc $(CC_FLAGS) -muclibc=$(prefix)
CXX       = $(AT_PRETTY) $(CCACHE) arm_linux_g++ $(CC_FLAGS) -muclibc=$(prefix)
else
CC        = $(AT_PRETTY) $(CCACHE) arm-linux-gcc $(CC_FLAGS)
CXX       = $(AT_PRETTY) $(CCACHE) arm-linux-g++ $(CC_FLAGS)
endif

OBJCOPY   = arm-linux-objcopy
LD        = $(AT_PRETTY) arm-linux-ld $(AXIS_LD_OUTPUT_FORMAT_OPTION)
AR        = $(AT_PRETTY) arm-linux-ar
RANLIB    = arm-linux-ranlib
STRIP     = $(AT_PRETTY) arm-linux-strip

endif # AXIS_TARGET_ARCH == arm

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_TARGET_ARCH),host)

# This is for building for the local host.

CC       ?= gcc
USER_CC  := $(CC)
CC        = $(AT_PRETTY) $(CCACHE) $(USER_CC)
CXX      ?= g++
USER_CXX := $(CXX)
CXX       = $(AT_PRETTY) $(CCACHE) $(USER_CXX)
LD        = $(AT_PRETTY) ld
AR        = $(AT_PRETTY) ar
RANLIB    = ranlib
STRIP     = $(AT_PRETTY) strip

endif # AXIS_TARGET_ARCH == host

###########################################################################

# Now that $(CC) is initialized for the intended target architecture, we
# can find out the version of the compiler.
ifndef AXIS_GCC_VERSION
AXIS_GCC_VERSION := $(shell $(CC) -dumpversion 2> /dev/null)
endif

export AXIS_GCC_VERSION

###########################################################################

# Define CFLAGS to something sensible.
CFLAGS    = -Wall -Wshadow

ifeq ($(AXIS_BUILDTYPE),host)
# cris-axis-linux-gnu has this in -isystem option.
# gcc_cris adds it when using -muclibc with elinux and clinux
CFLAGS   += -I$(prefix)/include
endif

ifeq ($(AXIS_OPT_DEBUG),y)
# Add suitable compiler flags for debugging.
CFLAGS   += -O0 -g -fno-omit-frame-pointer
else
CFLAGS   += -O2
ifneq ($(AXIS_TARGET_OS),elinux)
CFLAGS   += -g
endif
endif

# OBSOLETE: Add ELINUX define.
ifeq ($(AXIS_TARGET_OS),elinux)
CFLAGS   += -DELINUX
endif

# Make it possible to add flags from outside.
CFLAGS   += $(EXTRA_CFLAGS)

#--------------------------------------------------------------------------

# Can't use -pedantic due to use of long long in standard includes. :(
CXXFLAGS  = $(CFLAGS) -Wno-ctor-dtor-privacy -ansi -pipe

#--------------------------------------------------------------------------

# Define LDFLAGS to something sensible.
LDFLAGS   =

# These are added by the wrapper script when building with uClibc.
ifneq ($(AXIS_WILL_USE_LIB),UCLIBC)
LDFLAGS  += -L$(prefix)/lib -L$(prefix)/usr/lib
endif

# Help gcc find DSOs NEEDED by other DSOs.
ifneq ($(AXIS_TARGET_ARCH),cris)
LDFLAGS  += -Wl,-rpath-link,$(prefix)/lib,-rpath-link,$(prefix)/usr/lib
endif

# Strip the binaries when building for elinux unless debug or static.
ifeq ($(AXIS_TARGET_OS),elinux)
ifneq ($(AXIS_OPT_DEBUG),y)
ifneq ($(AXIS_OPT_STATIC),y)
LDFLAGS  += -s
endif
endif
endif

ifeq ($(AXIS_OPT_STATIC),y)
LDFLAGS  += -static
else
ifeq ($(AXIS_TARGET_OS),elinux)
LDFLAGS  += -shlib
endif
endif # AXIS_OPT_STATIC == y

# Make it possible to add link flags from outside.
LDFLAGS   += $(EXTRA_LDFLAGS)

#--------------------------------------------------------------------------

# Define LDLIBS to something sensible.
LDLIBS    =

# Make it possible to add libs from outside.
LDLIBS   += $(EXTRA_LDLIBS)

###########################################################################

CCACHE    =

ifeq ($(AXIS_USE_CCACHE),yes)
ifneq ($(shell which ccache 2>/dev/null),)
CCACHE    = ccache
endif
endif

###########################################################################
