# Hey Emacs, this Makefile is in -*- makefile -*- mode!

###########################################################################

# By default we do not build statically except for one target (clinux),
# but it can be enabled/disabled with the targets static and nostatic.
# The variable can also be specified on the make command line, either
# before or after 'make'.
AXIS_OPT_STATIC ?=

###########################################################################
# Start by deciding what we are building for

# If we have a .target-makefrag, read stuff from it.
-include $(makefrag_name)

# Build for host if the build type is not defined.
AXIS_BUILDTYPE ?= host

#--------------------------------------------------------------------------
# Deduce other build variables from AXIS_BUILDTYPE.

AXIS_BUILDTYPE_KNOWN = no

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),cris-axis-linux-gnu)
AXIS_TARGET_ARCH = cris
AXIS_TARGET_CPU = crisv10
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = GLIBC
AXIS_SHARE_LIBS = yes
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == cris-axis-linux-gnu

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),crisv32-axis-linux-gnu)
AXIS_TARGET_ARCH = cris
AXIS_TARGET_CPU = crisv32
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = GLIBC
AXIS_SHARE_LIBS = yes
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == crisv32-axis-linux-gnu

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),arm-axis-linux-gnu)
AXIS_TARGET_ARCH = arm
AXIS_TARGET_CPU = argus
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = GLIBC
AXIS_SHARE_LIBS = no
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == arm-axis-linux-gnu

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),cris-axis-linux-gnuuclibc)
AXIS_TARGET_ARCH = cris
AXIS_TARGET_CPU = crisv10
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = UCLIBC
AXIS_SHARE_LIBS = yes
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == cris-axis-linux-gnuuclibc

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),crisv32-axis-linux-gnuuclibc)
AXIS_TARGET_ARCH = cris
AXIS_TARGET_CPU = crisv32
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = UCLIBC
AXIS_SHARE_LIBS = yes
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == crisv32-axis-linux-gnuuclibc

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),arm-axis-linux-gnuuclibc)
AXIS_TARGET_ARCH = arm
AXIS_TARGET_CPU = argus
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = UCLIBC
AXIS_SHARE_LIBS = yes
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == arm-axis-linux-gnuuclibc

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),clinux)
AXIS_TARGET_ARCH = cris
AXIS_TARGET_CPU = crisv10
AXIS_TARGET_OS = linux
AXIS_WILL_USE_LIB = UCLIBC
# Default static
AXIS_OPT_STATIC = y
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == clinux

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),elinux)
AXIS_TARGET_ARCH = cris
AXIS_TARGET_CPU = crisv10
AXIS_TARGET_OS = elinux
AXIS_WILL_USE_LIB = UCLIBC
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == elinux

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_BUILDTYPE),host)
AXIS_TARGET_ARCH = host
AXIS_TARGET_CPU = host
AXIS_TARGET_OS = host
AXIS_SHARE_LIBS = no
AXIS_BUILDTYPE_KNOWN = yes
endif # AXIS_BUILDTYPE == host

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ifeq ($(AXIS_OPT_STATIC),y)
AXIS_SHARE_LIBS = no
endif

#--------------------------------------------------------------------------

ifneq ($(AXIS_BUILDTYPE_KNOWN),yes)
$(error Unknown build type '$(AXIS_BUILDTYPE)'!)
endif

###########################################################################

prefix    = $(AXIS_TOP_DIR)/target/$(AXIS_BUILDTYPE)
export prefix

#--------------------------------------------------------------------------

ifeq ($(AXIS_BUILDTYPE),host)
run_prefix = $(prefix)
else
run_prefix =
endif

export run_prefix

###########################################################################
