# Hey Emacs, this Makefile is in -*- makefile -*- mode!

###########################################################################

TARGETS_MAK_INCLUDED = y

# Set VERBOSE_FRAG to nothing to print that stuff if you want it,
# it clutters up the screen so it's @ by default.
VERBOSE_FRAG = @

ifndef AXIS_CHECK_STRING
AXIS_CHECK_STRING = AXIS_BUILDTYPE ?= $(AXIS_BUILDTYPE)
AXIS_NEG_CHECK = !
endif

###########################################################################
# The following is a set of standard rules to try to make sure we build
# and install the correct files.

## 
## The following standard targets exists:
## all                           - Build everything
## install                       - Build and install everything
## uninstall                     - Remove all installed files
## clean                         - Remove all generated files
## check                         - Run unit tests
all install uninstall clean check:

#--------------------------------------------------------------------------

$(AXIS_BUILDTYPES):
	$(VERBOSE_FRAG)echo AXIS_BUILDTYPE ?= $@ > .tmp$(makefrag_name)
ifeq ($(AXIS_OPT_DEBUG),y)
	$(VERBOSE_FRAG)echo AXIS_OPT_DEBUG = y >> .tmp$(makefrag_name)
endif
	@$(MAKE) --no-print-directory checkfrag AXIS_BUILDTYPE=$@ || ( rm -f .tmp$(makefrag_name); exit 1 )
	@rm -f .tmp$(makefrag_name)

#--------------------------------------------------------------------------

## 
## The following build options exist:
## debug                         - Enable debug
## nodebug                       - Do not enable debug
## static                        - Enable static linking
## nostatic                      - Do not enable static linking
## opt-<name>                    - Enable option <name>
## opt-no-<name>                 - Disable option <name>

# We cannot defer the dependency on the recursive rule to recurse.mak due to
# how make handles empty implicit rules.
opt-%:	do-opt-%-recurse
	@option=`echo $@ | sed 's/^\(opt-\)\(no-\)\?\(.*\)/\1\3/'`; \
	target=`echo AXIS_$$option | tr a-z- A-Z_`; \
	grep -v "$$target" $(makefrag_name) > .tmp$(makefrag_name) 2> /dev/null; \
	if [ "$$option" = "$@" ]; then \
	  echo $$target = y >> .tmp$(makefrag_name); \
	  neg_check='!'; \
	fi; \
	$(MAKE) --no-print-directory checkfrag AXIS_CHECK_STRING="$$target = y" AXIS_NEG_CHECK=$$neg_check || ( rm -f .tmp$(makefrag_name); exit 1 )
	@rm -f .tmp$(makefrag_name)

# This is just a placeholder to keep make happy, if recurse.mak is not
# included.
do-opt-%-recurse:
	@

# These are kept for ease of use.
debug:		opt-debug
nodebug:	opt-no-debug
static:		opt-static
nostatic:	opt-no-static

#--------------------------------------------------------------------------

checkfrag:	checkclean
	-@if ! cmp -s .tmp$(makefrag_name) $(makefrag_name); then \
	  rm -f .target*; \
	  echo "Updating $(makefrag_name)"; \
	  cp .tmp$(makefrag_name)  $(makefrag_name); \
	fi

# Trailing $$ when running grep so we get an exact match, or changing from
# *-gnuuclibc to *-gnu will not make clean.
checkclean:
ifneq ($(AXIS_CLEAN_WHEN_NEW_BUILD_TYPE),no)
	@if [ ! -f $(makefrag_name) ] || \
	    $(AXIS_NEG_CHECK) grep "$(AXIS_CHECK_STRING)$$" $(makefrag_name) > /dev/null; then \
	  NO_SUBDIR_RECURSION=1 $(MAKE) --no-print-directory clean; \
	fi
endif

###########################################################################

# Convenient to have if you want to check out the asm.
%.s:	%.c
	$(CC) $(CFLAGS) -S -o $@ $<

# Convenient to have if you want to check out what the preprocessor does.
%.i:	%.c
	$(CC) $(CFLAGS) -E -o $@ $<

###########################################################################
