#!/usr/bin/perl -w

#*****************************************************************************
#!
#! FILE NAME  : init_env_elinux.pl
#!
#! PARAMETERS : [CVSROOT]
#!
#! DESCRIPTION: Create init_env file wich sets proper environment variables.
#!
#! FUNCTIONS  : None.
#!
#!----------------------------------------------------------------------------
#! HISTORY
#!
#! $Log: init_env_elinux.pl,v $
#! Revision 1.19  2002/11/28 16:22:29  pkj
#! * Updated $CVSROOT.
#! * Moved $AX_PRODUCTDIR, $ELIBSDIR, $AX_PACKAGES and $TOOLS to the
#!   obsolete section.
#!
#! Revision 1.18  2002/09/18 12:36:21  pkj
#! Reorganized the tools directory.
#!
#! Revision 1.17  2002/08/05 14:55:44  martinnn
#! CVS root has changed.
#!
#! Revision 1.16  2002/03/19 13:30:43  tobiasa
#! * Don't actually run pwd until init_env is sourced
#! * ELINUXDIR is obsoleted by AXIS_KERNEL_DIR, only keep it for backward
#!   compability
#!
#! Revision 1.15  2001/10/15 15:10:41  pkj
#! And with that little backslash it may even work as expected...
#!
#! Revision 1.14  2001/10/15 15:10:01  pkj
#! Set $(DEVTREE_ROOT) for backwards compatibility. :(
#!
#! Revision 1.13  2001/10/11 12:45:07  pkj
#! Clean-up.
#!
#! Revision 1.12  2001/10/11 12:07:43  pkj
#! $EROOT, $HROOT and $DEVTREE_ROOT are now deprecated.
#! Use $AXIS_TOP_DIR instead.
#!
#! Revision 1.11  2000/09/13 11:10:19  johana
#! Added AX_PRODUCTDIR
#!
#! Revision 1.10  2000/05/09 16:08:34  johana
#! More comments in generated file, easier to edit it
#!
#! Revision 1.9  2000/03/03 17:24:18  johana
#! Added AX_PACKAGES
#!
#!----------------------------------------------------------------------------
#! Copyright (C) 1999-2002, Axis Communications AB, LUND, SWEDEN
#!****************************************************************************
# $Id: init_env_elinux.pl,v 1.19 2002/11/28 16:22:29 pkj Exp $

#****************** INCLUDE FILES SECTION ************************************

use strict;

#****************** VARIABLE DECLARATION SECTION *****************************

use vars qw($cvs_rep $root_pwd);

#****************** CONSTANT SECTION *****************************************

#****************** MAIN PROGRAM SECTION *************************************

$cvs_rep = (defined($ARGV[0]) ? $ARGV[0] : ":ext:dev-cvs.axis.se:/usr/local/cvs/linux");

open(MYFILE, ">init_env") || die "Can not create init_env file\n";

print MYFILE <<EOF;
# init_env: Sets some environment variables
# Run using '. init_env' or 'source init_env'

export CVSROOT=$cvs_rep

# Sometimes the AXIS_TOP_DIR contains /dsk/blablabla/user
# instead of a simple /h/user, you can edit it if you want
export AXIS_TOP_DIR=`pwd`

export AXIS_KERNEL_DIR=\$AXIS_TOP_DIR/os/linux
export APPS=\$AXIS_TOP_DIR/apps

export PATH=\$AXIS_TOP_DIR/tools/build/bin:\$PATH:/usr/local/cris

# These are obsolete and only provided here for backwards compability
export DEVTREE_ROOT=\$AXIS_TOP_DIR
export ELINUXDIR=\$AXIS_KERNEL_DIR
export ELIBSDIR=\$AXIS_TOP_DIR/libs
export AX_PACKAGES=\$AXIS_TOP_DIR/packages
export AX_PRODUCTDIR=\$AXIS_TOP_DIR
export TOOLS=\$AXIS_TOP_DIR/tools/build/bin
EOF

close MYFILE;

print `cat init_env`;

print "init_env file created, run using '. init_env' or 'source init_env'\n";

#****************** FUNCTION DEFINITION SECTION ******************************

#****************** END OF FILE init_env_elinux.pl ***************************
