#!/usr/bin/perl 

# $Id: strrpl.pl,v 1.1 2003/07/01 09:41:43 petern Exp $

use Getopt::Long;
   
#*****************************************************************************
##
## FUNCTION NAME: main
##
## DESCRIPTION  : Reads the options and calls replRec with and empty directory 
##                list if no recursivity was asked for, otherwise with a list 
##                with all the directory files in the current directory.
##
##****************************************************************************
sub main(){
    GetOptions('recursive'=> \$rec,
	       'quiet' => \$quiet, 
	       'help'       => \$help)
	or usage();
    
    if($#ARGV>=2 && !$help){
	$inFiles=$#ARGV-1; # the last two parameters are patterns
	for($index=0;$index<$inFiles;$index++){
	    push @fileARGV, $ARGV[$index];
	}
	$oldPattern=$ARGV[$index++];
	$oldPattern = escapePattern($oldPattern);
	$newPattern=$ARGV[$index];
    }
    else{
	usage();
    }
	
    $curDir="./";
    opendir DIRH, ".", or die "Couldn't open '.'\n";
    @allfiles=grep !/^\.\.?$/, readdir DIRH; #-- omit current and parent directory
    @alldir=grep -d, @allfiles;
    unless ($rec){ #-- supress recursion by emptying the directory list
	@alldir=grep -f, @alldir; #-- how do one just clear a list?
    }
    &replRec(@alldir)
}

#*****************************************************************************
##
## FUNCTION NAME: escapePattern
##
## PARAMETERS   : $pattern - the pattern to escape
##
## DESCRIPTION  : Escape characters with special meaning in PERL string 
##                macthing.
##
##****************************************************************************
sub escapePattern{
  my($pattern) = @_[0];
  
  # escape '(' anb ')'
  $pattern =~ s/\(/\\(/g;
  $pattern =~ s/\)/\\)/g;
  # escape '$'
  $pattern =~ s/\$/\\\$/g;
  # escape '$'
  $pattern =~ s/\^/\\\^/g;
  return $pattern;
}

#*****************************************************************************
##
## FUNCTION NAME: replRec
##
## PARAMETERS   : @dirfiles - All the directory files to visit 
##
## SIDE EFFECTS : Changes the current work directory, temporarily
##
## DESCRIPTION  : Traverses the directory tree in pre-order, that is replaces 
##                patterns in files first in the root directory and
##                and then in the sub directories given by @dirfiles. 
##                Calls itself recursively.
##****************************************************************************
sub replRec{
    local @dirfiles=@_;
    #-- First treat the ordinary files in the root 
    local @targetFiles=<@fileARGV>;
    foreach $file (@targetFiles){
	if(-e $file){
	    &replPattern($file);
	}
    }    
    #-- ...then visit the sub directories
    foreach $dirfile (@dirfiles){
	opendir curDIR, $dirfile;
	local $oldDir=$curDir;
	$curDir.="$dirfile/";
	#print "Entering $curDir\n";
	chdir $dirfile;
	local @allCurFiles=grep !/^\.\.?$/, readdir curDIR;
	local @allCurDirFiles=grep -d, @allCurFiles;
	&replRec(@allCurDirFiles);
	chdir "..";
	$curDir=$oldDir;
	closedir curDIR;
    }
}

#*****************************************************************************
##
## FUNCTION NAME: replPattern
##
## PARAMETERS   : @file - the file to replace pattern in
##                
## DESCRIPTION  : Replaces the pattern given in $ARGV[1] by the one given 
##                in $ARGV[2], in the file given by @file.
##
##****************************************************************************
sub replPattern{
    local $theFile=$_[0];
    open (FILEH,"$theFile") or die "Error: Couldn't open $curDir$theFile\n";
    local $j=0;
    local $nbrOfRepl=0;
    local @text;
    while ($line = <FILEH>){
	$text[$j]=$line;
	$nbrOfRepl+=($text[$j]=~ s/$oldPattern/$newPattern/g);
	$j++;
    }
    if($nbrOfRepl>0){
	$tmpFile="____denna_fil_kan_bara_inte_finnas";
	if(-e $tmpFile){
	    die "Temporary filename  occupied. Please remove or rename:\n 
                         $tmpFile\n";
	}
	open (TMPFILEH,">$tmpFile") 
	    or die "Error: Couldn't open temporary file: $tmpFile\n";
	print TMPFILEH @text;
	close TMPFILEH
	    or die "Error: Coulnd't save '", $tmpFile, "'\n";
	rename $tmpFile,$theFile
	    or die "Error: Coulnd't rename $curDir$tmpFile to $curDir$theFile \n";
	unless ($quiet){
	    print "$curDir$theFile: Replaced ", $nbrOfRepl, " occurence(s).\n"
	    }
    }
    else{
	# print $curDir$theFile,": No matches.\n";
    } 
    close FILEH;
}
   
#*****************************************************************************
##
## FUNCTION NAME: usage
##
## DESCRIPTION  : Prints the usage description to standard error
##
##****************************************************************************
sub usage {
    printf(STDERR "Usage: %s file... oldpattern newpattern [-r] [-h]\n",$0);
    printf(STDERR "  -r		search sub directories recursively\n");
    printf(STDERR "  -h		display help and exit\n");
    printf(STDERR "\nExamples:\n----------\n");
    printf(STDERR "%s README INSTALL play pray\n",$0);
    printf(STDERR "%s '*.cpp' '#include iostream.hh' '#include iosteam.hpp' -r\n",$0);
    printf(STDERR "\nOBSERVE: Specifying file... with substituters without quotes gives Perl a list of matching files in the current directory rather than the substituters themselves which is probably not what you want when you use the -r option.\n");
    printf(STDERR "\nWARNING: The last 2 parameters are always concidered to be patterns. This can be secured with exclusive use of options but it makes the average usage more tedious...\n");
    exit(1);
}

main();
