/* $Id: sermon.c,v 1.12 2001/11/05 12:25:54 johana Exp $  */

#include <sys/types.h>          /* socket, bind, accept */
#include <sys/stat.h>           /* open */
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>              /* stdin, stdout, stderr */
#include <sys/ioctl.h>
#include <termios.h>
#include <stdlib.h>
#include <sys/time.h>           /* timeval */
#include <stdlib.h>
#include <string.h>             /* memset used by FD_ZERO */
#include <ctype.h>
#include <errno.h>


/* To enable or disable use of select */
#define TEST_SELECT 1


// mode settings
int hex=0;
int use_raw = 0;
int ignore_checks =0;
int exit_on_ctrl_c =1;
int boot_mode = 0;
int silent =0;
unsigned long ser_tx_cnt = 0;
unsigned long stdin_rx_cnt = 0;

unsigned char buf[10];
int size;
int hstate=0; // used to format output when in hex mode.
char hbuf[16];

/*translation from bps to Baudrate */
static int translate_speed(int bps);

struct termios org_stdin_term;

void exit_function(void)
{
  int err;
  err = tcsetattr(STDIN_FILENO, TCSANOW, &org_stdin_term);
  if (err) perror("tcsetattr stdin");
}



int poll_ser_in(int serfd)
{
//           size = read(serfd, buf, sizeof(buf)-1);
//           if(size > 0) {
  size_t cnt = 0;
  while ((size = read(serfd, buf, sizeof(buf)-1))>0) {
    buf[size]='\0';
    cnt += size;
    if (hex)
    {
      uint i=0;
      while(i<size)
      {
        printf("%.2X ",buf[i]);
        // create some space for readability
        if (hstate==7)
          printf(" ");
        hbuf[hstate]=buf[i]; // save it
        // for each 16 chars print all printable chars
        if ((hstate++)>=15) { 
          for (hstate=0;hstate<=15 ;hstate++)
          {
            if (isprint(hbuf[hstate]))
              printf("%c",hbuf[hstate]);
            else
              printf(" ");
            // create some space for readability
            if (hstate==7)
              printf("  ");
          }
          printf("\n");
          // reset hstate and continue
          hstate=0;
        }
        i++;
      }
    }
    else // not hex mode
    {
      if (ignore_checks)
      {
        write (STDOUT_FILENO,buf,size);
      }
      else 
      {   
        printf("%s",buf);
      }
    }
    fflush(stdout);
  }
  return cnt;
}


extern int optind;



int
main(int argc,char *argv[])
{
  int serfd;
  int c;
  int optleft;
  int err=0;
  struct termios t;

  speed_t speed;
  char *com_dev = "/dev/ttyS0"; 
  char *com_speed = "115200";
  char *usage = 
    "Usage: sermon [options] [device [speed]]\n"
"connects stdin and stdout to a tty as raw as possible\n"
"\n"
"  options:\n"
" general options:\n"
"  -h,-?   display this help\n"
"  -b      Serial boot mode - send 784 byte at 9600 then signal to increase\n"
" input options:\n"
"  -c      ignore ctrl-c checks on stdin\n"
"  -r      use cfmakeraw on stdin\n"
" output options:\n"
"  -s      silent, dont print any information besides serial data\n"
"  -i      ignore is_print checks (output everything)\n"
"  -x      print in hex as well\n"
"\n"
"For dual direction sniffer cable look at\n"
"https://wwwint.axis.se/h/jonasd/sermon/index.html\n";

  optind=0;
  while ((c=getopt(argc,argv,"h?brcsix"))!=-1)
  {
    switch (c)
    {
     case 'h':
     case '?':
      printf("%s",usage);
      exit(1);
      break; /* not really needed */
     case 'b':
      use_raw = 1; 
      exit_on_ctrl_c=0;
      boot_mode = 1;
      com_speed = "9600";
      break;
     case 'r':
      use_raw=1;
      break;
     case 'c':
      exit_on_ctrl_c=0;
      break;
     case 's':
      silent=1;
      break;
     case 'i':
      ignore_checks=1;
      break;
     case 'x':
      hex=1;
      break;
    }    
  }
  optleft=argc-optind;
  if (optleft>=1){
    com_dev=argv[optind++];
  }
  if (optleft>=2){
    com_speed=argv[optind++];
  }
  speed = translate_speed(atoi(com_speed));
  // dont do this unless stdin is a tty (e.g. redirects)  
  if (isatty(STDIN_FILENO))
  { 
    err = tcgetattr(STDIN_FILENO, &org_stdin_term);
    if (err) perror("tcgetattr stdin");
    atexit(exit_function);
    err = tcgetattr(STDIN_FILENO, &t);
    if (err) perror("tcgetattr stdin");
    if (use_raw)
      {
        cfmakeraw(&t);
      }
    err = tcsetattr(STDIN_FILENO, TCSANOW, &t);
    if (err) perror("tcsetattr stdin");
  }
#if TEST_SELECT
  serfd = open(com_dev, O_RDWR | O_NONBLOCK);
  /* Oct 13, 1999: O_RDWR | O_NOCTTY | O_NONBLOCK don't work on elinux
   *            but O_RDWR | O_NONBLOCK does.
    */
#else
  serfd = open(com_dev, O_RDONLY);
#endif
  
  if( (serfd==-1) || speed==0 ) 
    {
      perror(com_dev);
      printf("\n%s",usage);
      exit(1);
    }
  
  tcgetattr(serfd, &t);
  cfmakeraw(&t);
  t.c_iflag = IGNBRK | IGNPAR;
//  t.c_iflag = PARMRK | BRKINT;
  t.c_cflag |=CLOCAL;
  //  t.c_cflag = CS8 | CREAD | CLOCAL;
//  t.c_iflag = IGNBRK;
  cfsetospeed(&t, speed);
  cfsetispeed(&t, speed);
  tcsetattr(serfd, TCSANOW, &t);
  if (!silent)
  {
    printf("Sniffing %s at %d Baud\r\n", com_dev, atoi(com_speed));
  }
  {
    int quit = 0;
    fd_set readfds;
    int selcode;
    struct timeval timeout;
    while (!quit)
    {
      FD_ZERO(&readfds);
      FD_SET(serfd, &readfds);
      FD_SET(STDIN_FILENO, &readfds);
      timeout.tv_sec=5;
      timeout.tv_usec=0;
      if ((selcode = select(serfd + 1,
                            &readfds,
                            NULL,
                            NULL,
                            &timeout)) != 0)
      {
        /* Something happened */
        if (selcode == -1)
        {
          perror("select(): ");
        }
        else
        {
          if (FD_ISSET(serfd, &readfds)) // serial data
          {
            poll_ser_in(serfd);

          }
          if (FD_ISSET(STDIN_FILENO, &readfds)) // stdin data
          {
            size_t to_read = sizeof(buf)-1;
            
            if (boot_mode && stdin_rx_cnt < 784)
            {
              size_t max_to_read = 784 - stdin_rx_cnt;
              if (to_read > max_to_read)
              {
                to_read = max_to_read;
              }
            }
            size = read(STDIN_FILENO, buf, to_read);
            if(size > 0) {
             
              stdin_rx_cnt += size;
              buf[size]='\0';
              if (exit_on_ctrl_c &&
                  (size == 1)  && 
                  (buf[0] == 0x03) /* Ctrl+C */){
                printf("Ctrl+C detected, exiting...\r\n");
                quit = 1;
              }
              else
              {
                ssize_t tot_written = 0;
                ssize_t written;
                while (tot_written < size){
                  written = write(serfd, &buf[tot_written], size-tot_written);
                  if (written > 0){
                    ser_tx_cnt += written;
                    tot_written += written;
#if 0                    
                    if (boot_mode && !silent)
                      printf(".");
#endif                    
                  }
                }
              }
              if (boot_mode && (stdin_rx_cnt == 784))
              {
                /* We have now written the 784 serial boot bytes. */
                struct termios ti;
                struct timeval tv0, tv1;
                tcdrain(serfd);
                /* Change baudrate to 115200 */
                tcgetattr(serfd, &ti);
                cfsetospeed(&ti, B115200);
                cfsetispeed(&ti, B115200);
#if 1 /* This handle kernel driver bug.. */
                /* Be paranoid and wait a little before changing baudrate */
                /* 10/9600 = 1042 us */
                gettimeofday(&tv0, NULL);
                do
                {
                  poll_ser_in(serfd);
                  gettimeofday(&tv1, NULL);
                }while( (tv1.tv_sec-tv0.tv_sec)*1000000+tv1.tv_usec - tv0.tv_usec < 2000);
#endif
                tcsetattr(serfd, TCSADRAIN, &ti);
                if (!silent)
                {
                  printf("Changed baudrate\n");
                }
                /* Wait a little before sending.. */
                gettimeofday(&tv0, NULL);
                do
                {
                  poll_ser_in(serfd);
                  gettimeofday(&tv1, NULL);
                }while( (tv1.tv_sec-tv0.tv_sec)*1000000+tv1.tv_usec - tv0.tv_usec < 400000);
                printf("Sending...\n");

              }
            }
            else
            {
              /* Error or EOF, let's quit */ 
              quit = 1;
            }
          }
        }
      }
    } /* while */
  }
  if (!silent)
  {
    printf("\nWrote %lu bytes to serial port.\n",ser_tx_cnt);
    if (boot_mode)
    {
      /* Stay in receive mode until nothing received for 4 seconds */
      struct timeval tv0, tv1,tv2;
      size_t cnt;
      gettimeofday(&tv0, NULL);
      gettimeofday(&tv2, NULL);
      do
      {
        cnt = poll_ser_in(serfd);
        gettimeofday(&tv1, NULL);
        if (cnt)
        {
          tv0.tv_sec = tv1.tv_sec;
        }
        if (tv1.tv_sec != tv2.tv_sec )
        {
          tv2 = tv1;
        }
      }while((tv1.tv_sec - tv0.tv_sec) < 4);
    }
  }
  
  close(serfd);
  if (isatty(STDIN_FILENO)){
    tcsetattr(STDIN_FILENO, TCSANOW, &org_stdin_term);
    if (err) perror("tcsetattr stdin");   
  }

  return 0;
}

/*
 * List of valid speeds.
 */

struct speed {
   int speed_int, speed_val;
} speeds[] = {
#ifdef B50
     { 50, B50 },
#endif
#ifdef B75
     { 75, B75 },
#endif
#ifdef B110
     { 110, B110 },
#endif
#ifdef B134
     { 134, B134 },
#endif
#ifdef B150
     { 150, B150 },
#endif
#ifdef B200
     { 200, B200 },
#endif
#ifdef B300
     { 300, B300 },
#endif
#ifdef B600
     { 600, B600 },
#endif
#ifdef B1200
     { 1200, B1200 },
#endif
#ifdef B1800
     { 1800, B1800 },
#endif
#ifdef B2000
     { 2000, B2000 },
#endif
#ifdef B2400
     { 2400, B2400 },
#endif
#ifdef B3600
     { 3600, B3600 },
#endif
#ifdef B4800
     { 4800, B4800 },
#endif
#ifdef B7200
     { 7200, B7200 },
#endif
#ifdef B9600
     { 9600, B9600 },
#endif
#ifdef B19200
     { 19200, B19200 },
#endif
#ifdef B38400
     { 38400, B38400 },
#endif
#ifdef B57600
     { 57600, B57600 },
#endif
#ifdef B115200
     { 115200, B115200 },
#endif
#ifdef EXTA
     { 19200, EXTA },
#endif
#ifdef EXTB
     { 38400, EXTB },
#endif
#ifdef B230400
     { 230400, B230400 },
#endif
#ifdef B460800
     { 460800, B460800 },
#endif
     { 0, 0 }
};

/********************************************************************
 *
 * Translate from bits/second to a speed_t.
 */

static int translate_speed (int bps)
{
   struct speed *speedp;
   
   if (bps != 0)
   {
     for (speedp = speeds; speedp->speed_int; speedp++)
     {
       if (bps == speedp->speed_int)
       {
         return speedp->speed_val;
       }
     }
     printf("speed %d not supported\n", bps);
   }
   return 0;
}
