/* rxtxtest.c
 * $Id: rxtxtest.c,v 1.3 2002/06/03 13:00:21 johana Exp $
 * Copyright (C) 2002 Axis Communication AB
 * Author: Mikael Starvik
 * Errorhandling improved by Johan Adolfsson
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>


static int data_connect(char* host, int port);
static void receive_data(int fd, int len);
static void send_data(int fd, char* dat, int len);

#define DEBUG(x) x

int main(int argc, char* argv[])
{
  int read_len, write_len;
  char* test_data;
  int i;
  unsigned char checksum = 0;
  int fd;
  
  if (argc != 5)
  {
    printf("Usage: rxtxtest address port receivesize sendsize\n");
    printf("If sendsize > receivesize we start by sending\n");
    exit(1);
  }

  fd = data_connect(argv[1], atoi(argv[2]));
  if (fd < 0)
  {
    perror("connect failed");
    exit(1);
  }
  read_len = 1+atoi(argv[3]);
  write_len = 1+atoi(argv[4]);
  
  test_data = (char *)malloc(write_len);
  for (i = 0; i < write_len-1; i++)
  {
    test_data[i] = i % 256;
    checksum += test_data[i];
  }
  
  test_data[write_len-1] = -checksum;

  if (write_len > read_len)
  {
    send_data(fd, test_data, write_len);
  }

  while(1)
  {
    receive_data(fd, read_len);
    send_data(fd, test_data, write_len);
  }
}

static int data_connect(char* host, int port)
{
  int s = -1;
  struct sockaddr_in server;  

  memset((char *) &server, 0, sizeof (server));
  server.sin_family = AF_INET;
  server.sin_addr.s_addr = inet_addr(host);
  if (server.sin_addr.s_addr == INADDR_NONE)
  {
    struct hostent *hp; 
    
    hp = gethostbyname(host);
    if (hp != NULL)
    {
      memcpy( (char *) &server.sin_addr, hp->h_addr, hp->h_length);
      server.sin_family = hp->h_addrtype;    /* Protocol family */
    }
    else 
      return -1;
  }
  
  server.sin_port = htons(port);/* Port number */
  s = socket(AF_INET, SOCK_STREAM, 0);
  if (s < 0)
    return -1;
  if ( connect(s, (struct sockaddr *) & server, sizeof server) < 0)
  {
    return -1;
  }
  return s;
}

static void send_data(int fd, char* data, int len)
{
  int written = 0;
  int cnt;
  
  DEBUG(printf("Send total %d\n", len));
  
  while (written < len)
  {
    cnt = write(fd, data + written, len - written);
    if (cnt > 0)
    {
      written += cnt;
    }
    else
    {
      perror("write():");
    }
    DEBUG(printf("Sent: %d\n", written));
  }

  DEBUG(printf("Sent %d bytes\n", len));
}

static void receive_data(int fd, int len)
{
  unsigned char* buf = malloc(len);
  int data_read = 0;
  int cnt;
  int i;
  int err_cnt = 0;
  unsigned char checksum = 0;
  fd_set rdfs;
  struct timeval tv;
  int retval;
  
  FD_ZERO(&rdfs);
  FD_SET(fd, &rdfs);
  tv.tv_sec = 5;
  tv.tv_usec = 0;
  
  DEBUG(printf("Read total %d\n", len));
  
  while (data_read < len)
  {
    retval = select(fd + 1, &rdfs, NULL, NULL, &tv);
    if (retval)
    {
      cnt = read(fd, buf + data_read, len - data_read);
      if (cnt >=0)
      {
        data_read += cnt;  
      }
      else
      {
        perror("read():");
      }
    }
    else
    {
      DEBUG(printf("Timeout %d read\n", data_read));
      break;
    }
    
    DEBUG(printf("Read: %d\n", data_read));
  }

  for (i = 0; i < data_read-1; i++)
  {
    checksum += buf[i];
    if (buf[i] != (unsigned char)(i&0xFF)){
      err_cnt++;
      if (err_cnt < 10)
      {
        printf("Error: byte %i was 0x%02X should be 0x%02X\n",
               i, buf[i], i&0xFF);
      }
    }
  }
  checksum += buf[i];
  
  printf("checksum 0x%02X, err_cnt %i\n", checksum, err_cnt);
  
  if (checksum != 0)
  {
    printf("Checksum error\n");
    for (i = 0; i < data_read; i++)
    {
      printf("%02X ", buf[i]);
      if (!((i+1)%16))
      {
        printf("\n");
      }
    }
    printf("\n");
    exit(1);
  }

  DEBUG(printf("Read %d bytes\n", data_read));
}
