//  tcp_perf.c
//
//  TCP performance bechmark utility
//	Written by  Zoran Tomicic <zoran@microtrol.com.au>
//
//	Portions of code taken from 
//  	tcptest.c
//  	$Id: tcptest.c
//  	TCP client and server test program 
//  	Copyright (C) Johan Adolfsson <Johan.Adolfsson@axis.com>
//
//
//	 compile for Linux :  gcc -o tcp_perf tcp_perf.c
//	 compile for Win32:   cl   -DWIN32 tcp_perf.c 
//


#ifdef WIN32

#include <winsock.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>


#define close(s) closesocket(s)

#pragma comment(lib, "ws2_32.lib")

#define EWOULDBLOCK	WSAEWOULDBLOCK

#else /* WIN32 */

#include <sys/types.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <sys/un.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <netinet/in.h>  /* IPPROTO_TCP */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/version.h>

#include <string.h>
#include <netdb.h>
#include <errno.h>
#include <unistd.h> /* close */

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>
#include <assert.h>


typedef struct
{
	unsigned User;
	unsigned System;
	unsigned Nice;
	unsigned Idle;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
	unsigned Iowait;
	unsigned IRQ;
	unsigned SoftIRQ;
#endif
} SYSTEM_STAT;

static int GetSysStat(SYSTEM_STAT * pInfo)
{
    char buf [2048], *s;
  	FILE *fp;
	const char *szFileName = "/proc/stat";
	char dummy[50];
  
  	if (NULL == pInfo) 
  	{
    	return -1;
  	}

  	if ((fp = fopen (szFileName, "r")) == NULL) 
  	{
    	return (-1);
  	} 
  
  	if ((s = fgets (buf, 2048, fp)) == NULL) 
  	{
    	fclose (fp);
    	return (-1);
  	}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
  	sscanf (buf, "%s %u %u %u %u", 
			dummy,
  			&(pInfo->User),
  			&(pInfo->Nice),
  			&(pInfo->System),
  			&(pInfo->Idle) );
#else
	sscanf (buf, "%s %u %u %u %u %u %u %u", 
			dummy,
  			&(pInfo->User),
  			&(pInfo->Nice),
  			&(pInfo->System),
  			&(pInfo->Idle),
			&(pInfo->Iowait),
			&(pInfo->IRQ), 
			&(pInfo->SoftIRQ) );
#endif
  	fclose (fp);
  	return 0;
}

#endif 

unsigned GetJiffies(void)
{
#ifdef WIN32
	return GetTickCount()/10;
#else
	SYSTEM_STAT s;
	GetSysStat (&s);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
	return s.User+s.System+s.Idle;
#else
	return s.User+s.System+s.Idle+s.Iowait+s.IRQ+s.SoftIRQ;
#endif
#endif	
}	


int SetNonBlock(int s, int On)
{
	long OnFlag = On;
#ifdef WIN32
	return ioctlsocket(s, FIONBIO, &OnFlag);
#else
	return ioctl(s, FIONBIO, &OnFlag);
#endif	
}	

int GetError(void)
{
	int err;
#ifdef WIN32
	err = WSAGetLastError ();
#else
	err = errno;
#endif	
	return err;
}	


#define MAX_UDP_LEN	1400

static int SockInitFlag;

void SocketCleanup()
{
#ifdef WIN32
	if (SockInitFlag)
		WSACleanup( );
#endif
}

static void Fatal(const char* str)
{
	printf("Fatal error: %s\n", str);
	SocketCleanup();
	exit(-1);
}

void SocketInit()
{
#ifdef WIN32
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
	wVersionRequested = MAKEWORD( 1, 1 );
	err = WSAStartup( wVersionRequested, &wsaData );
	
	if ( err != 0 ) 
	{
    /* Tell the user that we could not find a usable */
    /* WinSock DLL.                                  */    
		Fatal("Failed to init winsock\n");		
	}
#endif
	SockInitFlag = 1;
}

static int Connect(const char* host, unsigned short portnr)
{
	int s = -1;
	struct sockaddr_in server;  

	memset((char *) &server, 0, sizeof (server));
    server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(host);
    if (server.sin_addr.s_addr == INADDR_NONE)
	{
		struct hostent *hp; 

		hp = gethostbyname(host);
		if (hp != NULL)
		{
			memcpy( (char *) &server.sin_addr, hp->h_addr, hp->h_length);
			server.sin_family = hp->h_addrtype;    /* Protocol family */
		}
		else 
			return -1;
	}

	server.sin_port = htons(portnr);/* Port number */
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s < 0)
		return -1;
	if ( connect(s, (struct sockaddr *) & server, sizeof server) < 0)
	{
		return -1;
	}
	return s;
}

static int Listen( unsigned short Port)
{
	int s = 0;

	struct sockaddr_in sa_in;

	memset(&sa_in, 0, sizeof (sa_in));
	s = socket(AF_INET, SOCK_STREAM, 0);  
	sa_in.sin_port=htons(Port);

	sa_in.sin_family = AF_INET;

	/* Bind to any port and IP address */  
	if (bind(s, (struct sockaddr *) & sa_in, sizeof sa_in) < 0)
	{
		printf("error %d in bind (%s)\n", errno, strerror(errno));
	}

	/* Accept any client */
	listen(s, 1);  
	return s;
}

static int UdpBind( unsigned short Port)
{
	struct sockaddr_in sa_in;
	int s = 0;

	s = socket(AF_INET, SOCK_DGRAM, 0);  
	memset(&sa_in, 0, sizeof (sa_in));
	sa_in.sin_port=htons(Port);

	sa_in.sin_family = AF_INET;

	/* Bind to any port and IP address */  
	if (bind(s, (struct sockaddr *) & sa_in, sizeof (sa_in)) < 0)
	{
		printf("error %d in bind (%s)\n", errno, strerror(errno));
	}
	return s;
}


static int Accept( int SrvSock)
{
  	int g = 0;
  	int len;
  
  	struct sockaddr_in from;
  
  	len = sizeof (from);          
  	memset(&from, 0, len);  
  	g = accept(SrvSock, (struct sockaddr *) & from, &len);
  	return g;
}	

static int SockRead(int Sock, void* Buf, unsigned Count)
{
	// Block until Count bytes from Sock are read into the Buf
	// or until error is encountered
	// Returns actual number of recv() calls executed

	int nCalls = 0;
	char* p = (char*) Buf;
	while (Count)
	{
		int r = recv(Sock, p, Count,0);
		if (r <= 0)
			return r;
		p += r;
		Count -= r;
		nCalls++;
	}
	return nCalls; // Ok
}	

static int SockWrite(int Sock, void* Buf, unsigned Count)
{
	// Block until Count bytes from Buf are transferred to the Sock
	// or until error is encountered
	// Returns actual number of send() calls executed

	char* p = (char*) Buf;
	int nCalls = 0;
	while (Count)
	{
		int r = send(Sock, p, Count,0);
		if (r <= 0)
			return r;
		p += r;
		Count -= r;
		nCalls++;
	}
	return nCalls; // Ok
}	

#ifndef WIN32

void MakeSysLoadString(SYSTEM_STAT* Before, char* LoadStr)
{
// Calculate system load from Before up to Now

	unsigned TimeUser, TimeSys, TimeIdle, Load, TotalTime, uLoad, sLoad;
	SYSTEM_STAT Now;

	GetSysStat (&Now);
	TimeUser = Now.User   - Before->User;
	TimeSys  = Now.System - Before->System;
	TimeIdle = Now.Idle   - Before->Idle;
	TotalTime = TimeSys+TimeUser+TimeIdle;

	if (TotalTime == 0)
	{
		uLoad = 0;
		sLoad = 0;
		Load = 0;
	}
	else
	{
		Load  = ((TimeUser + TimeSys)* 100)/ TotalTime;
		sLoad = (TimeSys*100)/TotalTime;
		uLoad = Load-sLoad;
	}
	// format system load in %  
	sprintf(LoadStr,", average CPU load during transfer %u%% (S:%d%% U:%d%%)", 
	                   Load, sLoad, uLoad);
}	

#endif

//---------------------------------------------------------------

typedef struct
{
   unsigned MaxPacket;	
   unsigned CurrPacket;	
} UDP_PKT_HEADER;	

#define TOTAL_PACKET_LEN  (MAX_UDP_LEN+sizeof(UDP_PKT_HEADER))

static void UdpServer(unsigned short Port)
{
	unsigned Total;
	unsigned MaxPacket;
	unsigned CurrPacket;
	unsigned Delta;

	char LoadStr[100];

	unsigned  Start, End;
	char* UdpBuf;
	int SrvSock;
	int PacketLen;
	UDP_PKT_HEADER* pHdr;
	struct sockaddr_in Remote;
	int LenRemote;

#ifndef WIN32
	SYSTEM_STAT Before;
#endif

	UdpBuf = malloc(TOTAL_PACKET_LEN);
	if (!UdpBuf)
		Fatal("Could not allocate UDP recv buffer");

	SrvSock = UdpBind(Port);
	if (SrvSock < 0)
		Fatal("Could not create UDP server socket");
	for (;;)
	{
		Total = 0;
		MaxPacket  = 0;
		CurrPacket = 0;

		// first 8 bytes Udp packet contain the CurrPacket & MaxPacket,
		// encoded  in network byte order
		LenRemote = sizeof(Remote);
		PacketLen = recvfrom(SrvSock, UdpBuf, TOTAL_PACKET_LEN, 0, (struct sockaddr*) &Remote, &LenRemote);
		if (PacketLen <= 0)
		{
			close(SrvSock);
			Fatal("Start UDP packet receive error\n");
		}

		PacketLen -= sizeof(UDP_PKT_HEADER);
		pHdr = (UDP_PKT_HEADER*) UdpBuf;
		MaxPacket  = ntohl(pHdr->MaxPacket);
		CurrPacket = ntohl(pHdr->CurrPacket);

		Total += PacketLen;

#ifndef WIN32
		GetSysStat (&Before);
#endif
		Start = GetJiffies();

		while (CurrPacket <  MaxPacket)
		{
			PacketLen = recvfrom(SrvSock, UdpBuf, TOTAL_PACKET_LEN, 0, 0, 0);
			if (PacketLen <= 0)
			{
				close(SrvSock);
				Fatal("Start UDP packet receive error\n");
			}
			pHdr = (UDP_PKT_HEADER*) UdpBuf;
			if (ntohl(pHdr->MaxPacket) != MaxPacket)
			{
				close(SrvSock);
				Fatal("Bad UDP packet header\n");
			}
			if (ntohl(pHdr->CurrPacket)-1 != CurrPacket)
			{
				char s[100];
				sprintf(s,"Packet lost: expected %d, got %d", 
						    CurrPacket+1,
						    ntohl(pHdr->CurrPacket));
				close(SrvSock);
				Fatal(s);
			}
			CurrPacket = ntohl(pHdr->CurrPacket);

			if ((CurrPacket % 10) == 0)
			{
				// send the handshake packet every 10 packets
				sendto(SrvSock, UdpBuf, sizeof(UDP_PKT_HEADER), 0, 
		                   (struct sockaddr*) &Remote, sizeof(Remote));
			}

			PacketLen -= sizeof(UDP_PKT_HEADER);
			Total += PacketLen;
		}

		End = GetJiffies();
#ifndef WIN32
		MakeSysLoadString(&Before, LoadStr);
#else
		LoadStr[0] = 0;
#endif
	
		Delta = End-Start;
		if (Delta)
			printf("Time to receive %u bytes was %u msec (%u bytes/sec)\n%d packets received%s\n",
			   	  Total, Delta*10, (Total/Delta)*100, MaxPacket, LoadStr);
		else
			printf("Time difference too small, increase the <bytes to send> parameter\n");
	}
}	


static void UdpClient(const char* Host, unsigned short Port, unsigned MsgSize, unsigned BufSize)
{
	unsigned MaxPacket;
	unsigned CurrPacket;
	unsigned PacketLen;
	unsigned Sent;
	unsigned Total;
	unsigned MsgLen;
	UDP_PKT_HEADER* pHdr;
	struct sockaddr_in server;  
	char* UdpBuf;
	int   Sock;

	UdpBuf = malloc(MAX_UDP_LEN+8);
	if (!UdpBuf)
		Fatal("Could not allocate UDP send buffer");

	Sock = UdpBind(Port);
	if (Sock < 0)
		Fatal("Could not create UDP client socket");

	SetNonBlock(Sock, 1);	

	memset((char *) &server, 0, sizeof (server));
    server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(Host);
    if (server.sin_addr.s_addr == INADDR_NONE)
	{
		struct hostent *hp; 

		hp = gethostbyname(Host);
		if (hp != NULL)
		{
			memcpy( (char *) &server.sin_addr, hp->h_addr, hp->h_length);
			server.sin_family = hp->h_addrtype;    /* Protocol family */
		}
		else
			Fatal("Unknown host"); 
	}
	server.sin_port=htons(Port);

	pHdr = (UDP_PKT_HEADER*) UdpBuf;
	MsgLen = MsgSize;
	Total = 0;
	MaxPacket  = MsgLen/BufSize;
	if (MsgLen % BufSize)
	{
		MaxPacket++;
	}

	for (CurrPacket = 1;CurrPacket <= MaxPacket; CurrPacket++)
	{
		// first 8 bytes Udp packet contain the CurrPacket & MaxPacket,
		// encoded  in network byte order
		pHdr->MaxPacket  = htonl(MaxPacket);
		pHdr->CurrPacket = htonl(CurrPacket);

		if (MsgLen > BufSize)
			PacketLen  = BufSize+ sizeof(UDP_PKT_HEADER);
		else
			PacketLen  = MsgLen+ sizeof(UDP_PKT_HEADER);

		do 
		{
			Sent = sendto(Sock, UdpBuf, PacketLen, 0, 
		                   (struct sockaddr*) &server, sizeof(server));
		}  while (Sent < 0 && GetError() == EWOULDBLOCK);

		if (Sent <= 0)
		{
			close(Sock);
			Fatal("UDP packet send error\n");
		}
		Total  += (PacketLen - sizeof(UDP_PKT_HEADER));
		MsgLen -= (PacketLen - sizeof(UDP_PKT_HEADER));

		if ((CurrPacket % 10) == 0)
		{
			UDP_PKT_HEADER Hdr;
			int r;
			do 
			{
				r = recvfrom (Sock, (char*) &Hdr, sizeof(Hdr), 0, 0, 0);
			}  while (r < 0 && GetError() == EWOULDBLOCK);

			if (r <= 0)
				Fatal("Error on handshake packet receive");

			if (ntohl(Hdr.MaxPacket) != MaxPacket || ntohl(Hdr.CurrPacket) != CurrPacket)
				Fatal("Bad handshake packet");
		}
	}

	free(UdpBuf);
	close(Sock);
	printf("%d bytes sent to %s in %d packets\n", Total, Host, MaxPacket);
}	

static void TcpServer(unsigned short Port, unsigned TcpBufSize, int continuous)
{
	int ClientConn;
	unsigned StartMsgLen;
	unsigned MsgLen;
	unsigned NetwMsgLen;
	int s;
	int error;
	unsigned Start, End;
	char* TcpBuf;
	int SrvSock;
	int n;

#ifndef WIN32
	SYSTEM_STAT Before;
#endif

	TcpBuf = malloc(TcpBufSize);
	if (!TcpBuf)
		Fatal("Could not allocate recv buffer");

	SrvSock= Listen(Port);
	if (SrvSock < 0)
		Fatal("Could not create server socket");
	do
	{
		printf("Waiting for client connection\n");
		ClientConn = Accept(SrvSock);
		if (ClientConn < 0)
			continue;

		printf("Accepted client connection\n");

		// first 4 bytes of the message contain the length of the data to follow
		// encoded  in network byte order
		s = SockRead(ClientConn, &NetwMsgLen, 4);
		if (s <= 0)
		{
			close(ClientConn);
			printf("MsgLen receive error\n");
			continue;
		}
		error = 0;
		n = 0;

#ifndef WIN32
		GetSysStat (&Before);
#endif
		
		Start = GetJiffies();
		StartMsgLen = ntohl(NetwMsgLen);

		MsgLen = StartMsgLen;
		while (MsgLen > TcpBufSize)
		{
			s = SockRead(ClientConn, TcpBuf, TcpBufSize);
			if ( s <= 0)
			{
				error = 1;
				break;
			}
			MsgLen -= TcpBufSize;
			n += s;
		}

		if (!error && MsgLen)
		{
			s = SockRead(ClientConn, TcpBuf, MsgLen);
			if ( s <= 0)
				error = 1;
			else
				n += s;
		}

		if (!error)
		{
			unsigned Delta;
			char LoadStr[100];

			LoadStr[0] = 0;

			End = GetJiffies();

#ifndef WIN32
			MakeSysLoadString(&Before, LoadStr);
#else
			LoadStr[0] = 0;
#endif
			Delta = End-Start;
			if (Delta)
				printf("Time to receive %u bytes was %u msec (%u bytes/sec)\n%d recv() calls made%s\n",
				   	  StartMsgLen, Delta*10, (StartMsgLen/Delta)*100,n, LoadStr);
			else
				printf("Time difference too small, increase the <bytes to send> parameter\n");
		}
		else
			printf("Receive error\n");
		
		close(ClientConn);
	} while (continuous);
}	

static void TcpClient(const char* Host, unsigned short Port, unsigned MsgSize, unsigned TcpBufSize)
{
	int      Sock;
	unsigned MsgLen;
	unsigned NetwMsgLen;
	int s;
	unsigned i;
	int error;
	char* TcpBuf;
	int n = 0;

	TcpBuf = malloc(TcpBufSize);
	if (!TcpBuf)
		Fatal("Could not allocate send buffer");
	for (i = 0; i < TcpBufSize; i++)
		TcpBuf[i] = (char) i;

	printf("Connecting to %s:%u...\n", Host, Port);
	Sock= Connect(Host,Port);
	if (Sock < 0)
		Fatal("Could not connect to server");

	printf("Connected to %s!\n", Host);
	MsgLen = MsgSize;
	NetwMsgLen = htonl(MsgLen);		// convert message length to network byte order

	for (i = 0; i < TcpBufSize; i++)
		TcpBuf[i] = (char) i;

	s = SockWrite(Sock, &NetwMsgLen, 4);
	if (s <= 0)
	{
		Fatal("Could not send the message length");
	}
	error = 0;

	while (MsgLen > TcpBufSize)
	{
		s = SockWrite(Sock, TcpBuf, TcpBufSize);
		if ( s <= 0)
		{
			error = 1;
			break;
		}
		MsgLen -= TcpBufSize;
		n += s;
	}

	if (!error && MsgLen)
	{
		s = SockWrite(Sock, TcpBuf, MsgLen);
		if ( s <= 0)
			error = 1;
		else
			n += s;
	}
	close(Sock);

	if (!error)
		printf("%d bytes sent to %s  %d send() calls made\n", MsgSize, Host,n);
	else
		Fatal("Send failed\n");
}	

static void	MemTest(unsigned Size, unsigned Repeat)
{
	void * pSrc;
	void * pDest;
	unsigned Start, End;
	unsigned Total,Delta;

	Total = Size*Repeat;

	pSrc = malloc(Size);
	if (!pSrc)
		Fatal("Source buffer malloc failed");
	pDest = malloc(Size);
	if (!pDest)
		Fatal("Destination buffer malloc failed");

	printf("Copy %u blocks of %u bytes ...\n", Repeat, Size);
	Start = GetJiffies();
	while (Repeat--)
	{
		memcpy(pDest,pSrc, Size);
	}
	End = GetJiffies();

	free(pSrc);
	free(pDest);
	
	Delta = End-Start;
	if (Delta)
		printf("Time to copy %u bytes was %u msec (%u bytes/sec)\n",
		   	    Total, Delta*10, (Total/Delta)*100);
	else
		printf("Time difference too small, increase the <repeat> parameter\n");
}	


static void	FillTest(unsigned Size, unsigned Repeat)
{
	void * pDest;
	unsigned Start, End;
	unsigned Total,Delta;

	Total = Size*Repeat;

	pDest = malloc(Size);
	if (!pDest)
		Fatal("Destination buffer malloc failed");

	printf("Fill %u blocks of %u bytes ...\n", Repeat, Size);
	Start = GetJiffies();
	while (Repeat--)
	{
		memset(pDest,0x5a, Size);
	}
	End = GetJiffies();
	free(pDest);
	
	Delta = End-Start;
	if (Delta)
		printf("Time to fill %u bytes was %u msec (%u bytes/sec)\n",
		   	    Total, Delta*10, (Total/Delta)*100);
	else
		printf("Time difference too small, increase the <repeat> parameter\n");
}	


static void Usage(void)
{
	printf("usage for TCP server(receiver) : tcp_perf  -s <port> <buffer size>\n");
	printf("                                       use -so instead of -s to run once\n");
	printf("usage for TCP client(sender)   : tcp_perf  -c <host> <port> <bytes to send> <buffer size>\n");
	printf("usage for UDP server(receiver) : tcp_perf  -us <port>\n");
	printf("usage for UDP client(sender)   : tcp_perf  -uc <host> <port> <bytes to send> <UDP buffer size>\n");
	printf("                                            <UDP buffer size> may not be over %d\n", MAX_UDP_LEN);
	printf("usage for memory copy speed test: tcp_perf  -m <buffer size> <repeat>\n");
	printf("usage for memory fill speed test: tcp_perf  -f <buffer size> <repeat>\n");
}	

int main(int argc, char **argv)
{
	unsigned short Port;
	char* Host;
	unsigned MsgSize;
	unsigned BufSize;
	int cont;

	SocketInit();
	switch(argc)
	{
		case 6:
			if (!strcmp(argv[1], "-c"))
			{
				Host =argv[2];
				Port = atoi(argv[3]);
				MsgSize = atoi(argv[4]);
				BufSize = atoi(argv[5]);
				TcpClient(Host,Port,MsgSize, BufSize);		
			}
			else if (!strcmp(argv[1], "-uc"))
			{
				Host =argv[2];
				Port = atoi(argv[3]);
				MsgSize = atoi(argv[4]);
				BufSize = atoi(argv[5]);
				if (BufSize > MAX_UDP_LEN)
					Usage();
				else
					UdpClient(Host,Port,MsgSize, BufSize);		
			}
			else
				Usage();
			break;

		case 4:
			if (!strcmp(argv[1], "-s"))
			{
				Port    = atoi(argv[2]);
				BufSize = atoi(argv[3]);
				TcpServer(Port,BufSize,1);		
			}
			if (!strcmp(argv[1], "-so"))
			{
				Port    = atoi(argv[2]);
				BufSize = atoi(argv[3]);
				TcpServer(Port,BufSize,0);
			}
			else if (!strcmp(argv[1], "-m"))
			{
				unsigned MemSize;
				unsigned Repeat;
				
				MemSize = atoi(argv[2]);
				Repeat  = atoi(argv[3]);
				MemTest(MemSize, Repeat);
			}
			else if (!strcmp(argv[1], "-f"))
			{
				unsigned FillSize;
				unsigned Repeat;
				
				FillSize = atoi(argv[2]);
				Repeat  = atoi(argv[3]);
				FillTest(FillSize, Repeat);
			}
			else
				Usage();
			break;

		case 3:
			if (!strcmp(argv[1], "-us"))
			{
				Port    = atoi(argv[2]);
				UdpServer(Port);		
			}
			else
				Usage();
			break;

		default:
			Usage();
			break;
	}

	SocketCleanup();
	return 0;
}	
