/* tcptest.c
  $Id: tcptest.c,v 1.6 2002/02/25 13:55:31 johana Exp $
   TCP client and server test program 
   Copyright (C) Johan Adolfsson <Johan.Adolfsson@axis.com>

Feb 10, 2000 Johan Adolfsson  Fixed write() returning -1, added SO_LINGER

Mar 13 2000  Johan Adolfsson  Included changes by Magnus Wennergren:
   99-06-29:
   Magnus Wennergren
   Added Windows console app compability.
   Link with ws2_32.lib / ws2_32.dll (winsock 2.x binary)

 */

#ifdef WIN32

#include <windows.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <fcntl.h>
#include <time.h>

#pragma warning( disable: 4101 4244 4018 4716 4133

#define sleep(s) Sleep(s)
#define bzero(mem, sz) ZeroMemory(mem, sz)
#define bcopy(dest, source, length) CopyMemory(dest,source,length)
#define write(a, b, c) send(a,b,c,0)
#define close(s) closesocket(s)

HANDLE hStdin;
DWORD fdwSaveOldMode;

char pending_ch;
unsigned pending_ch_count;

#define DISABLED_INPUT_MODES \
	(ENABLE_PROCESSED_INPUT | \
	ENABLE_LINE_INPUT | \
	ENABLE_MOUSE_INPUT | \
	ENABLE_WINDOW_INPUT | \
	ENABLE_ECHO_INPUT)

#else /* WIN32 */

#include <sys/types.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <sys/un.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <netinet/in.h>  /* IPPROTO_TCP */


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <string.h>

#include <netdb.h>
#include <errno.h>
#include <unistd.h> /* close */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>
#include <assert.h>
#include <time.h>

#endif /* else WIN32 */

#ifndef MIN
#define MIN(a,b) (a<b)?a:b
#endif

#define FALSE 0
#define TRUE 1
typedef int bool;


char* hostname = "171.16.3.157";
char* inputfilename = NULL;

char relay_hostname[100] = "nean.lfv.se";
unsigned short relay_portnr = 32321;
bool do_relay = FALSE;




char* outputfilename = "/dev/null";
int echo_testpattern = 0;
int echo_rxdata = 0;

unsigned long ctestsize = 0; /* client test size */
unsigned long stestsize = 0; /* server test size */
unsigned long buf_size = 1024;

int telnet_test = 0;
int telnet_test2 = 0;

long loopback_time_ms = -1; /* -1 = No loopback */
unsigned long error_cnt = 0;


u_short portnr = 2001;
u_short listener_portnr = 2001;



unsigned long int glob_wait = 0;
unsigned int listener_type = SOCK_STREAM;
unsigned int client_type = SOCK_STREAM;
unsigned long close_interval = 0;


#define QUIT_CHAR 0x04


int client_server_app(void);
int read_inputfile_app(void);
int testpattern_app(void);
int relay_app(void);
int telnet_test_app(void);
int telnet_test_app2(void);

/* ----------------------------------------------------------------------- */

#ifdef WIN32

void my_error_exit(const char* str)
{
	printf("%s\n", str);
	exit(-1);
}

void socket_init()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
	wVersionRequested = MAKEWORD( 1, 1 );
	err = WSAStartup( wVersionRequested, &wsaData );
	
	if ( err != 0 ) 
	{
    /* Tell the user that we could not find a usable */
    /* WinSock DLL.                                  */    
		my_error_exit("Failed to init winsock\n");		
	}
}

void socket_done()
{
	WSACleanup( );
}

void keyb_init()
{
    DWORD fdwMode;
	
	hStdin = GetStdHandle(STD_INPUT_HANDLE); 
    if (hStdin == INVALID_HANDLE_VALUE) 
        my_error_exit("GetStdHandle"); 
 
    /* Save the current input mode, to be restored on exit. */
 
    if (! GetConsoleMode(hStdin, &fdwSaveOldMode) ) 
        my_error_exit("GetConsoleMode"); 

	fdwMode = fdwSaveOldMode & ~DISABLED_INPUT_MODES;

    if (! SetConsoleMode(hStdin, fdwMode) ) 
        my_error_exit("SetConsoleMode"); 
}

void keyb_done()
{
	SetConsoleMode(hStdin, fdwSaveOldMode);
}

void get_pending_chars()
{
/* Wait for input events. */
	
	if( WaitForSingleObject(hStdin, 10) == WAIT_OBJECT_0 )
	{
		INPUT_RECORD input;
		unsigned events_read;

		while( GetNumberOfConsoleInputEvents(
			hStdin, &events_read) && events_read)
		{
			ReadConsoleInput(hStdin, &input, 1, &events_read);

			if( input.EventType == KEY_EVENT && 
				input.Event.KeyEvent.bKeyDown &&
				input.Event.KeyEvent.uChar.AsciiChar
				)
			{
				/* ascii char found. */
				pending_ch = input.Event.KeyEvent.uChar.AsciiChar;
				pending_ch_count = input.Event.KeyEvent.wRepeatCount;
				break;
			}
		}
	}
}

int keyb_getkey()
{
	int ch = EOF; /* No key */

	if( pending_ch_count == 0)
	{
		get_pending_chars();
	}

	if( pending_ch_count != 0)
	{
		ch = pending_ch;
		pending_ch_count--;
	}

	return ch;
}

#else /* WIN32 */

void keyb_init()
// Initialise keyboard
{
/* linux */
  system("stty raw -echo brkint opost onlcr -ocrnl -onocr -onlret isig");
}

void keyb_done()
{
  system("stty -raw echo ");
}

int keyb_FD = 0; /* stdin */

int keyb_getkey()
{
  char s[2];
  
  int ch = EOF; /* No key */
  fd_set fds;
  struct timeval timeout;
  timeout.tv_sec=0;
  timeout.tv_usec=10;
  FD_ZERO(&fds);
  FD_SET(keyb_FD,&fds); // 1, 2
  if (select(keyb_FD+1,&fds,0,0,&timeout)==1)
  {
    /*printf("select!\n");*/
#if 1    
    if (read(keyb_FD,s,1)==1)
    {
      /*printf("read ok\n");*/
      ch = s[0];
    }
#else    
    ch=getchar();
#endif    
  }
  
  return ch;
}

#endif /* else WIN32 */

void poll_keyboard(void)
{
  int ch = EOF;
  keyb_init();
  
  while (ch != QUIT_CHAR)
  {
    sleep(1);
    
    printf(".");
    ch = keyb_getkey();
    if (ch != EOF)
    {
      printf("'%c'", ch);
    }
    fflush(stdout);
  }
  
}


int client_connect(const char* host, u_short portnr)
  /* uses client_type for SOCK_STREAM or SOCK_DGRAM */
{
  int s = 0;
  struct sockaddr_in server;  
  struct hostent *hp; 
  printf("Connecting to %s:%u\n", host, portnr);
  
  hp = gethostbyname(host);
  if (hp != NULL)
  {
    bzero((char *) &server, sizeof server);
    bcopy(hp->h_addr, (char *) &server.sin_addr, hp->h_length);
    server.sin_family = hp->h_addrtype;    /* Protocol family */
    server.sin_port = htons(portnr);/* Port number */
    s = socket(AF_INET, client_type, 0);
    if (s < 0)
    {
      printf("error in socket\n");
    }    
    if (connect(s, (struct sockaddr *) & server, sizeof server) < 0)
    {
      printf("connect() error in bind\n");
      s = 0;
    }
  }
  return s;
}

int server_listen_tcp_udp(u_short listener_portnr, unsigned long theType)
  /* theType could be SOCK_STREAM or SOCK_DGRAM */
{
  int s = 0;
  int len;
  
  struct sockaddr_in sa_in;
  struct sockaddr_in sa_server;

  bzero((char *) &sa_in, sizeof sa_in);
  s = socket(AF_INET, theType, 0);  
  sa_in.sin_port=htons(listener_portnr);

  sa_in.sin_family = AF_INET;

  /* Bind to any port and IP address */  
  if (bind(s, (struct sockaddr *) & sa_in, sizeof sa_in) < 0)
  {
    printf("error %d in bind (%s)\n", errno, strerror(errno));
  }
  
  /* Locate the port number and report to user */
  len = sizeof sa_server;
  if (getsockname(s, (struct sockaddr *) & sa_server, &len) < 0)
  {
    printf("error %d in getsockname\n", errno);
  }
  
  /* Accept any client */

  listen(s, 1);  
  return s;
}

int server_listen(u_short listener_portnr)
{
  return server_listen_tcp_udp(listener_portnr, SOCK_STREAM);
}

int server_listen_udp(u_short listener_portnr)
{
  return server_listen_tcp_udp(listener_portnr, SOCK_DGRAM);
}

int non_block_read(int g, char* buf, int buf_size)
{
  long int ret = 0;
  fd_set fds;
  struct timeval timeout;
  timeout.tv_sec=0;
  timeout.tv_usec=10;
  FD_ZERO(&fds);
  FD_SET(g,&fds);
  ret = 0;
  if (select(g+1,&fds,0,0,&timeout)==1)
  {
    /*printf("select!\n");*/
#if 0    
    ret=read(g, buf,buf_size);
#else    
    ret=recvfrom(g, buf,buf_size, 0, NULL,0);
#endif    
    if (!ret)
    {
      /* close */
      /* printf("zero length msg\n"); */
/*      go = 0;
      server_con = 0;*/
      ret = -1; /* force error */
      
      /*break;*/                 /* Zero-length message */
    }
  }
  return ret;
}


/* #define BUF_SIZE 1024*/

int main(int argc, char **argv)
{
  int arg_ofs=1;
  int option = 1;
  if (argc == 1 || (argc == 2 && strcmp(argv[0], "-h")))
  {
    /* usage */
    printf("tcptest [-options] [-l listenerport] [[host] [portnr]]\n");
    printf(
      "  -h               - This help:\n"
      "  -uc              - Client UDP\n"
      "  -us              - Server UDP\n"
      "  -ub              - Both UDP\n"
      "  -bBUF_SIZE       - Size of chunks\n"
      "  -cCLOSE_INTERVAL - Close if idle\n"
      "  -etx             - Echo TX test data\n"
      "  -erx             - Echo received data\n"
      "  -rhost:portnr    - Relay data to host:portnr\n"
      "  -iINPUTFILE      - Send the file to host\n"
      "  -oOUTPUTFILE     - Write received data to this file (default /dev/null )\n"
      "  -wWAITTIME       - Tme to wait in seconds before closing (default 0)\n"
      "  -L MS            - Looptest, used together with -t and -b to expect\n"
      "                     and check rx data\n"
      "  -tSIZE           - Connect and send test pattern with SIZE bytes\n"
      "  -TSIZE           - When someone connects, send test pattern with SIZE bytes\n"
      "  -l listenerport  - Port to listen to (default 2001)\n"
      "  -telnet          - Send telnet test\n"
      "  -telnet2         - Send telnet test2\n"
      "  host             - Host to connect to\n"
      "  portnr           - Port nr to connect to (default 2001)\n"
      );
    
    exit(1);
    
  }
  


  while (option && argc>arg_ofs)
  {
    if (argv[arg_ofs][0] == '-')
    {
      /* an option */
      switch (argv[arg_ofs][1])
      {
       case 'l':
        arg_ofs++;
        
         listener_portnr = atol(argv[arg_ofs]);
         printf("Listener port: %i\n", listener_portnr);
         break;       
       case 'b':
         buf_size = atol(&argv[arg_ofs][2]);
         if (buf_size<1)
         {
           buf_size = 1;
         }
         printf("Using buffersize: %lu\n", buf_size);
         break;          
       case 'c':
         close_interval = atol(&argv[arg_ofs][2]);
         printf("close_interval: %lu\n", close_interval);
         
         break;
       case 'u':
        if (argv[arg_ofs][2] == 's')
        {
          printf("Server is UDP\n");
          listener_type = SOCK_DGRAM;
        }
        else if (argv[arg_ofs][2] == 'c')
        {
          printf("Client using UDP\n");
          client_type = SOCK_DGRAM;
        }
        else if (argv[arg_ofs][2] == 'b')
        {
          printf("Both Client and Server is UDP\n");
          listener_type = SOCK_DGRAM;
          client_type = SOCK_DGRAM;          
        }
          
      
        break;
        
       case 'w':
         glob_wait = atol(&argv[arg_ofs][2]);
         printf("wait: %lu\n", glob_wait);
         
         break;         
       case 'L':
        arg_ofs++;
        
         loopback_time_ms = atol(argv[arg_ofs]);
         printf("Loopback time (ms): %li\n", loopback_time_ms);
         break;      
       case 'i':
         inputfilename = &argv[arg_ofs][2];
         printf("Using inputfile: %s\n", inputfilename);
         break;
       case 'o':
         outputfilename = &argv[arg_ofs][2];
         printf("Using outputfile: %s\n", outputfilename);
         break;
       case 'e':
        if (argv[arg_ofs][2] == 't') /* etx */
        {
          echo_testpattern = 1;
        }
        else if (argv[arg_ofs][2] == 'r') /* erx */
        {
          echo_rxdata = 1;
        }
        break;
       case 't':
        if (strcmp(argv[arg_ofs], "-telnet") == 0)
        {
          printf("Telnet test\n");
          telnet_test = 1;
        }
        else if (strcmp(argv[arg_ofs], "-telnet2") == 0)
        {
          printf("Telnet test2\n");
          telnet_test2 = 1;
        }
        else
        {
          ctestsize = atol(&argv[arg_ofs][2]);
          printf("Using ctestsize: %lu\n", ctestsize);
        }
        
         break;
       case 'T':
         stestsize = atol(&argv[arg_ofs][2]);
         printf("Using stestsize: %lu\n", stestsize);
         
         break;
       case 'r':
       {
         if (strlen(argv[arg_ofs])>2)
         {
           char* s;           
           strcpy(relay_hostname, &argv[arg_ofs][2]);
           s=strchr(relay_hostname, ':');
           if (s!=NULL)
           {
             s[0]='\0'; /* terminate string*/
             relay_portnr = atol(&s[1]);
           }
         }
         
         printf("Using relay: %s:%u\n", relay_hostname, relay_portnr);
         do_relay = TRUE;
         break;                  
       }
       
       default:
         printf("unknown option: %s\n", argv[arg_ofs]);
         break;
      }
      arg_ofs++;      
    }
    else
    {
      option = 0;
    }

  }

  keyb_init();    
#ifdef WIN32
  socket_init();
#endif
#if 0
  /* This used to be without -l */
  if (argc>arg_ofs)
  {
    listener_portnr=atol(argv[arg_ofs]);
    arg_ofs++;
  }
#endif
  if (argc>arg_ofs)
  {
    hostname = argv[arg_ofs];
    arg_ofs++;
  }
  if (argc>arg_ofs)
  {
    portnr = atol(argv[arg_ofs]);
    arg_ofs++;
  }
  if (do_relay)
  {
    relay_app();
  }
  else if (inputfilename)
  {
    read_inputfile_app();
  }
  else if (ctestsize>0)
  {
    testpattern_app();
  }
  else if (telnet_test)
  {
    telnet_test_app();
  }
  else if (telnet_test2)
  {
    telnet_test_app2();
  }
  else
  {
    printf("first round...\n");
    while(client_server_app())
    {
      printf("next round...\n");
      
    }
  }
  
  keyb_done();
#ifdef WIN32
  socket_done();
#endif

  if (glob_wait > 0)
  {
    printf("Wait %lu before closing!\n", glob_wait);
    sleep(glob_wait);    
  }
  
  return error_cnt;
  
}/* main */


int poll_listener(int f)
{
  int g = 0;
  int len;
  
  fd_set fds;
  struct timeval timeout;
  struct sockaddr_in from;
  
  timeout.tv_sec=0;
  timeout.tv_usec=10;

  bzero((char *) &from, sizeof from);  
  FD_ZERO(&fds);
  FD_SET(f,&fds);
  if (select(f+1,&fds,0,0,&timeout)==1)
  {
    if (listener_type == SOCK_STREAM)
    {
      len = sizeof from;          
      g = accept(f, (struct sockaddr *) & from, &len);
      if (g < 0)
      {
        printf("error in accept\n");
      }
      printf("Server connect!\n");
    }
    else
    {
      return f;
    }
  }
  
  return g;
}
        
#define RELAY_MAX 100
int relay_app(void)
{
  int l = 0;
  int server = 0;
  int client = 0;
  char* buf;
  int ret;
  int server_list[ RELAY_MAX];
  int client_list[ RELAY_MAX];
  int cnt = 0;
  int i;
  
  buf = malloc(buf_size);    

  l=server_listen_tcp_udp(listener_portnr, listener_type);
  printf("Listening on port #%d, connecting to %s:%u\n",
         listener_portnr, relay_hostname, relay_portnr);
  while (l>0)
  {
    server = poll_listener(l);

    if (server)
    {
      printf("Server connected!\n");
      client = client_connect(relay_hostname, relay_portnr);
      if (client)
      {
        printf("Client connected!\n");
        client_list[cnt]=client;
        server_list[cnt]=server;
        cnt++;
      }
      else
      {
        printf("Failed to connect to %s:%u\n", relay_hostname, relay_portnr);
        close(server);
      }
    }
    /* read and write data */
    for (i=0; i<cnt; i++)
    {
      client = client_list[i];
      server = server_list[i];
      if (client && server)
      {
        ret = non_block_read(client, buf, buf_size);
        if (ret>0)
        {
          ret = write(server, buf, ret);
          if (ret<0)
          {
            close(server);
            server = 0;          
          }
        }
        else
        {
          if (ret<0)
          {
            close(client);
            client = 0;
          }
        }
        
        ret = non_block_read(server, buf, buf_size);
        if (ret>0)
        {
          ret = write(client, buf, ret);
          if (ret<0)
          {
            close(client);
            server = 0;          
          }
        }
        else
        {
          if (ret<0)
          {
            close(server);
            server = 0;
          }
        }
      }
      if (client == 0)
      {
        close(server);
        cnt--;
        client_list[i] = client_list[cnt];
        server_list[i] = server_list[cnt];
      }
      else if (server == 0)
      {
        close(client);
        cnt--;
        client_list[i] = client_list[cnt];
        server_list[i] = server_list[cnt];        
      }
    } /* for */
  } /* while */

  for (i=0; i< cnt; i++)
  {
    close(client_list[i]);
    close(server_list[i]);
  }
  close(l);
  printf("End relay\n");
  return 0;
  
}

  
int read_inputfile_app(void)
{
  int g = 0;
  char *buf;
  char *sbuf;
  unsigned long cnti = 0;
  unsigned long cnto_tot = 0;
  unsigned long cnto = 0;
  unsigned long bytes_sent = 0;
  time_t t1, t2;

  
  FILE *fin;
  FILE *fout;

  buf = malloc(buf_size);
  sbuf = malloc(buf_size);
  /* Read buffer from file */
  if ((fin = fopen(inputfilename, "r")) != NULL)
  {
    if ((fout = fopen(outputfilename, "w")) != NULL)
    {
      
      g = client_connect(hostname, portnr);
      if (g)
      {
        printf("Client connect!\n");
        t1 = time(NULL);
        
        cnti = fread(buf, 1, buf_size, fin);
        while (cnti>0)
        {
          cnto_tot=0;
          do
          {
            cnto = write(g, &buf[cnto_tot], cnti-cnto_tot);
            bytes_sent += cnto;
            cnto_tot += cnto;
          }
          while(cnto_tot<cnto);


          
          {
            int ret = non_block_read(g, sbuf, buf_size);
            if (ret>0)
            {
              fwrite(sbuf, 1, ret, fout);
            }
          }
          
          
          cnti = fread(buf, 1, buf_size, fin);        
        }
        t2 = time(NULL);

        {
          unsigned long t_tot = t2 - t1+1;
          
          printf("Wrote %lu bytes in %lu seconds: mean %lu bytes/sec\n",
                 bytes_sent, t_tot, bytes_sent/t_tot);
        }
        close(g);              
      } /* g */
      fclose(fout);
      
    } /* fout */
    fclose(fin);    
  } /* fin */
  else
  {
    printf("Failed to open %s\n", inputfilename);
  }
  free(sbuf);
  free(buf);
  return 0;
  
} /* read_inputfile_app */
#define TEST (0)

void send_testpattern(int socket_handle, long int testsize)
{
  char *buf;
  char *rx_buf;
  unsigned long cnti = 0;
  unsigned long cnto_tot = 0;
  unsigned long cnto = 0;
  unsigned long bytes_sent = 0;
  unsigned long bytes_received = 0;
  unsigned long rxi = 0;
  unsigned long rx_tot = 0;
  time_t t1, t2;
  unsigned long row_cnt = 0;
  buf = malloc(buf_size+100);
  rx_buf = malloc(buf_size+100);
  t1 = time(NULL);
  while (bytes_sent < testsize)
  {
      
    cnti = 0;
    do{
      sprintf(&buf[cnti],
              "-%8lu 1234567890 abcdefghij klmnopqrstuvwxyz\r\n",
              bytes_sent+cnti);
      
      
      buf[cnti+row_cnt] = '\\';
      if (row_cnt>0)
      {
        buf[cnti+row_cnt-1] = ' ';            
      }
      if (row_cnt<47)
      {
        buf[cnti+row_cnt+1] = ' ';            
      }          
      
      switch (row_cnt%6)
      {
       case 0:
       case 1:
       case 2:             
        buf[cnti+row_cnt%3] = '\\';
        break;
       case 3:
        buf[cnti+2] = '/';
        break;                          
       case 4:
        buf[cnti+1] = '/';
        break;                          
       case 5:             
        buf[cnti+0] = '/';
        break;
       default:
        break;
      }
      cnti += strlen(&buf[cnti]);          
      row_cnt =(row_cnt+1)% (48);
    }
    while(cnti+bytes_sent<testsize && (cnti<buf_size));
    
    cnto_tot=0;
#if TEST        
    printf("Buffer:'\n %s'\n",   &buf[cnto_tot]);
#endif  
    
    rx_tot = 0;
    do{
#if TEST
      printf("chunk:%lu bytes sent in %lu byte chunks\n",
             cnti, MIN(buf_size, cnti-cnto_tot));
      
      printf(buf);
      cnto = cnti;
#else
/*          printf("chunk:%lu bytes sent in %lu byte chunks\n",
            cnti, MIN(buf_size, cnti-cnto_tot));
*/
      
      
      
      cnto = write(socket_handle, &buf[cnto_tot], 
                   MIN(buf_size, cnti-cnto_tot));
      if (echo_testpattern && cnto > 0)
      {
        printf("%.*s", (int)cnto, &buf[cnto_tot]);
      }
      if (cnto < 0)
      {
        /* Could be EAGAIN */
        cnto = 0;
      }
      
#endif          
      bytes_sent += cnto;
      cnto_tot += cnto;
#if 1
      rxi = non_block_read(socket_handle, &rx_buf[rx_tot], 
                           buf_size-rx_tot);
      if (rxi < 0)
      {
        /* Could be EAGAIN */
        rxi = 0;
      }
      rx_tot += rxi;
#endif
    }while(cnto_tot<cnti);
    
    
    if (loopback_time_ms >= 0)
    {
      time_t rxt1, rxt2;
      unsigned long rxt_tot;
      int cmp_res = 0;
      struct timeval timeout;
      
      rxt1 = time(NULL);
      /* Loop until everything sent has received */
      do 
      {
        timeout.tv_sec = loopback_time_ms / 1000;
        timeout.tv_usec = (loopback_time_ms % 1000) * 1000;
        rxi = non_block_read(socket_handle, &rx_buf[rx_tot], 
                             buf_size-rx_tot);
        select(0,NULL, NULL, NULL, &timeout);
        if (rxi < 0)
        {
          /* Could be EAGAIN */
          rxi = 0;
        }
        rx_tot += rxi;
        rxt2 = time(NULL);
        
        rxt_tot = rxt2 - rxt1;
      }while((rx_tot < cnti) && (rxt_tot < 10));
      
      /* printf("read: %lu tot: %lu\n", rxi, rx_tot); */
      cmp_res = memcmp(buf, rx_buf, rx_tot);
      
      
      if ((rx_tot == cnto_tot) && cmp_res == 0)
      {
        if (!echo_rxdata)
        {
          if (error_cnt > 0)
          {
            printf("%lu ok %lu/%lu %lu\n", 
                   rx_tot, bytes_sent, testsize, error_cnt);
          }
          else
          {
            printf("%lu ok %lu/%lu \n", 
                   rx_tot, bytes_sent, testsize);
            
          }
          
        }
        
      }
      else
      {
        if (rx_tot != cnto_tot)
        {
          error_cnt++;
          printf("Error #%lu RX count: %lu != %lu\n", error_cnt, 
                 rx_tot, cnto_tot);

        }
        if (cmp_res != 0)
        {
          error_cnt++;
          printf("Error #%lu RX data: Data differs at rx: %lu tx: %lu bytes_sent: %lu!\n", 
                 error_cnt, rx_tot, cnto_tot, bytes_sent);

        }
      }
    } /* if loopback */
    bytes_received += rx_tot;
    if (echo_rxdata)
    {
      printf("%.*s", (int)rx_tot, rx_buf);
    }
    
  } /* while */
  t2 = time(NULL);

  {
    unsigned long t_tot = t2 - t1+1;
    
    printf("Wrote %lu bytes in %lu seconds: mean %lu bytes/sec\n",
           bytes_sent, t_tot, bytes_sent/t_tot);
    if (error_cnt > 0)
    {
      printf("Number of errors: %lu\n", error_cnt);
    }
  }
  free(buf);
  printf("\n");
}

int testpattern_app(void)
{
  int g = 0;
  char *sbuf;
  
  FILE *fout;

  sbuf = malloc(buf_size);
  /* generate test pattern */
  if ((fout = fopen(outputfilename, "w")) != NULL)
  {
    struct linger ling;
    
    g = client_connect(hostname, portnr);
    ling.l_onoff = 1;
    ling.l_linger = 6000; /* In 1/100 secs */
    setsockopt(g, SOL_SOCKET, SO_LINGER, &ling, sizeof(ling));
        
    if (g)
    {
      printf("Client connect!\n");
      send_testpattern(g, ctestsize);
      
      
      {
        int ret = non_block_read(g, sbuf, buf_size);
        if (ret > 0)
        {
          fwrite(sbuf, 1, ret, fout);
        }
      }
      
      shutdown(g,1);
      
      close(g);      
    } /* g */
    fclose(fout);    
  } /* fout */
  else
  {
    printf("Failed to open outputfile: %s\n", outputfilename);
  }


  free(sbuf);

  return 0;
} /* testpattern_app */


int client_server_app(void)
{
  int f=0;
  int g = 0;                              /* Socket-handles */
  int len, i = 0, j, ret, go = 1;
  long ins = 0L;
  struct sockaddr_in from;

  char *buf;
  char *pbuf;
  char sbuf[10100];
  char *s;
  int ch;
  unsigned long cnt = 0;
  unsigned long bytes_sent = 0;
  int server_con = 0;
  int quit = 0;
  
  buf = malloc(buf_size);
  
  /* poll_keyboard();*/
  
  bzero((char *) &from, sizeof from);



  f=server_listen_tcp_udp(listener_portnr, listener_type);

  printf("Listening on port #%d, connecting to %s:%u\n",
         listener_portnr, hostname, portnr);
  
  /*
   * Allow five concurrent servers. Silly thing to do - this program
   * terminates after first session
   */
  

  do
  {
#if 1
    printf("Listening... Press a key to connect!\n");
    server_con = 0;
    while (!server_con && !quit)
    {
      cnt++;
      if (cnt % 100 == 0)
      {
        /*printf(".");*/
      }      
      ch = keyb_getkey();
      if (ch != EOF)
      {
        char b=ch;
        if (ch == QUIT_CHAR)
        {
          printf("quitting\n");          
          quit = 1;
          go =0;
          
          break;
        }
        else
        {
          /* connect */
          g = client_connect(hostname, portnr);
          printf("Client connect!\n");
          close_interval = 0;
          
          write(g, &b,1);
          server_con = 1;
        }
        
        /*printf("key %c pressed\n",ch);*/
        printf("%c",b); /* echo */
      }
      {
        fd_set fds;
        struct timeval timeout;
        timeout.tv_sec=0;
        timeout.tv_usec=10;
        FD_ZERO(&fds);
        FD_SET(f,&fds);
        ret = 0;
        if (select(f+1,&fds,0,0,&timeout)==1)
        {
          len = sizeof from;
          if (listener_type == SOCK_STREAM)
          {
            g = accept(f, (struct sockaddr *) & from, &len);
            if (g < 0)
            {
              printf("error in accept\n");
              exit(4);
            }
          }
          else
          {
            /* UDP */
            g = f;
          }
          
          server_con = 1;
          printf("Server connect!\n");
          if (stestsize){
            send_testpattern(g, stestsize);
          }
        }
      }
      fflush(stdout);      
    } /* while */
#else
    len = sizeof from;    
    g = accept(f, (struct sockaddr *) & from, &len);
    if (g < 0)
    {
      printf("error in accept\n");
      exit(4);
    }
    printf("Server connect!\n");
#endif
    
    /*fcntl(g, F_SETFL, O_NONBLOCK);*/
    go = 1;
    

    
    pbuf = buf;
    cnt = 0;
    printf("Reading data...\n");
    while (!quit && go && server_con)
    {
      cnt++;
      if (cnt % 100 == 0)
      {
        /*printf(".");*/
      }
      
      /*sleep(1);*/
      ch = keyb_getkey();
      if (ch != EOF)
      {
        char b=ch;
        if (ch == QUIT_CHAR)
        {
          printf("quitting\n");
          quit = 1;
          break;
          
        }
        
        /*printf("key %c pressed\n",ch);*/
        printf("%c",b); /* echo */

        if (b == '\r' || b == '\n')
        {
          ret=write(g, &b,1);
          /*ret=write(g, &b,0);          */
          
          /*fcntl(g, F_SETFL, O_NONBLOCK);          */
        }
        else
        {
          ret=write(g, &b,1);          
        }
        
        
        if (ret != 1)
        {
          printf("write error\n");
          go = 0;
        }
        bytes_sent++;
      }
      
/*      memset(sbuf, '\0', sizeof(sbuf));*/
#if 1
      ret = non_block_read(g, sbuf, buf_size);
      if (ret<0)
      {
        /* close */
        /* printf("zero length msg\n"); */
        go = 0;
        server_con = 0;
        
        /*break;*/                 /* Zero-length message */
      }
      i++;                           /* Update number of received
                                      * parts */          
      
#else          
      ret = recv(g, sbuf, buf_size, 0);   /* Read socket */
      if (!ret)
      {
        /* close */
        /* printf("zero length msg\n"); */
        
        break;                 /* Zero-length message */
      }
      i++;                           /* Update number of received
                                      * parts */      
#endif          

      
      if (ret < 0)                   /* Stop for error */
      {
        printf("error %d in read\n", errno);
        break;
      }
      
      /* Copy received data to buffer */
      for (j = 0; j < ret; j++)
      {
        ins++;
        
        if (ins < buf_size)
        {
          *pbuf++ = sbuf[j];
        }
        
/* by Johan , print to screen to */
        putchar(sbuf[j]);
        if (close_interval && (ins == close_interval))
        {
          printf("Forcing close\n");
          
          close(g);
          g = 0;
          go = 0;
          break;
          
          
        }
        
      }
      fflush(stdout);
      
      if (strncmp("POST",sbuf,4)==0)
      {
        /* its a POST */
        ret = recv(g, sbuf, sizeof(sbuf), 0);   /* Read socket */
        if (ret>0)
        {
          sbuf[ret]='\0';
          
          printf(sbuf);
        }
        
        
        s="HTTP/1.0 204 No content\r\n"
          "Connection: close\r\n"
          "Content-length: 0\r\n\r\n";
        
        send(g,s,strlen(s),0);
        break;
        
      }else
      {
        if (strncmp("GET ",sbuf,4)==0)       
        {
          if (sbuf[5]=='s')
          {
            s="HTTP/1.0 200 OK\r\n"
              "Connection: close\r\n"
              "Content-length: 10\r\n\r\n1234567890";      
          }
          
          else
          {
            s="HTTP/1.0 204 No content\r\n"
              "Connection: close\r\n"
              "Content-Type: html/text\r\n"                      
              "Content-Length: 0\r\n\r\n";
          }
          
          send(g,s,strlen(s),0);
          break;
        }
        
      }
      
      
      if (i % 100 == 0)
      {
        /*printf(".");*/
      }
      
    } /* while */
    
    printf("\nRead %d blocks total %ld bytes\n"
           "Sent %ld bytes\n"
           , i, ins, bytes_sent);    
#if 1
    {
      FILE *fp;      
      /* Save buffer in file */
      if ((fp = fopen("output", "w")) != NULL)
      {
        pbuf = buf;
        while (ins)
        {
          fputc(*pbuf++, fp);            /* Flush buffer to file */
          ins--;
        } 
        
        fclose(fp);
      }
      
    }
#endif    
    printf("Terminating\n");
    
    /* Close sockets */
    if (g>0)
    {
      shutdown(g,0);
      close(g);
    }
    
  }while(!quit);
/*  
  if (f>0)
  {
    shutdown(f,0);
    close(f);
  }
  */
  close(f);
  
  free(buf);

  return !quit;
}

void fwrite_str(const char *s, FILE *f)
{
  fwrite(s, strlen(s), 1, f);
}

#define TELOPT_BINARY         ((unsigned char)0)
#define TELOPT_ECHO           ((unsigned char)1)
#define TELOPT_SGA            ((unsigned char)3)
#define TELOPT_TM             ((unsigned char)6)
#if 1 /* TN3270E_INCLUDED || TN5250E_INCLUDED */
#define TELOPT_TERMINAL_TYPE  ((unsigned char)24)
#define TELOPT_EOR            ((unsigned char)25)
#define TELOPT_NEW_ENVIRON    ((unsigned char)39)
#define TELOPT_TN3270E        ((unsigned char)40)
#endif

/* COM-PORT-OPTION according to RFC2217 */
#define TELOPT_COM_PORT_OPTION  ((unsigned char)44)

/* AXCOMP protocol option (we made this up) */
#define TELOPT_AXCOM_PORT_OPTION  ((unsigned char)64) 

  
 
int telnet_test_app(void)
{
  int g = 0;
  char *sbuf;
  unsigned char buf[1000];
  int num_read;
  
  FILE *fout;
  unsigned char iac[2]={0xFF, 0x00 };
  unsigned char iaciac[2]={0xFF, 0xFF };
  unsigned char iac_will[3]={0xFF, 0xFB,0 };
  unsigned char iac_do[3]={0xFF, 0xFD, 0 };
  
  sbuf = malloc(buf_size);
  /* generate test pattern */
  if ((fout = fopen(outputfilename, "w")) != NULL)
  {
    g = client_connect(hostname, portnr);
    if (g)
    {
      fwrite_str("Options read:", fout);
      /* First read options */
      printf("read...\n");
      num_read = read(g, buf, 1000);
      fwrite(buf, num_read, 1, fout);
      printf("Write BINARY\n");
      /* Write telnet negotiations: */
      write(g, "BINARY",6);
      /* 2 options in one packet */
      iac_will[2] = TELOPT_BINARY;
      iac_do[2] = TELOPT_BINARY;
      memcpy(buf, iac_will, 3);
      memcpy(&buf[3], iac_do, 3);
      write(g, buf,6);
      write(g, iac_do,3);
      sleep(2);
      printf("Write SGA\n");
      write(g, "SGA",3);
      iac_will[2] = TELOPT_SGA;
      iac_do[2] = TELOPT_SGA;
      /* One option in one packet */
      write(g, iac_will,3);
      sleep(2);
      /* One option over many packets:1+1+1 */
      write(g, &iac_do[0],1);
      sleep(1);
      write(g, &iac_do[1],1);
      sleep(1);
      write(g, &iac_do[2],1);
      sleep(2);
      printf("Write RFC2217\n");
      iac_will[2] = TELOPT_COM_PORT_OPTION;
      iac_do[2] = TELOPT_COM_PORT_OPTION;
      write(g, "RFC2217",7);
      /* One option over many packets,2+1 and 1+2 */
      write(g, &iac_do[0],2);
      sleep(1);
      write(g, &iac_do[2],1);
      sleep(2);
      write(g, &iac_will[0],1);
      sleep(1);
      write(g, &iac_will[1],2);
      sleep(2);
      printf("Write IACIAC\n");
      write(g, "IACIAC",6);
      write(g, &iaciac[0],2);
      sleep(2);
      printf("Write IAC  IAC\n");
      write(g, "IAC  IAC",8);
      write(g, &iac[0],1);
      sleep(2);
      write(g, &iac[0],1);
      sleep(2);
      printf("TEXT\n");
      write(g, "TEXT",4);
      sleep(2);
      write(g, "IAC:\xFF",5);
      sleep(2);
      write(g, "\xFF""iac:\xFF",6);
      sleep(2);
      write(g, "\xFF""END",4);
      sleep(1);
      printf("read..\n");
      num_read = read(g, buf, 1000);
      fwrite(buf, num_read, 1, fout);
      close(g);
    }
    fclose(fout);
  }
  return 0;
}

const char telnet_tx1[21] = 
{ 0xFF, 0xFD, 0x03, 0xFF, 0xFB, 0x03, 0xFF, 0xFD, 0x01, 0xFF,
  0xFD, 0x00, 0xFF, 0xFB, 0x00, 0xFF, 0xFD, 0x2C, 0xFF, 0xFB, 0x2C
};
/* 3 in response */
/* then 25 in response */
const char telnet_tx2[59] = 
{ 0xFF, 0xFA, 0x2C, 0x0A, 0x1E, 0xFF, 0xF0, 0xFF, 0xFA, 0x2C,
  0x01, 0x00, 0x01, 0xC2, 0x00, 0xFF, 0xF0, 0xFF, 
  0xFA, 0x2C, 0x02, 0x08, 0xFF, 0xF0, 0xFF, 0xFA,
  0x2C, 0x03, 0x01, 0xFF, 0xF0, 0xFF, 0xFA, 0x2C, 
  0x04, 0x01, 0xFF, 0xF0, 0xFF, 0xFA, 0x2C, 0x05,
  0x08, 0xFF, 0xF0, 0xFF, 0xFA, 0x2C, 0x05, 0x01, 
  0xFF, 0xF0, 0xFF, 0xFA, 0x2C, 0x05, 0x10, 0xFF,
  0xF0
};
/* 7 in response */
/* 52 in response - first byte wrong? */

int telnet_test_app2(void)
{
  int g = 0;
  char *sbuf;
  unsigned char buf[1000];
  int num_read;
  int i;
  
  FILE *fout;
  
  sbuf = malloc(buf_size);
  /* generate test pattern */
  if ((fout = fopen(outputfilename, "w")) != NULL)
  {
    g = client_connect(hostname, portnr);
    if (g)
    {
      fwrite_str("Options read:", fout);
      /* First read options */
      printf("read...\n");
      fwrite("read1:          ", 16, 1, fout);
      num_read = read(g, buf, 1000);
      fwrite(buf, num_read, 1, fout);
      printf("Write tx1\n");
      fwrite("write1:         ", 16, 1, fout);
      write(g, telnet_tx1, sizeof(telnet_tx1));
      fwrite(telnet_tx1, sizeof(telnet_tx1), 1, fout);
      sleep(1);
      fwrite("read2:          ", 16, 1, fout);
      num_read = read(g, buf, 1000);
      fwrite(buf, num_read, 1, fout);
      printf("Write tx2\n");
      fwrite("write2:         ", 16, 1, fout);
      write(g, telnet_tx2, sizeof(telnet_tx2));
      fwrite(telnet_tx2, sizeof(telnet_tx2), 1, fout);
      i = 10000;
      printf("Sleeping...\n");      
      while (--i)
      {
        sleep(1);
        num_read = read(g, buf, 1000);
        fwrite("read:           ", 16, 1, fout);
        fwrite(buf, num_read, 1, fout);
        fflush(fout);
        if (keyb_getkey() == 'q'){
          i=1;
        }
      }
      close(g);
    }
    fclose(fout);
  }
  return 0;
}
